/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "CentralWidget/centraltoolbar.h"
#include "Widget/spacer.h"

#include <QtGui>

namespace GLOBAL {
    QMap<QString, QAction*>* globalActions();
}

CentralToolBar* CentralToolBar::INSTANCE = 0;
CentralToolBar* _CentralToolBar() {return CentralToolBar::instance();}

/*******************************************************************************
    CentralToolBar
*******************************************************************************/
CentralToolBar::CentralToolBar(QWidget *parent) : QToolBar( "CentralToolBar", parent )
{
    INSTANCE   = this;
    _menuModel = MenuModel::instance();
  
    this->setObjectName(QString::fromUtf8("CentralToolBar"));
    this->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum );
    this->setMaximumHeight(28);
    this->setFloatable(false);
    this->setMovable(false);
    this->setContentsMargins( 0, 0, 0, 0);
    this->setToolButtonStyle( Qt::ToolButtonIconOnly );
    this->setIconSize( QSize( 20, 20) );
  
    _currentBrowserIdx = 0;
  
    QMap<QString, QAction*> *actions = GLOBAL::globalActions();
  
    //! NavigatorBar
    _navigatorBar = new NavigatorBar();
    _navigatorBar->setModel(_menuModel);

    //! Prev/Next
    actions->insert("goPrev", new QAction(QIcon::fromTheme("go-previous", QIcon(":/images/go-previous.png")),tr("Go back"),this));     
    actions->insert("goNext", new QAction(QIcon::fromTheme("go-next", QIcon(":/images/go-next.png")),tr("Go forward"),this));     
        
    //! Search field widget
    _searchField = new SearchField(this);
    _searchField->setInactiveText(tr("Search in Library"));

    //! Tools Button and Menu
    QToolButton* toolButton = new QToolButton(this);
    toolButton->setAutoRaise(true);
    toolButton->setIcon(QIcon(":/images/folder-music-48x48.png"));
    toolButton->setToolTip(tr("Tools"));
    toolButton->setPopupMode(QToolButton::InstantPopup);
 
    QMenu *menu = new QMenu(this);
      QMenu *m0 = menu->addMenu(tr("&Show/Hide panel"));
      m0->addAction(actions->value("showMenuBar"));
      m0->addAction(actions->value("showPlaylist"));
      m0->addAction(actions->value("showMenu"));
    menu->addSeparator();
    QMenu *m1 = menu->addMenu(tr("&Add to playqueue"));
      m1->addAction(actions->value("addfile"));
      m1->addAction(actions->value("adddir"));
      m1->addAction(actions->value("addurl")); 
      menu->addAction(actions->value("clear"));
      menu->addAction(actions->value("save"));
      menu->addAction(actions->value("autosave"));
      QMenu *m2 = menu->addMenu(tr("&Choose playlist mode"));
      m2->addAction(actions->value("playlistModeTitle"));
      m2->addAction(actions->value("playlistModeAlbum"));
      m2->addAction(actions->value("playlistModeArtist"));
    menu->addSeparator();
      menu->addAction(actions->value("forceRebuild"));
      menu->addAction(actions->value("coverThread"));
      menu->addAction(actions->value("collectionSettings"));
    menu->addSeparator();
      menu->addAction(actions->value("radioSettings"));
      menu->addAction(actions->value("allSettings"));
      menu->addAction(actions->value("audioEq"));
    menu->addSeparator();
      menu->addAction(actions->value("helpYaRock"));
      menu->addAction(actions->value("helpQt"));
    menu->addSeparator();
      menu->addAction(actions->value("quit"));
  
    toolButton->setMenu(menu);

    //!Tool Bar Settup
    QFrame* separator1 = new QFrame( this );
    separator1->setFrameShape( QFrame::VLine );
    QFrame* separator2 = new QFrame( this );
    separator2->setFrameShape( QFrame::VLine );
      
    QWidget *expandSpacerWidget = new QWidget(this);
    expandSpacerWidget->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
    
    this->addWidget( new FixedSpacer( this, QSize(5, 0)) );
    this->addWidget( toolButton );
    this->addWidget( separator2 ) ;
    this->addAction( actions->value("goPrev") );
    this->addAction( actions->value("goNext") );
    this->addWidget( separator1 );
    this->addWidget( _navigatorBar );    
    this->addWidget( expandSpacerWidget);
    this->addWidget( _searchField );
    this->addWidget( new FixedSpacer( this, QSize(5, 0)) );
    
    //! Signals
    connect(actions->value("goPrev"), SIGNAL(triggered()), this, SLOT(activePrev()));
    connect(actions->value("goNext"), SIGNAL(triggered()), this, SLOT(activeNext()));
    connect(_searchField, SIGNAL(signalSearchTriggered()), this, SLOT(sendSearch()));
}

//! ------------ sendSearch ----------------------------------------------------
void CentralToolBar::sendSearch() 
{
    //! enregistrement dans l'historique si nouvelle recherche
    updateCurrentIndex(_browserList.at(_currentBrowserIdx));
    emit searchActivated(_searchField->text());
}

//! ------------ update Mode ---------------------------------------------------
void CentralToolBar::updateMode(MAIN::ViewMode mode)
{
    _searchField->setEnable(mode != MAIN::ViewFiler && mode != MAIN::ViewContext);  
}



//! ------------ updateCurrentIndex --------------------------------------------
void CentralToolBar::updateCurrentIndex(const QModelIndex &idx)
{
    //qDebug() << "-- CentralToolBar -> updateCurrentIndex  #1"; 
    if(!idx.isValid())  return;

    //! nouveau dclenchement --> on supprime les next
    for(int i = 0; i < _currentBrowserIdx; i++) {
      _browserList.removeAt(i);
      _searchList.removeAt(i);
    }
    
    _browserList.prepend(idx);
    _searchList.prepend(_searchField->text());
    _currentBrowserIdx = 0;

    GLOBAL::globalActions()->value("goPrev")->setEnabled(_currentBrowserIdx < _browserList.size() -1);
    GLOBAL::globalActions()->value("goNext")->setEnabled(_currentBrowserIdx > 0);
  
    //! limite de la taille de l'historique de navigation
    if(_browserList.size() >= 20 && _currentBrowserIdx != _browserList.size() -1)
      _browserList.takeLast();
}

void CentralToolBar::activePrev()
{
    if(_currentBrowserIdx < _browserList.size() -1) 
    {
      _currentBrowserIdx++;
      const QModelIndex idx = _browserList.at(_currentBrowserIdx);

      //!emit prevNextActivated(idx);
      emit prevNextActivated(idx, _searchList.at(_currentBrowserIdx));
      _searchField->setText(_searchList.at(_currentBrowserIdx));
      _searchField->clearFocus();

      GLOBAL::globalActions()->value("goPrev")->setEnabled(_currentBrowserIdx < _browserList.size() -1);
      GLOBAL::globalActions()->value("goNext")->setEnabled(_currentBrowserIdx > 0);
    }
}

void CentralToolBar::activeNext()
{
    if(_currentBrowserIdx > 0) 
    {
      _currentBrowserIdx--;
      const QModelIndex idx = _browserList.at(_currentBrowserIdx);

      //!emit prevNextActivated(idx);
      emit prevNextActivated(idx, _searchList.at(_currentBrowserIdx));
      _searchField->setText(_searchList.at(_currentBrowserIdx));
      _searchField->clearFocus();
      
      GLOBAL::globalActions()->value("goPrev")->setEnabled(_currentBrowserIdx < _browserList.size() -1);
      GLOBAL::globalActions()->value("goNext")->setEnabled(_currentBrowserIdx > 0);
    }
}
