/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "CentralWidget/menumodel.h"
#include "CentralWidget/menuitem.h"
#include "viewmode.h"
#include "utilities.h"      // CONFIGDIR

#include <QDebug>
#include <QDir>
#include <QMap>
#include <QString>
#include <QStringList>
#include <QAction>


namespace GLOBAL {
    QMap<QString, QAction*>* globalActions();
}

MenuModel* MenuModel::INSTANCE = 0;
MenuModel* _MenuModel() {return MenuModel::instance();}
/*******************************************************************************
    MenuModel
*******************************************************************************/
MenuModel::MenuModel(QObject *parent) : QStandardItemModel(parent)
{
    INSTANCE = this;
    configureGlobalAction();
    populateMenu(); 
    configureMenuAction(); 
}


MenuModel::~MenuModel()
{
    delete _radioItem;
    delete _computerItem;
}

//! ----------- configureGlobalAction ------------------------------------------
void MenuModel::configureGlobalAction()
{
    QMap<QString, QAction*> *actions = GLOBAL::globalActions(); 

    actions->insert("dashboardview", new QAction(this));  
    actions->insert("contextview", new QAction(this));  
    actions->insert("historyview", new QAction(this));  
    actions->insert("filerview", new QAction(this));      
    actions->insert("albumview",new QAction(this));      
    actions->insert("artistview",  new QAction(this));       
    actions->insert("trackview", new QAction(this));       
    actions->insert("genreview", new QAction(this));      
    actions->insert("yearview", new QAction(this));      
    actions->insert("favoriteview", new QAction(this));       
    actions->insert("playlistview", new QAction(this));            
    actions->insert("playlistdetailview", new QAction(this));       
    actions->insert("icecastview", new QAction(this));      
    actions->insert("shoutcastview", new QAction(this));      
    actions->insert("customradioview", new QAction(this)); 
    actions->insert("favoriteradioview", new QAction(this));      

    //! pour chaque actions on utilise le ViewMode (cf. viewmode.h)
    actions->value("dashboardview")->setData(QVariant(int(MAIN::ViewDashBoard)));
    actions->value("contextview")->setData(QVariant(int(MAIN::ViewContext)));
    actions->value("historyview")->setData(QVariant(int(MAIN::ViewHistory)));
    actions->value("filerview")->setData(QVariant(int(MAIN::ViewFiler)));
    actions->value("albumview")->setData(QVariant(int(MAIN::ViewAlbum)));
    actions->value("artistview")->setData(QVariant(int(MAIN::ViewArtist)));
    actions->value("trackview")->setData(QVariant(int(MAIN::ViewTrack)));
    actions->value("genreview")->setData(QVariant(int(MAIN::ViewGenre)));
    actions->value("yearview")->setData(QVariant(int(MAIN::ViewYear)));
    actions->value("favoriteview")->setData(QVariant(int(MAIN::ViewFavorite)));
    actions->value("playlistview")->setData(QVariant(int(MAIN::ViewPlaylist)));
    actions->value("playlistdetailview")->setData(QVariant(int(MAIN::ViewPlaylistDetail)));
    actions->value("icecastview")->setData(QVariant(int(MAIN::ViewIceCast)));
    actions->value("shoutcastview")->setData(QVariant(int(MAIN::ViewShoutCast)));
    actions->value("customradioview")->setData(QVariant(int(MAIN::ViewCustomRadio)));
    actions->value("favoriteradioview")->setData(QVariant(int(MAIN::ViewFavoriteRadio)));
}

//! ----------- model population -----------------------------------------------
void MenuModel::populateMenu()
{
   QMap<QString, QAction*> *actions = GLOBAL::globalActions();
    
   QStandardItem *rootItem = this->invisibleRootItem();

   //! populate Root Item
   MenuItem* item1 = new MenuItem(tr("Context view"), QIcon(":/images/info.png"));
   item1->setData(QVariant::fromValue(new QAction(QIcon(":/images/info.png"), tr("Context view"), this)), MenuActionRole);
   item1->setData(QVariant::fromValue(actions->value("contextview")), GlobalActionRole);
   rootItem->appendRow(item1);

   MenuItem* item5 = new MenuItem(tr("Dashboard View"), QIcon(":/images/chart-64x64.png"));
   item5->setData(QVariant::fromValue(new QAction(QIcon(":/images/chart-64x64.png"), tr("Dashboard view"), this)), MenuActionRole);
   item5->setData(QVariant::fromValue(actions->value("dashboardview")), GlobalActionRole);
   rootItem->appendRow(item5);

   MenuItem* item4 = new MenuItem(tr("History view"), QIcon(":/images/history-48x48.png"));
   item4->setData(QVariant::fromValue(new QAction(QIcon(":/images/history-48x48.png"), tr("History view"), this)), MenuActionRole);
   item4->setData(QVariant::fromValue(actions->value("historyview")), GlobalActionRole);
   rootItem->appendRow(item4);
   
   MenuItem* item2 = new MenuItem(tr("Music browser"), QIcon(":/images/folder-music-48x48.png"));
   item2->setData(QVariant::fromValue(new QAction(QIcon(":/images/folder-music-48x48.png"), tr("Music browser"), this)), MenuActionRole);
   rootItem->appendRow(item2);

   MenuItem* item3 = new MenuItem(tr("Playlist browser"), QIcon(":/images/folder-playlist-48x48.png"));
   item3->setData(QVariant::fromValue(new QAction(QIcon(":/images/folder-playlist-48x48.png"), tr("Playlist browser"), this)), MenuActionRole);
   rootItem->appendRow(item3);

   _radioItem = new MenuItem(tr("Radio browser"), QIcon(":/images/media-url-48x48.png"));
   _radioItem->setData(QVariant::fromValue(new QAction(QIcon(":/images/media-url-48x48.png"), tr("Radio browser"), this)), MenuActionRole);   
   rootItem->appendRow(_radioItem);
 
   _computerItem = new MenuItem(tr("Computer"), QIcon(":/images/computer-48x48.png"));
   _computerItem->setData(QVariant::fromValue(new QAction(QIcon(":/images/computer-48x48.png"), tr("Computer"), this)), MenuActionRole);
   rootItem->appendRow(_computerItem);

   /*
   MenuItem* item6 = new MenuItem(tr("Internet services"), QIcon(":/images/internet.png"), MenuFirstLevelType);
   homeItem->appendRow(item6);
   actions->value("m_internetBrowser")->setData(QVariant::fromValue(indexFromItem(item6)));
   item6->setData(QVariant::fromValue(actions->value("m_internetBrowser")), ActionRole);
   */  
   
   //! populate Collection Item
   MenuItem* item21 = new MenuItem(tr("album view"), QIcon(":/images/album.png"));
   item21->setData(QVariant::fromValue(new QAction(QIcon(":/images/album.png"), tr("album view"), this)), MenuActionRole);
   item21->setData(QVariant::fromValue(actions->value("albumview")), GlobalActionRole);
   item2->appendRow(item21);

   MenuItem* item22 = new MenuItem(tr("artist view"), QIcon(":/images/view-artist.png")); 
   item22->setData(QVariant::fromValue(new QAction(QIcon(":/images/view-artist.png"), tr("artist view"), this)), MenuActionRole);
   item22->setData(QVariant::fromValue(actions->value("artistview")), GlobalActionRole);
   item2->appendRow(item22);

   MenuItem* item23 = new MenuItem(tr("track view"), QIcon(":/images/track-48x48.png")); 
   item23->setData(QVariant::fromValue(new QAction(QIcon(":/images/track-48x48.png"), tr("track view"), this)), MenuActionRole);
   item23->setData(QVariant::fromValue(actions->value("trackview")), GlobalActionRole);
   item2->appendRow(item23);

   MenuItem* item24 = new MenuItem(tr("genre view"), QIcon(":/images/genre.png")); 
   item24->setData(QVariant::fromValue(new QAction(QIcon(":/images/genre.png"), tr("genre view"), this)), MenuActionRole);
   item24->setData(QVariant::fromValue(actions->value("genreview")), GlobalActionRole);
   item2->appendRow(item24);
   
   MenuItem* item25 = new MenuItem(tr("year view"), QIcon(":/images/date-48x48.png")); 
   item25->setData(QVariant::fromValue(new QAction(QIcon(":/images/date-48x48.png"), tr("year view"), this)), MenuActionRole);
   item25->setData(QVariant::fromValue(actions->value("yearview")), GlobalActionRole);
   item2->appendRow(item25);
   
   MenuItem* item26 = new MenuItem(tr("favorite view"), QIcon(":/images/favorites-48x48.png")); 
   item26->setData(QVariant::fromValue(new QAction(QIcon(":/images/favorites-48x48.png"), tr("favorite view"), this)), MenuActionRole);
   item26->setData(QVariant::fromValue(actions->value("favoriteview")), GlobalActionRole);
   item2->appendRow(item26);

   //! populate Paylist Browser Item
   MenuItem* item31 = new MenuItem(tr("playlist view"), QIcon(":/images/media-playlist-48x48.png")); 
   item31->setData(QVariant::fromValue(new QAction(QIcon(":/images/media-playlist-48x48.png"), tr("playlist view"), this)), MenuActionRole);
   item31->setData(QVariant::fromValue(actions->value("playlistview")), GlobalActionRole);
   item3->appendRow(item31);

   MenuItem* item32 = new MenuItem(tr("track view"), QIcon(":/images/track-48x48.png")); 
   item32->setData(QVariant::fromValue(new QAction(QIcon(":/images/track-48x48.png"), tr("track view"), this)), MenuActionRole);
   item32->setData(QVariant::fromValue(actions->value("playlistdetailview")), GlobalActionRole);
   item3->appendRow(item32);

   //! populate Radio Browser Item
   populateRadioItem();
   
  /*//! populate Internet services Item
   MenuItem* item61 = new MenuItem(tr("jamendo service"), QIcon(":/images/jamendo.png"), MenuSecondLevelType); 
   item6->appendRow(item61);
   actions->value("m_jamendoview")->setData(QVariant::fromValue(indexFromItem(item61)));
   item61->setData(QVariant::fromValue(actions->value("m_jamendoview")), ActionRole);
*/
  
   //! populate Computer Item
   populateComputerItem();
}

//! --------- populateComputerItem ---------------------------------------------
void MenuModel::populateComputerItem() 
{
   QMap<QString, QAction*> *actions = GLOBAL::globalActions();

   MenuItem* item51 = new MenuItem(tr("Home directory"), QIcon(":/images/folder.png"));
   item51->setData(QVariant::fromValue(new QAction(QIcon(":/images/folder.png"), tr("Home directory"), this)), MenuActionRole);
   item51->setData(QVariant::fromValue(actions->value("filerview")), GlobalActionRole);
   item51->setData(QVariant(QDir::homePath()), FilerPlaceRole);
   _computerItem->appendRow(item51);

   MenuItem* item52 = new MenuItem(tr("Root directory"), QIcon(":/images/folder.png"));
   item52->setData(QVariant::fromValue(new QAction(QIcon(":/images/folder.png"), tr("Root directory"), this)), MenuActionRole);
   item52->setData(QVariant::fromValue(actions->value("filerview")), GlobalActionRole);
   item52->setData(QVariant(QDir::rootPath()), FilerPlaceRole);
   _computerItem->appendRow(item52);

   //! read mtab file 
   QStringList mtabMounts;
   QFile mtab("/etc/mtab");
   mtab.open(QFile::ReadOnly);
   QTextStream stream(&mtab);
   do mtabMounts.append(stream.readLine());
   while (!stream.atEnd());
   mtab.close();

   foreach(QString item, mtabMounts)
   if( item[0] == '/'                     &&
       item.split(" ").at(2) != "iso9660" &&
       item.split(" ").at(2) != "fat") 
   {
     QString place = item.split(" ").at(1);
     
     MenuItem* placeItem = new MenuItem(place, QIcon::fromTheme("drive-harddisk"));
     placeItem->setData(QVariant::fromValue(new QAction(QIcon::fromTheme("drive-harddisk"), place, this)), MenuActionRole);
     placeItem->setData(QVariant::fromValue(actions->value("filerview")), GlobalActionRole);
     placeItem->setData(QVariant(place), FilerPlaceRole);
     _computerItem->appendRow(placeItem);
  }
}

//! --------- populateUserRadioFile --------------------------------------------
void MenuModel::populateRadioItem() 
{
    QMap<QString, QAction*> *actions = GLOBAL::globalActions();

    //! removeRows for radio Item
    _radioItem->removeRows(0, _radioItem->rowCount() ); 

    //! add IceCast
    MenuItem *mi1 = new MenuItem(tr("icecast stream"), QIcon(":/images/icecast.png"));
    mi1->setData(QVariant::fromValue(new QAction(QIcon(":/images/icecast.png"), tr("icecast stream"), this)), MenuActionRole);
    mi1->setData(QVariant::fromValue(actions->value("icecastview")), GlobalActionRole);
    _radioItem->appendRow(mi1);

    //! add ShoutCast
    MenuItem *mi2 = new MenuItem(tr("shoutcast stream"), QIcon(":/images/shoutcast.png"));
    mi2->setData(QVariant::fromValue(new QAction(QIcon(":/images/shoutcast.png"), tr("shoutcast stream"), this)), MenuActionRole);
    mi2->setData(QVariant::fromValue(actions->value("shoutcastview")), GlobalActionRole);
    _radioItem->appendRow(mi2);
  
    //! search for xspf custom radio file 
    const QStringList filters = QStringList() << "*.xspf";
    QDir sourceDir(UTIL::CONFIGDIR + "/radio");
    sourceDir.setNameFilters(filters);  

    QStringList entryList = sourceDir.entryList(filters, QDir::Files, QDir::Name);
    foreach (const QString &file, entryList) 
    {
      //qDebug() << "-- MenuModel --> populateUserRadioFile : custom radio file  " << file;

      MenuItem *mi = new MenuItem(QFileInfo(file).baseName(),QIcon(":/images/media-url-48x48.png"));
  
      mi->setData(QVariant::fromValue(new QAction(QIcon(":/images/media-url-48x48.png"), QFileInfo(file).baseName(), this)), MenuActionRole);
      mi->setData(QVariant::fromValue(actions->value("customradioview")), GlobalActionRole);
      mi->setData(QVariant(file), FileRole);
     _radioItem->appendRow(mi);
    }
    
    //! add Favorite
    MenuItem *mi3 = new MenuItem(tr("favorite stream"), QIcon(":/images/favorites-48x48.png"));
    mi3->setData(QVariant::fromValue(new QAction(QIcon(":/images/favorites-48x48.png"), tr("favorite stream"), this)), MenuActionRole);
    mi3->setData(QVariant::fromValue(actions->value("favoriteradioview")), GlobalActionRole);
    _radioItem->appendRow(mi3);    
}


//! --------- configureMenuAction ----------------------------------------------
void MenuModel::configureMenuAction() 
{
   for (int i=0; i < this->rowCount(QModelIndex()); i++) 
   {
    const QModelIndex childIdx = this->index(i, 0, QModelIndex());
    QAction *a = childIdx.data(MenuActionRole).value<QAction*>();
    a->setData(QVariant::fromValue(childIdx));
    connect(a, SIGNAL(triggered()),this, SLOT(findSender()));
  
    QStandardItem *childItem = itemFromIndex( childIdx );
  
    for (int j=0; j < childItem->rowCount(); j++) {
         const QModelIndex child2Idx = this->index(j, 0, childIdx);
         QAction *a = child2Idx.data(MenuActionRole).value<QAction*>();
         a->setData(QVariant::fromValue(child2Idx));
         connect(a, SIGNAL(triggered()),this, SLOT(findSender()));
      }
  }
}

//! --------- update -----------------------------------------------------------
// mise  jour ncessaire lors d'un ajout de fichier de radio utilisateur  
void MenuModel::update()
{ 
    populateRadioItem();
    configureMenuAction();
    emit modelUpdated();
} 

//! ----------- IndexFromViewMode ----------------------------------------------
QModelIndex MenuModel::IndexFromViewMode(MAIN::ViewMode m)
{
  //qDebug() << "-- MenuModel::IndexFromViewMode rechercher ViewMode m " << m;

  for (int j=0; j < this->rowCount(QModelIndex()); j++) 
  {
    const QModelIndex childIdx = this->index(j, 0, QModelIndex());
      
    QAction *a = childIdx.data(GlobalActionRole).value<QAction*>();
    if(a) {
      MAIN::ViewMode mode = MAIN::ViewMode(a->data().toInt());
      if(mode == m)
        return childIdx;
    }

    for (int i=0; i < this->rowCount(childIdx); i++) 
    {
        const QModelIndex child2Idx = this->index(i, 0, childIdx);

        QAction *a = child2Idx.data(GlobalActionRole).value<QAction*>();
        if(a) {
          MAIN::ViewMode mode = MAIN::ViewMode(a->data().toInt());
          if(mode == m) 
            return child2Idx;
        }
    }
  }

  return QModelIndex();
}


//! ----------- findSender -----------------------------------------------------
// Quand une action provenant du menu (action associ au item et visible par les menu 
// du breadcrumb --> on retrouve l'index dans le modele
void MenuModel::findSender()
{
   QAction *action = qobject_cast<QAction *>(sender());
   if(!action) return;
   
   QModelIndex idx = action->data().value<QModelIndex>();
    
   //qDebug() << "-- MenuModel --> findSender idx" << idx;
   emit modelItemActivated(idx);
}
