/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "CollectionModel/collectionmodel.h"
#include "MediaItem/mediaitem.h"

#include <QDebug>
#include <QRegExp>

CollectionModel* CollectionModel::INSTANCE = 0;
CollectionModel* _CollectionModel() {return CollectionModel::instance();}

/*******************************************************************************
    CollectionModel
*******************************************************************************/
CollectionModel::CollectionModel(QObject *parent) : QObject(parent)
{
    INSTANCE = this;
         
    _rootItem        = new MediaItem();
    _rootItem->type  = TYPE_UNKWON;
    
    _filterPattern   = "";
    _playingItemIdx  = 0;
    
}

CollectionModel::~CollectionModel()
{
    delete _rootItem;
}


MediaItem* CollectionModel::getItem(MediaItem* parent,const int childIdx)
{
    if(!parent) return 0;
    
    if(childIdx < 0 || childIdx >= parent->childCount()) return 0;
    
    return parent->child(childIdx);
}


MediaItem* CollectionModel::rootItem()
{
    return _rootItem;
}


MediaItem* CollectionModel::addChild(MediaItem* parent)
{
    if(!parent) return 0;
  
    return parent->addChildren();
}


void CollectionModel::clear()
{
    trackItemList.clear();
    albumItemList.clear();
    
    delete _rootItem;
    _rootItem        = new MediaItem();
    _rootItem->type  = TYPE_UNKWON;

    emit modelCleared();
}

//! ----------------------- Get tracks method ----------------------------------
QStringList CollectionModel::getItemChildrenTracks(MediaItem *parent)
{
    QStringList result;
    if(!parent) return result;

    MediaItem* item = parent;
    
    if (item->type == TYPE_TRACK) {
      if(isTrackFiltered(item)) {
        result.append( item->data[TRACK::url].toString() );
      }
    } 
    else {
      //! Recursive !!
      for (int i = 0; i < item->childCount(); i++) {
        result.append( getItemChildrenTracks(item->child(i)) );
      }
    }
    return result;
}

QStringList CollectionModel::getAlbumChildrenTracksGenre(MediaItem *album,const QString& genre) 
{
    QStringList result;
    if(!album) return result;
    if(album->type != TYPE_ALBUM) return result;

    for (int i = 0; i < album->childCount(); i++) {
    
      if(!isTrackFiltered(album->child(i))) continue;
      
      if (!QRegExp(genre, Qt::CaseSensitive).exactMatch(album->child(i)->data[TRACK::genre].toString())) continue;
      
      result.append( album->child(i)->data[TRACK::url].toString() );
    }
    return result;
}

     
//! ----------------------- Favorite status method -----------------------------
bool CollectionModel::itemIsFavorite(MediaItem * item) 
{
    switch (item->type) {
      case TYPE_TRACK  : break; //! unexpected case
      case TYPE_STREAM : return item->data[STREAM::isFavorite].toBool();break;
      case TYPE_ARTIST : return item->data[ARTIST::isFavorite].toBool();break;
      case TYPE_ALBUM  : return item->data[ALBUM::isFavorite].toBool();break;
      case TYPE_PLAYLIST : return item->data[PLAYLIST::isFavorite].toBool();break;
      default: break;
    }
    return false;
}


void CollectionModel::setItemIsFavorite(MediaItem * item, bool isfavorite) 
{
    switch (item->type) {
      case TYPE_TRACK  : break; //! unexpected case  
      case TYPE_STREAM : item->data[STREAM::isFavorite] = isfavorite;break;
      case TYPE_ARTIST : item->data[ARTIST::isFavorite] = isfavorite;break;
      case TYPE_ALBUM  : item->data[ALBUM::isFavorite] = isfavorite;break;
      case TYPE_PLAYLIST : item->data[PLAYLIST::isFavorite] = isfavorite;break;
      default: break;
    }
}


//! ----------------------- playing status method ------------------------------
bool CollectionModel::itemIsPlaying(const MediaItem * item)
{
    switch (item->type) {
      case TYPE_TRACK  : return item->data[TRACK::isPlaying].toBool();break;
      case TYPE_STREAM : return item->data[STREAM::isPlaying].toBool();break;
      case TYPE_ARTIST : return item->data[ARTIST::isPlaying].toBool();break;
      case TYPE_ALBUM  : return item->data[ALBUM::isPlaying].toBool();break;
      case TYPE_PLAYLIST : return item->data[PLAYLIST::isPlaying].toBool();break;
      default: break;
    }
    return false;
}


void CollectionModel::setItemIsPlaying(MediaItem * item, bool isplaying) 
{
    switch (item->type) {
      case TYPE_TRACK  : item->data[TRACK::isPlaying] = isplaying;break;
      case TYPE_STREAM : item->data[STREAM::isPlaying] = isplaying;break;
      case TYPE_ARTIST : item->data[ARTIST::isPlaying] = isplaying;break;
      case TYPE_ALBUM  : item->data[ALBUM::isPlaying] = isplaying;break;
      case TYPE_PLAYLIST : item->data[PLAYLIST::isPlaying] = isplaying;break;
      default: break;
    }
  
    emit signalPlayingStatusChanged();
}


MediaItem* CollectionModel::getPlayingTrackItem() 
{    
    return _playingItemIdx;
}

QString CollectionModel::getPlayingTrackItemGenre() 
{
    if(!_playingItemIdx) return QString();

    return _playingItemIdx->data[TRACK::genre].toString();
}


void CollectionModel::updatePlayingItem(const QString& filename)
{
  for ( int i = 0; i < _rootItem->childCount(); i++ )
   {
     MediaItem* artistItem = _rootItem->child(i);
     for (int j = 0; j < artistItem->childCount(); j++)
     {
       MediaItem* albumItem = artistItem->child(j);
       for (int k = 0; k < albumItem->childCount(); k++)
       {
         MediaItem* trackItem = albumItem->child(k);
	 
         if (filename == trackItem->data[TRACK::url].toString()) 
         //if (QRegExp(filename, Qt::CaseSensitive).exactMatch(trackItem->getData(2).toString())) 
         {
           _playingItemIdx = trackItem;
           trackItem->data[TRACK::isPlaying] = true;
           albumItem->data[ALBUM::isPlaying] = true;
           artistItem->data[ARTIST::isPlaying] = true;
           qDebug() << " ---- CollectionModel::updatePlayingItem emit signalPlayingStatusChanged";

           emit signalPlayingStatusChanged();
           return;
          }
      } // end for track
    } // end for album
  }  // end for artist
}

void CollectionModel::updateBrokenItem(const QString filename)
{
  for ( int i = 0; i < _rootItem->childCount(); i++ )
   {
     MediaItem* artistItem = _rootItem->child(i);
     for (int j = 0; j < artistItem->childCount(); j++)
     {
       MediaItem* albumItem = artistItem->child(j);
       for (int k = 0; k < albumItem->childCount(); k++)
       {
         MediaItem* trackItem = albumItem->child(k);
	 
         if (filename == trackItem->data[TRACK::url].toString()) 
         {
           trackItem->data[TRACK::isBroken] = true;
           emit signalPlayingStatusChanged();
           return;
          }
      } // end for track
    } // end for album
  }  // end for artist
}

void CollectionModel::stopPlayingItem()
{
    MediaItem* trackItem = getPlayingTrackItem();
    if(!trackItem) return;

    trackItem->data[TRACK::isPlaying] = false;
    
    MediaItem* albumItem = trackItem->parent();
    albumItem->data[ALBUM::isPlaying] = false;
  
    MediaItem* artistItem = albumItem->parent();
    artistItem->data[ARTIST::isPlaying] = false;
 
    _playingItemIdx = 0;
    emit signalPlayingStatusChanged();
}



//! ------------------------- filtering method ---------------------------------
// 0.0.43 : ajout return true if filterPatter = ""
bool CollectionModel::isArtistFiltered(MediaItem * artistItem) 
{
  if (!artistItem) return false;
  if (_filterPattern.isEmpty()) return true;
    
  //! check artist item 
  if (artistItem->data[ARTIST::name].toString().contains(_filterPattern, Qt::CaseInsensitive))
    return true;

  //! check child album
  for (int i = 0; i < artistItem->childCount(); i++) 
  {
    MediaItem* albumItem = artistItem->child(i);
    if (albumItem->data[ALBUM::name].toString().contains(_filterPattern, Qt::CaseInsensitive))
      return true;

    //! check child track
    for (int j = 0; j < albumItem->childCount(); j++)
    {
      MediaItem* trackItem = albumItem->child(j);
      if ( trackItem->data[TRACK::url].toString().contains(_filterPattern, Qt::CaseInsensitive) ||
           trackItem->data[TRACK::genre].toString().contains(_filterPattern, Qt::CaseInsensitive) )
        return true;
     }
  }

  return false;
}

bool CollectionModel::isAlbumFiltered(MediaItem* albumItem) 
{
  if (!albumItem) return false;
  if (_filterPattern.isEmpty()) return true;
      
  //! check album item 
  if (albumItem->data[ALBUM::name].toString().contains(_filterPattern, Qt::CaseInsensitive))
    return true;
    
  //! check artist parent
  MediaItem *artistItem = albumItem->parent();
  if (artistItem->data[ARTIST::name].toString().contains(_filterPattern, Qt::CaseInsensitive))
      return true;
  
  //! check child track
  for (int i = 0; i < albumItem->childCount(); i++)
  {
    MediaItem* trackItem = albumItem->child(i);
    if ( trackItem->data[TRACK::url].toString().contains(_filterPattern, Qt::CaseInsensitive) ||
         trackItem->data[TRACK::genre].toString().contains(_filterPattern, Qt::CaseInsensitive) )
      return true;
  }

  return false;
}


bool CollectionModel::isTrackFiltered(MediaItem* trackItem) 
{
  if (!trackItem) return false;
  if (_filterPattern.isEmpty()) return true;
      
  //! check track item 
  if ( trackItem->data[TRACK::url].toString().contains(_filterPattern, Qt::CaseInsensitive) ||
       trackItem->data[TRACK::genre].toString().contains(_filterPattern, Qt::CaseInsensitive) )
    return true;
    
  //! check album parent
  MediaItem *albumItem = trackItem->parent();
  if (albumItem->data[ALBUM::name].toString().contains(_filterPattern, Qt::CaseInsensitive))
      return true;
  
  //! check artist parent
  MediaItem* artistItem = albumItem->parent();
  if (artistItem->data[ARTIST::name].toString().contains(_filterPattern, Qt::CaseInsensitive))
      return true;

  return false;
}



void CollectionModel::sendPlaycountUpdated() 
{
    emit signalPlaycountChanged(); 
}


