/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef _COLLECTION_MODEL_H
#define _COLLECTION_MODEL_H

#include <QVariant>
#include <QString>
#include <QStringList>
#include <QObject>


/*******************************************************************************
    CollectionModel
*******************************************************************************/
class MediaItem;

class CollectionModel : public QObject
{
static CollectionModel* INSTANCE;

Q_OBJECT
  public:
     CollectionModel(QObject *parent = 0);
     ~CollectionModel();
     static CollectionModel* instance() { return INSTANCE; }    
    
     //! custom method
     MediaItem* rootItem();
     MediaItem* addChild(MediaItem* parent);
     MediaItem* getItem(MediaItem* parent,const int childIdx); 
     void clear();
     
     //! get Track method
     QStringList getItemChildrenTracks(MediaItem *parent);
     QStringList getAlbumChildrenTracksGenre(MediaItem *album,const QString& genre);

     //! filtering method
     bool isTrackFiltered(MediaItem * trackItem);
     bool isAlbumFiltered(MediaItem * albumItem);
     bool isArtistFiltered(MediaItem * artistItem);
     QString                      _filterPattern;
     
     //! playing status method
     bool itemIsPlaying(const MediaItem* item);
     void setItemIsPlaying(MediaItem * item, bool playing);
     
     MediaItem* getPlayingTrackItem();
     QString getPlayingTrackItemGenre();
     void updatePlayingItem(const QString& filename);
     void stopPlayingItem();

     //! Favorites status method
     bool itemIsFavorite(MediaItem * item);
     void setItemIsFavorite(MediaItem * item, bool isfavorite);
     
     //! Broken status method
     void updateBrokenItem(const QString filename);

     //! list of MediaItem
     QList<MediaItem *>      trackItemList;
     QList<MediaItem *>      albumItemList;
     
     void sendPlaycountUpdated();
     
  signals:
    void signalPlayingStatusChanged();
    void signalFavoriteStatusChanged();    
    void signalPlaycountChanged();
    void modelCleared();

  private:
     MediaItem *       _rootItem;
     MediaItem *       _playingItemIdx;
};

#endif // _COLLECTION_MODEL_H
