/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "CollectionModel/collectionpopulator.h"
#include "CollectionModel/collectionmodel.h"
#include "MediaItem/mediaitem.h"
#include "Database/database.h"
#include "utilities.h"

#include <QFileInfo>
#include <QSqlField>
#include <QSqlQuery>
#include <QSqlDriver>
#include <QVariant>
#include <QDebug>

/*******************************************************************************
    CollectionPopulator
*******************************************************************************/
CollectionPopulator::CollectionPopulator()
{
  _collectionModel = CollectionModel::instance();
  _player          = Player::instance();
  _isRunning       = false;
}

void CollectionPopulator::run()
{
    qDebug() << " --- CollectionPopulator--> Starting model population";

    Database db;
    if (!db.connect("populateModel")) {
        _isRunning = false;
        return;
    }

    _collectionModel->clear();

    MediaItem*  artistItem;
    MediaItem*  albumItem;
    MediaItem*  trackItem;
    {
      QSqlQuery artistsQuery("SELECT id,interpret,favorite,playcount FROM interprets ORDER BY interpret ASC",*db.sqlDb());
      while (artistsQuery.next()) 
      {
          artistItem = _collectionModel->addChild(_collectionModel->rootItem());
          artistItem->type = TYPE_ARTIST;
          artistItem->data[ARTIST::id]            =  artistsQuery.value(0).toInt();
          artistItem->data[ARTIST::name]          =  artistsQuery.value(1).toString();
          artistItem->data[ARTIST::coverpathlist] =  QStringList();
          artistItem->data[ARTIST::isFavorite]    =  artistsQuery.value(2).toInt();
          artistItem->data[ARTIST::playcount]     =  artistsQuery.value(3).toInt();

          QSqlQuery albumsQuery("SELECT id,album,year,cover,favorite,playcount FROM albums WHERE id IN (SELECT album FROM tracks WHERE interpret="+artistsQuery.value(0).toString()+") ORDER BY year ASC;",
                                          *db.sqlDb());

          while (albumsQuery.next()) 
          {
              albumItem = _collectionModel->addChild(artistItem);
              albumItem->type = TYPE_ALBUM;
              albumItem->data[ALBUM::id]            =  albumsQuery.value(0).toInt();
              albumItem->data[ALBUM::name]          =  albumsQuery.value(1).toString();
              albumItem->data[ALBUM::year]          =  albumsQuery.value(2).toString();
              albumItem->data[ALBUM::coverpath]     =  albumsQuery.value(3).toString();
              albumItem->data[ALBUM::isFavorite]    =  albumsQuery.value(4).toInt();
              albumItem->data[ALBUM::playcount]     =  albumsQuery.value(5).toInt();
              
              _collectionModel->albumItemList.append(albumItem);
              
              QSqlQuery tracksQuery("SELECT id,trackname,filename,track,genre,length FROM view_tracks WHERE albumID="+albumsQuery.value(0).toString()+" AND interpretID="+artistsQuery.value(0).toString()+" ORDER BY track ASC;",
                                              *db.sqlDb());
              while (tracksQuery.next()) 
              {
                  trackItem = _collectionModel->addChild(albumItem);
                  trackItem->type = TYPE_TRACK;
                  trackItem->data[TRACK::id]         =  tracksQuery.value(0).toInt();
                  trackItem->data[TRACK::title]      =  tracksQuery.value(1).toString();
                  trackItem->data[TRACK::url]        =  tracksQuery.value(2).toString();
                  trackItem->data[TRACK::num]        =  tracksQuery.value(3).toString();
                  trackItem->data[TRACK::artist]     =  artistItem->data[ARTIST::name];
                  trackItem->data[TRACK::album]      =  albumItem->data[ALBUM::name];
                  trackItem->data[TRACK::duration]   =  tracksQuery.value(5).toInt();
                  trackItem->data[TRACK::genre]      =  tracksQuery.value(4).toString();
                  
                  _collectionModel->trackItemList.append(trackItem);
                  
              } //! fin while track
          } //! fin while album
      } //! Fin while artist
    }

    //! Sort Media Track Item list By Genre
    qSort(_collectionModel->trackItemList.begin(), _collectionModel->trackItemList.end(), 
          MEDIA::compareTrackItemGenre);
   
    //! Sort Media Album Item list By Year
    qSort(_collectionModel->albumItemList.begin(), _collectionModel->albumItemList.end(), 
          MEDIA::compareAlbumItemYear);

    
    //! Update of playing item status with current track playing from Player 
    QString currentTrack = _player->currentMediaSource();
    if(!currentTrack.isEmpty()) {
        _collectionModel->updatePlayingItem(QFileInfo(currentTrack).absoluteFilePath());
    }
    qDebug() << " --- CollectionPopulator -->Ending model population:";

    _isRunning = false;

    emit modelPopulated();
}
