/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "histomodel.h"
#include "MediaItem/mediaitem.h"
#include "Player/player.h"
#include "Database/database.h"
#include <QSqlQuery>
#include <QSqlDriver>

#include "collectionmodel.h"


HistoModel* HistoModel::INSTANCE = 0;
HistoModel* _HistoModel() {return HistoModel::instance();}

/*******************************************************************************
    HistoModel
*******************************************************************************/
HistoModel::HistoModel(QObject *parent) : QObject(parent) 
{
    qDebug() << " ---- HistoModel --> creation";
    INSTANCE          = this;
    modelUpdated      = false;
    _filterPattern    = "";
    _activeItem       = 0;
}

void HistoModel::clear()
{
    //! Do not delete item because we use Collection Model item
    qDeleteAll(_listMediaItem.constBegin(), _listMediaItem.constEnd());    
    _listMediaItem.clear();
    _activeItem       = 0;
}

//! ------------------ HistoModel::updateModel ----------------------------------
void HistoModel::updateModel()
{
    qDebug() << " ---- HistoModel --> updateModel";
    this->clear();
    
    Database db;
    if (!db.connect("Histo")) {
        return;
    }
    
    QSqlQuery histoQuery("SELECT id,url,name,date FROM histo ORDER BY date DESC",*db.sqlDb());
    while (histoQuery.next()) 
    {
      MediaItem *mi = new MediaItem();

      QString url =  histoQuery.value(1).toString();
      if(MEDIA::isUrlLocal(url)) 
      {
	MediaItem* miDb = MEDIA::FromDataBase("HistoModel",url);
        if(miDb) {
	  delete mi;
	  mi = miDb;
	  mi->data[TRACK::name]       = histoQuery.value(2).toString();
	  mi->data[TRACK::lastPlayed] = histoQuery.value(3).toString();
	} 
	else {
	  mi->type                    = TYPE_TRACK;
	  mi->data[TRACK::url]        = url;
	  mi->data[TRACK::name]       = histoQuery.value(2).toString();
	  mi->data[TRACK::title]      = histoQuery.value(2).toString();
	  mi->data[TRACK::lastPlayed] = histoQuery.value(3).toString();
	}
      }
      else 
      {
        mi->type                     = TYPE_STREAM;
        mi->data[STREAM::url]        = url;
        mi->data[STREAM::name]       = histoQuery.value(2).toString();
        mi->data[STREAM::lastPlayed] = histoQuery.value(3).toString();
      }
      
      _mutex.lock();
      _listMediaItem.append(mi);
      _mutex.unlock();

    } // end while history request
    
    modelUpdated = true;
    
    QString currentTrack = Player::instance()->currentMediaSource();
    if(!currentTrack.isEmpty()) {
        updatePlayingItem(currentTrack);
    }
}


//!------------HistoModel::getItem ---------------------------------------------
MediaItem* HistoModel::getItem(int row) 
{
   if (row < 0 || row >= _listMediaItem.size())
        return 0;

   return _listMediaItem.at(row);
}

//! --------- playing item method ----------------------------------------------
void HistoModel::updatePlayingItem(const QString& url)
{
  //qDebug() << " ---- HistoModel --> updatePlayingItem" << url;

  for ( int i = 0; i < _listMediaItem.size(); i++ )
  {
     MediaItem* mi = _listMediaItem.at(i);
     if (url ==  mi->data[TRACK::url].toString()) //! works STREAM or TRACK
     {
       _activeItem = mi;
       if(mi->type == TYPE_TRACK)
         mi->data[TRACK::isPlaying]         = true;
       else
         mi->data[STREAM::isPlaying]        = true;
       
       emit signalPlayingStatusChanged();
       break;
     }
  }
}

void HistoModel::stopPlayingItem()
{
  if(!_activeItem) return;
  
  
  if(_activeItem->type == TYPE_TRACK)
    _activeItem->data[TRACK::isPlaying]         = false;
  else
    _activeItem->data[STREAM::isPlaying]        = false;
       
  _activeItem = 0;
 
}  
  
//! --------- Filtering method -------------------------------------------------
bool HistoModel::isItemFiltered(const int row)
{
  if(_filterPattern.isEmpty()) return true;
  
  MediaItem* mi = getItem(row);
  
  if ( mi->type == TYPE_STREAM && mi->data[STREAM::name].toString().contains(_filterPattern, Qt::CaseInsensitive) )
      return true;
  else if ( mi->type == TYPE_TRACK && mi->data[TRACK::name].toString().contains(_filterPattern, Qt::CaseInsensitive) )
      return true;
  
  return false;
}


//! --------- broken item method ----------------------------------------------
void HistoModel::updateBrokenItem(const QString& url)
{
  //qDebug() << " ---- HistoModel --> updateBrokenItem" << url;

  for ( int i = 0; i < _listMediaItem.size(); i++ )
  {
     MediaItem* mi = _listMediaItem.at(i);    
    
     if (url ==  mi->data[TRACK::url].toString()) //! works STREAM or TRACK
     {
       if(mi->type == TYPE_TRACK)
         mi->data[TRACK::isBroken]         = true;
       else
         mi->data[STREAM::isBroken]        = true;
       
       emit signalPlayingStatusChanged();
       break;
     }
  }
}
