/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "CollectionModel/plsfilemodel.h"
#include "MediaItem/mediaitem.h"

#include <QDebug>
#include <QRegExp>

PlsFileModel* PlsFileModel::INSTANCE = 0;
PlsFileModel* _PlsFileModel() {return PlsFileModel::instance();}



/*******************************************************************************
    PlsFileModel
*******************************************************************************/
PlsFileModel::PlsFileModel(QObject *parent) : QObject(parent)
{
    INSTANCE = this;
         
    _rootItem        = new MediaItem();
    _rootItem->type  = TYPE_UNKWON;
    
    _filterPattern   = "";
    _playingItemIdx  = 0;
}

PlsFileModel::~PlsFileModel()
{
    delete _rootItem;
}


MediaItem* PlsFileModel::getItem(MediaItem* parent,const int childIdx) 
{
    if(!parent) return 0;
    
    if(childIdx < 0 || childIdx >= parent->childCount()) return 0;
    
    return parent->child(childIdx);
}


MediaItem* PlsFileModel::rootItem()
{
  return _rootItem;
}


MediaItem* PlsFileModel::addChild(MediaItem* parent)
{
  if(!parent) return 0;
  
  return parent->addChildren();
}


void PlsFileModel::clear()
{
    delete _rootItem;
    _rootItem        = new MediaItem();
    _rootItem->type  = TYPE_UNKWON;
    
    emit modelCleared();
}

//! ----------------------- Get urls method ------------------------------------
QStringList PlsFileModel::getItemChildrenUrls(MediaItem *parent)
{
    QStringList result;
    if(!parent) return result;

    MediaItem* item = parent;
    
    if (item->type == TYPE_TRACK || item->type == TYPE_STREAM) {
      if(isUrlFiltered(item))
        result.append( item->data[TRACK::url].toString() ); // works also for STREAM
    } 
    else {
      //! Recursive !!
      for (int i = 0; i < item->childCount(); i++) {
        result.append( getItemChildrenUrls(item->child(i)) );
      }
    }
    return result;
}
     
//! ----------------------- Favorite status method -----------------------------
bool PlsFileModel::itemIsFavorite(MediaItem * item) 
{
  switch (item->type) {
    case TYPE_TRACK  : break; //! unexpected case
    case TYPE_STREAM : return item->data[STREAM::isFavorite].toBool();break;
    case TYPE_ARTIST : return item->data[ARTIST::isFavorite].toBool();break;
    case TYPE_ALBUM  : return item->data[ALBUM::isFavorite].toBool();break;
    case TYPE_PLAYLIST : return item->data[PLAYLIST::isFavorite].toBool();break;
    default: break;
  }
  return false;
}


void PlsFileModel::setItemIsFavorite(MediaItem * item, bool isfavorite) 
{
  switch (item->type) {
    case TYPE_TRACK  : break; //! unexpected case  
    case TYPE_STREAM : item->data[STREAM::isFavorite] = isfavorite;break;
    case TYPE_ARTIST : item->data[ARTIST::isFavorite] = isfavorite;break;
    case TYPE_ALBUM  : item->data[ALBUM::isFavorite] = isfavorite;break;
    case TYPE_PLAYLIST : item->data[PLAYLIST::isFavorite] = isfavorite;break;
    default: break;
  }
}


//! ----------------------- playing status method ------------------------------
bool PlsFileModel::itemIsPlaying(MediaItem * item)
{
  switch (item->type) {
    case TYPE_TRACK  : return item->data[TRACK::isPlaying].toBool();break;
    case TYPE_STREAM : return item->data[STREAM::isPlaying].toBool();break;
    case TYPE_ARTIST : return item->data[ARTIST::isPlaying].toBool();break;
    case TYPE_ALBUM  : return item->data[ALBUM::isPlaying].toBool();break;
    case TYPE_PLAYLIST : return item->data[PLAYLIST::isPlaying].toBool();break;
    default: break;
  }
  return false;
}


void PlsFileModel::setItemIsPlaying(MediaItem * item, bool isplaying) 
{
  switch (item->type) {
    case TYPE_TRACK  : item->data[TRACK::isPlaying] = isplaying;break;
    case TYPE_STREAM : item->data[STREAM::isPlaying] = isplaying;break;
    case TYPE_ARTIST : item->data[ARTIST::isPlaying] = isplaying;break;
    case TYPE_ALBUM  : item->data[ALBUM::isPlaying] = isplaying;break;
    case TYPE_PLAYLIST : item->data[PLAYLIST::isPlaying] = isplaying;break;
    default: break;
  }
  
  emit signalPlayingStatusChanged();
}


MediaItem* PlsFileModel::getPlayingUrlItem() 
{
   for ( int i = 0; i < _rootItem->childCount(); i++ )
   {
     MediaItem* playlistItem = _rootItem->child(i);
     for (int j = 0; j < playlistItem->childCount(); j++)
     {
       if( itemIsPlaying(playlistItem->child(j)) )
         return playlistItem->child(j);
     } // end for playlist item
  } // end for playlist

  return 0;
}



void PlsFileModel::updatePlayingItem(const QString& url)
{
  for ( int i = 0; i < _rootItem->childCount(); i++ )
   {
     MediaItem* playlistItem = _rootItem->child(i);
     for (int j = 0; j < playlistItem->childCount(); j++)
     {
       MediaItem* urlItem = playlistItem->child(j);
       if (url == urlItem->data[TRACK::url].toString()) //! work for STREAM & TRACK
       {
	 if(urlItem->type == TYPE_TRACK)
	   urlItem->data[TRACK::isPlaying] = true;
	 else
	   urlItem->data[STREAM::isPlaying] = true;
	 
         playlistItem->data[PLAYLIST::isPlaying] = true;
         qDebug() << " ---- PlsFileModel::updatePlayingItem emit signalPlayingStatusChanged";
         emit signalPlayingStatusChanged();
         return;
       }
    } // end for track
  }  // end for playlist
}

void PlsFileModel::updateBrokenItem(const QString& url)
{
  for ( int i = 0; i < _rootItem->childCount(); i++ )
  {
     MediaItem* playlistItem = _rootItem->child(i);
     for (int j = 0; j < playlistItem->childCount(); j++)
     {
       MediaItem* trackItem = playlistItem->child(j);
       if (url == trackItem->data[TRACK::url].toString())
       {
         trackItem->data[TRACK::isBroken] = true;
         emit signalPlayingStatusChanged();
         return;
       }
     } // end track loop
  }  // end for playlist loop
}

void PlsFileModel::stopPlayingItem()
{
    MediaItem* trackItem = getPlayingUrlItem();
    if(!trackItem) return;

    if(trackItem->type == TYPE_TRACK)
      trackItem->data[TRACK::isPlaying] = false;
    else if(trackItem->type == TYPE_STREAM)
      trackItem->data[STREAM::isPlaying] = false;
    
    //! update playlist parents
    MediaItem* playlistItem = trackItem->parent();
    playlistItem->data[PLAYLIST::isPlaying] = false;
 
    emit signalPlayingStatusChanged();
}


//! ------------------------- filtering method ---------------------------------
// 0.0.43 : ajout return true if filterPatter = ""
bool PlsFileModel::isPlaylistFiltered(MediaItem* playlistItem) 
{
  if (!playlistItem) return false;
  if (_filterPattern.isEmpty()) return true;
      
  //! check playlist name
  if (playlistItem->data[PLAYLIST::url].toString().contains(_filterPattern, Qt::CaseInsensitive) ||
      playlistItem->data[PLAYLIST::name].toString().contains(_filterPattern, Qt::CaseInsensitive)
     )
    return true;

  //! check playlist child (url name)
  for (int i = 0; i < playlistItem->childCount(); i++) 
  {
      MediaItem* trackItem = playlistItem->child(i);
      if ( (trackItem->type == TYPE_TRACK && trackItem->data[TRACK::title].toString().contains(_filterPattern, Qt::CaseInsensitive)) ||
	   (trackItem->type == TYPE_STREAM && trackItem->data[STREAM::name].toString().contains(_filterPattern, Qt::CaseInsensitive)) )
        return true;
   }

  return false;
}



bool PlsFileModel::isUrlFiltered(MediaItem* urlItem) 
{
  if (!urlItem) return false;
  if (_filterPattern.isEmpty()) return true;
      
  //! check track item 
  if ( (urlItem->type == TYPE_TRACK && urlItem->data[TRACK::title].toString().contains(_filterPattern, Qt::CaseInsensitive)) ||
       (urlItem->type == TYPE_STREAM && urlItem->data[STREAM::name].toString().contains(_filterPattern, Qt::CaseInsensitive)) )
    return true;
    
  //! check playlist parent name & url
  MediaItem* playlistItem = urlItem->parent();
  if (playlistItem->data[PLAYLIST::url].toString().contains(_filterPattern, Qt::CaseInsensitive) ||
      playlistItem->data[PLAYLIST::name].toString().contains(_filterPattern, Qt::CaseInsensitive)
     )
    return true;
 
  return false;
}
