/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "CollectionModel/plsfilepopulator.h"
#include "CollectionModel/plsfilemodel.h"
#include "MediaItem/mediaitem.h"
#include "Database/database.h"


#include <QFileInfo>
#include <QSqlField>
#include <QSqlQuery>
#include <QSqlDriver>
#include <QVariant>
#include <QDebug>



/*******************************************************************************
    PlsFilePopulator
*******************************************************************************/
PlsFilePopulator::PlsFilePopulator()
{
  _plsFileModel    = PlsFileModel::instance();
  _player          = Player::instance();
  _isRunning       = false;
}

void PlsFilePopulator::run()
{
    _isRunning = true;
    qDebug() << " --- PlsFilePopulator --> Starting model population";
    
    Database db;
    if (!db.connect("plsfilepopulator")) {
      _isRunning = false;
      return;
    }
    
    _plsFileModel->clear();

    MediaItem* playlistItem;
    MediaItem* trackItem;
    {
        QSqlQuery playlistsQuery("SELECT id,filename,name,icon,rules,type,mtime,favorite FROM playlists ORDER BY type ASC",*db.sqlDb());

        while (playlistsQuery.next()) 
        {
            playlistItem = _plsFileModel->addChild(_plsFileModel->rootItem());
            playlistItem->type = TYPE_PLAYLIST;
            playlistItem->data[PLAYLIST::id]         =  playlistsQuery.value(0).toInt();
            playlistItem->data[PLAYLIST::url]        =  playlistsQuery.value(1).toString();
            playlistItem->data[PLAYLIST::name]       =  playlistsQuery.value(2).toString();
            playlistItem->data[PLAYLIST::icon]       =  playlistsQuery.value(3).toString();
            playlistItem->data[PLAYLIST::rules]      =  playlistsQuery.value(4).toString();
            playlistItem->data[PLAYLIST::date]       =  playlistsQuery.value(6).toUInt();
            playlistItem->data[PLAYLIST::type]       =  playlistsQuery.value(5).toInt();
            playlistItem->data[PLAYLIST::isFavorite] =  playlistsQuery.value(7).toInt();
          
            QSqlQuery playlistItemQuery("SELECT id,url,name FROM playlistitem WHERE playlist="+QString::number(playlistsQuery.value(0).toInt())+"", *db.sqlDb());
  
            while (playlistItemQuery.next()) 
            {
                trackItem = _plsFileModel->addChild(playlistItem);
                QString url = playlistItemQuery.value(1).toString();
		
                if(MEDIA::isUrlLocal(url)) {
		  QSqlField data("col",QVariant::String);
		  data.setValue(QFileInfo(url).absoluteFilePath());
		  QString fname = db.sqlDb()->driver()->formatValue(data,false);
		  
		  QSqlQuery tracksQuery("SELECT id,filename,trackname,track,length,interpret,genre,album,year FROM view_tracks WHERE filename="+fname+" LIMIT 1;",
                                              *db.sqlDb());
        
                  if (tracksQuery.first()) {
		    trackItem->type = TYPE_TRACK;
		    trackItem->data[TRACK::id]       = tracksQuery.value(0).toInt();
		    trackItem->data[TRACK::url]      = url;
		    trackItem->data[TRACK::name]     = tracksQuery.value(1).toString();
		    trackItem->data[TRACK::title]    = tracksQuery.value(2).toString();
		    trackItem->data[TRACK::num]      = tracksQuery.value(3).toInt();
		    trackItem->data[TRACK::duration] = tracksQuery.value(4).toInt();
		    trackItem->data[TRACK::artist]   = tracksQuery.value(5).toString();
		    trackItem->data[TRACK::genre]    = tracksQuery.value(6).toString();
		    trackItem->data[TRACK::album]    = tracksQuery.value(7).toString();
		    trackItem->data[TRACK::year]     = tracksQuery.value(8).toInt();
		  } 
		  else {
		    trackItem->type    = TYPE_TRACK;
		    trackItem->data[TRACK::id]       =  playlistItemQuery.value(0).toInt();
		    trackItem->data[TRACK::url]      =  url;
		    trackItem->data[TRACK::title]     =  playlistItemQuery.value(2).toString();
		  }
                }
                else {
                  trackItem->type    = TYPE_STREAM;
                  trackItem->data[STREAM::id]      =  playlistItemQuery.value(0).toInt();
                  trackItem->data[STREAM::url]     =  url;
                  trackItem->data[STREAM::name]    =  playlistItemQuery.value(2).toString();                
                }
                  
            } //! Fin while playlist item
        } //! Fin while playlist
    }
	    
	    
    //! Update of playing item status with current track playing from Player 
    QString currentTrack = _player->currentMediaSource();
    if(!currentTrack.isEmpty()) {
          _plsFileModel->updatePlayingItem(currentTrack);
    }
    qDebug() << " --- PlsFilePopulator --> End model population";

    _isRunning = false;
    emit modelPopulated();
}
