/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef _COLLECTION_SCENE_H_
#define _COLLECTION_SCENE_H_

#include "CollectionView/graphicitem.h"
#include "CollectionView/virtualplaylist.h"
#include "Player/player.h"

#include "viewmode.h"

#include <QStringList>
#include <QGraphicsScene>
#include <QtGui>

class CollectionModel;
class PlsFileModel;
class HistoModel;

/*******************************************************************************
    CollectionScene 
*******************************************************************************/
class CollectionScene : public QGraphicsScene
{
Q_OBJECT
  public: 
    CollectionScene(QWidget *parent);
    ~CollectionScene();
    void  setVirtualPlaylist(VirtualPlaylist  *playlist);
    int   getSceneInfoSize() {return _infoSize;}

    MAIN::ViewMode                _collectionMode;
    CollectionModel              *_collectionModel;
    PlsFileModel                 *_plsFileModel;
    HistoModel                   *_histoModel;

    QList<CategorieGraphicItem*> _listCategorie;    

  private :
    VirtualPlaylist             *_virtualPlaylist;
    Player                      *_player;
    QWidget                     *_parentWidget;
        
    QList<TrackGraphicItem*>    _listTrack;    
    int                         _infoSize;

    QMenu                       *_menuAlbum;
    QMenu                       *_menuAlbumGenre;
    QMenu                       *_menuArtist;
    QMenu                       *_menuPlaylist;    
    QAction                     *albumFavoriteAct;
    QAction                     *artistFavoriteAct;
    QAction                     *playlistFavoriteAct;
    QAction                     *extendedView;
    QAction                     *gridView;
    QGraphicsItem               *_mouseGrabbedItem;
    
    bool                        isAlbumGridView;
    
  protected:
    void contextMenuEvent(QGraphicsSceneContextMenuEvent *event);

  private slots :
    //! population scene method
    void populateAlbumScene();
    void populateAlbumGridScene();
    void populateArtistScene();
    void populateTrackScene();
    void populateGenreScene();
    void populateYearScene();
    void populateFavoriteScene();
    void populateFavoritePlaylistScene();
    void populatePlaylistScene();
    void populatePlaylistDetailScene();
    void populateHistoScene();
	
    //! playing item scene method
    void playAlbum();
    void playAlbumGenre();
    void playArtist();
    void playTrack();
    void playPlaylist();
    //! album graphic item method
    void loadCoverArtFromFile();
    void loadLastFmCover();
    void removeCoverArt();
    void modifyAlbumInfo();
    void modifyGenreInfo();
    void modifyArtistInfo();
    void updateAlbumFavorite();
    void updateArtistFavorite();
    void removePlaylistFromDisk();
    void modifyPlaylistInfo();
    void updatePlaylistFavorite();    
    //! history scene method
    void clearHistory();
    //! update scene method
    void updateMouseGrabbedItem();
    void itemDoubleClick();
    //! album view style
    void setAlbumExtendedView();
    void setAlbumGridView();
    
  public slots:
    void updateAlbumViewParam();
    void populateScene();
    void updateTrackPlaying(const QString& filename);
    void updateTrackBroken(const QString& filename);
    
  signals:
    void signal_RepopulateCollection();
    void signal_RepopulatePlsFile();
};

#endif // _COLLECTION_SCENE_H_
