/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "CollectionView/graphicitem.h"
#include "MediaItem/mediaitem.h"
#include "MediaItem/mediamimedata.h"
#include "utilities.h"

#include "qpixmapfilter.h"   //! version 0.0.44

#include "CollectionModel/collectionmodel.h"  //! 0.0.45
#include "CollectionModel/plsfilemodel.h"     //! 0.0.45

#include "SmartPlaylist/smartplaylist.h"      //! 0.0.49

#include "GraphicViewWidget/graphicviewwidget.h"  //! 0.0.50

#include <QPen>
#include <qstyle.h>

namespace GLOBAL {
    QMap<QString, QAction*>* globalActions();
}

//!-----------------------------------------------------------------------------
//! InfoGraphicItem 
//!-----------------------------------------------------------------------------
InfoGraphicItem::InfoGraphicItem() 
{
   setAcceptsHoverEvents(false);
   setAcceptDrops(false);
   setFlag(QGraphicsItem::ItemIsSelectable, false);
   setFlag(QGraphicsItem::ItemIsMovable, false);
   setFlag(QGraphicsItem::ItemIgnoresTransformations, true);
   _width = 630;
}

QRectF InfoGraphicItem::boundingRect() const
{
    return QRectF(0, 0, _width, 40);
}

void InfoGraphicItem::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * )
{
Q_UNUSED(option)

   //! draw text
   QRect rect = boundingRect().toRect();
   if(_width < 250)
     return;
       
   //! Draw the box
   painter->setPen(QPen(QColor(50, 200, 200, 90), 1));
   painter->setBrush(QBrush( QColor( 50, 200, 200, 65) ,Qt::SolidPattern));
   painter->drawRoundedRect(rect.adjusted(50,0,-100,0), 10.0, 10.0);

   //! Draw the text
   painter->setFont(QFont("Ubuntu", 10, QFont::Black));
   painter->setPen(QColor(0x3a3842));

   QFontMetrics metric( painter->font());
   QString elided = metric.elidedText ( _text, Qt::ElideRight, _width -100);
   painter->drawText(rect.adjusted(50,10,-100,-10), Qt::AlignCenter, elided);
}


//!-----------------------------------------------------------------------------
//! CategorieGraphicItem 
//!-----------------------------------------------------------------------------
CategorieGraphicItem::CategorieGraphicItem() 
{
   setAcceptsHoverEvents(false);
   setAcceptDrops(false);
   setFlag(QGraphicsItem::ItemIsSelectable, false);
   setFlag(QGraphicsItem::ItemIsMovable, false);
   setFlag(QGraphicsItem::ItemIgnoresTransformations, true);
   _width = 630;
}

QRectF CategorieGraphicItem::boundingRect() const
{
    return QRectF(0, 0, _width, 50);
}

void CategorieGraphicItem::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * )
{
Q_UNUSED(option)

   //! draw categorie text
  QFontDatabase * fontdb = new QFontDatabase();
  painter->setFont(fontdb->font("Segoe UI", "Light", 14) );
  
  QColor outlineColor = QColor ( 0, 0, 0, 50);
  QColor inlineColor = QColor ( 0, 0, 0, 255);

  painter->setPen(inlineColor);
  painter->drawStaticText(5,0, QStaticText(_catName));
 
  //BEGIN: horizontal line
  {
    QPoint start(boundingRect().toRect().left()+3, boundingRect().toRect().top()+22);
    start.rx() += 3;
    QPoint horizontalGradTop(boundingRect().toRect().topLeft());
    horizontalGradTop.rx() += boundingRect().toRect().width() - 6;
    QLinearGradient gradient(start, horizontalGradTop);
    gradient.setColorAt(0, inlineColor);
    gradient.setColorAt(1, outlineColor);	
    painter->fillRect(QRect(start, QSize(boundingRect().toRect().width() - 6, 1)), gradient);
  }
  //END: horizontal line
}

/*
********************************************************************************
*                                                                              *
*    AlbumGraphicItem                                                          *
*                                                                              * 
********************************************************************************
*/
AlbumGraphicItem::AlbumGraphicItem()
{
   setAcceptsHoverEvents(true);
   setAcceptDrops(true);
   setFlag(QGraphicsItem::ItemIsSelectable, false);
   setFlag(QGraphicsItem::ItemIsMovable, false);
   setFlag(QGraphicsItem::ItemIgnoresTransformations, true);
   setCursor(Qt::OpenHandCursor);
   
   //! option configuration
   QStyle *style =  QApplication::style();
   opt.widget = 0;
   opt.palette = QApplication::palette();
   opt.font = QApplication::font();
   opt.fontMetrics = QFontMetrics(opt.font);
    
   opt.showDecorationSelected = style->styleHint(QStyle::SH_ItemView_ShowDecorationSelected);
   int pm = style->pixelMetric(QStyle::PM_IconViewIconSize);
    
   opt.decorationSize = QSize(pm, pm);
   opt.decorationPosition = QStyleOptionViewItem::Top;
   opt.displayAlignment = Qt::AlignCenter;
   
   opt.locale.setNumberOptions(QLocale::OmitGroupSeparator);
   opt.state |= QStyle::State_Active;
   opt.state |= QStyle::State_Enabled;
}


QRectF AlbumGraphicItem::boundingRect() const
{
    return QRectF(0, 0, 140, 150);
}

void AlbumGraphicItem::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget)
{
Q_UNUSED(option)
   //! get Model data 
   QString _title     =  media->data[ALBUM::name].toString();
   QString _year      =  media->data[ALBUM::year].toString();
   QString _coverPath =  media->data[ALBUM::coverpath].toString();
      
   QPixmap pixTemp(QSize(140,150));
   {
      pixTemp.fill(Qt::transparent);
      QPainter p;
      p.begin(&pixTemp);
 
      //! Draw frame for State_HasFocus item
      QStyle *style = widget ? widget->style() : QApplication::style();
      opt.rect = boundingRect().toRect().adjusted(0,0,0,0);  
      style->drawControl(QStyle::CE_ItemViewItem, &opt, &p, widget);
      
      //! paint cover art
      //QPixmap pix = QPixmap(UTIL::CONFIGDIR + "/albums/" + _coverPath);
      //if(pix.isNull()) pix = QPixmap(":/images/emptycoverblue110x110.png");
      
              //! TEST 0.0.45c
              QPixmap pix;
              if(!QPixmapCache::find(_coverPath, pix)) {
                pix = QPixmap(UTIL::CONFIGDIR + "/albums/" + _coverPath);
                if(pix.isNull()) {
                  pix = QPixmap(":/images/emptycoverblue110x110.png");
                }
                else {
                  QPixmapCache::insert(_coverPath, pix);
                }
              }
              //! fin TEST 0.0.45c
      
      p.drawPixmap(15,10, pix);

    p.end();
   }
   
   painter->drawPixmap(0, 0, pixTemp);
   
   //! manual graphics shadow (Qgraphics Effect cause refresh pb)
   drawShadow(painter,QPointF(0,0),pixTemp,boundingRect());  
  
   //! paint title
   QColor color = opt.state & QStyle::State_HasFocus ? QColor("white") : QColor(0x315996);
   painter->setPen(color);
   painter->setFont(QFont("Arial", 8, QFont::Normal));
    
   QFontMetrics metric( painter->font());
   QString _titleElided = metric.elidedText ( _title, Qt::ElideRight, 280);
   QRect textBox = QRect (0,110,140, 40);
   QRect fontrect = metric.boundingRect (textBox,  Qt::AlignCenter | Qt::TextWordWrap, _titleElided, 0, 0 );
   painter->drawText(textBox.adjusted(0, metric.height(), 0, 0) , Qt::AlignCenter | Qt::TextWordWrap, _titleElided);
   
   //! paint year
   painter->drawText(QRect(0, 0, 140, 10), Qt::AlignTop | Qt::AlignCenter, _year);      

   //! paint playing or favorite attibute
   if(media->data[ALBUM::isPlaying].toBool())
     painter->drawPixmap(118, 0, QPixmap(":/images/media-playing.png"));
      
   if(media->data[ALBUM::isFavorite].toBool())
     painter->drawPixmap(121, 30, QPixmap(":/images/favorites-18x18.png"));
}




void AlbumGraphicItem::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
{
  Q_UNUSED(event)
  opt.state |= QStyle::State_HasFocus;
  opt.state |= QStyle::State_Selected;
  update();
}

void AlbumGraphicItem::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
{
  Q_UNUSED(event)
  opt.state &= ~QStyle::State_HasFocus;
  opt.state &= ~QStyle::State_Selected;
  update();
}

void AlbumGraphicItem::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
  Q_UNUSED(event)
}

void AlbumGraphicItem::contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
{
  Q_UNUSED(event) 
}

void AlbumGraphicItem::mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * event ) 
{
  Q_UNUSED(event)
  QMap<QString, QAction*> *actions = GLOBAL::globalActions();
  
  QVariant v; 
  v.setValue(static_cast<QGraphicsItem*>(this));
  
  (actions->value("itemDoubleClick"))->setData(v); 
  (actions->value("itemDoubleClick"))->trigger();
} 



void AlbumGraphicItem::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
{
    MediaMimeData* mimedata = new MediaMimeData();
    
    for(int i =0; i < this->media->childCount();i++) {
      if( CollectionModel::instance()->isTrackFiltered(this->media->child(i)) )
        mimedata->addMediaItem(this->media->child(i));
    }
    
    QDrag *drag = new QDrag(event->widget());
    drag->setMimeData(mimedata);
    
    //! set pixmap
    QPixmap pix;
    QPixmapCache::find(media->data[ALBUM::coverpath].toString(), pix);
    if(!pix.isNull())
      drag->setPixmap(pix);
    
    drag->start();
}




/*
********************************************************************************
*                                                                              *
*    AlbumGenreGraphicItem                                                     *
*                                                                              * 
********************************************************************************
*/
AlbumGenreGraphicItem::AlbumGenreGraphicItem()
{

}


void AlbumGenreGraphicItem::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
{
    MediaMimeData* mimedata = new MediaMimeData();
    
    for(int i =0; i < this->media->childCount();i++) 
    {
      MediaItem* trackItem = this->media->child(i);
      if(!CollectionModel::instance()->isTrackFiltered(this->media->child(i)) ) continue;
      if(trackItem->data[TRACK::genre].toString() == _genre)
        mimedata->addMediaItem(trackItem);
    }
    
    QDrag *drag = new QDrag(event->widget());
    drag->setMimeData(mimedata);
    
    //! set pixmap
    QPixmap pix;
    QPixmapCache::find(media->data[ALBUM::coverpath].toString(), pix);
    if(!pix.isNull())
      drag->setPixmap(pix);
    
    drag->start();  
}


/*
********************************************************************************
*                                                                              *
*    ArtistGraphicItem                                                         *
*                                                                              * 
********************************************************************************
*/
ArtistGraphicItem::ArtistGraphicItem()
{
   setAcceptsHoverEvents(true);
   setAcceptDrops(true);
   setFlag(QGraphicsItem::ItemIsSelectable, false);
   setFlag(QGraphicsItem::ItemIsMovable, false);
   setFlag(QGraphicsItem::ItemIgnoresTransformations, true);
   setCursor(Qt::OpenHandCursor);  

   //! option configuration
   QStyle *style =  QApplication::style();
   opt.widget = 0;
   opt.palette = QApplication::palette();
   opt.font = QApplication::font();
   opt.fontMetrics = QFontMetrics(opt.font);
    
   opt.showDecorationSelected = style->styleHint(QStyle::SH_ItemView_ShowDecorationSelected);
   int pm = style->pixelMetric(QStyle::PM_IconViewIconSize);
    
   opt.decorationSize = QSize(pm, pm);
   opt.decorationPosition = QStyleOptionViewItem::Top;
   opt.displayAlignment = Qt::AlignCenter;
   
   opt.locale.setNumberOptions(QLocale::OmitGroupSeparator);
   opt.state |= QStyle::State_Active;
   opt.state |= QStyle::State_Enabled;
}

QRectF ArtistGraphicItem::boundingRect() const
{
    return QRectF(0, 0, 140, 150);
}



void ArtistGraphicItem::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget)
{
Q_UNUSED(option)
   //! get data 
   QString      _name          = media->data[ARTIST::name].toString();
   QStringList  _coverPathList = media->data[ARTIST::coverpathlist].toStringList();
   
   QPixmap basePixmap = QPixmap( 140, 150 );
   {
     basePixmap.fill( Qt::transparent );
     QPainter pt( &basePixmap );
     
     //! Draw frame for State_HasFocus item
     QStyle *style = widget ? widget->style() : QApplication::style();
     opt.rect = boundingRect().toRect().adjusted(0,0,0,0);  
     style->drawControl(QStyle::CE_ItemViewItem, &opt, &pt, widget);   
     
     //! affichage des covers
     for (int i = 0; i <_coverPathList.size(); i++) {
              //! TEST 0.0.45c
              QPixmap pix;
              if(!QPixmapCache::find(_coverPathList[i], pix)) {
                pix = QPixmap(UTIL::CONFIGDIR + "/albums/" + _coverPathList[i]);
                if(pix.isNull()) {
                  pix = QPixmap(":/images/emptycoverblue110x110.png");
                }
                else {
                  QPixmapCache::insert(_coverPathList[i], pix);
                }
              }
              //! fin TEST 0.0.45c



      
         pix = pix.scaled(QSize(85,85), Qt::KeepAspectRatio, Qt::SmoothTransformation);
         qreal angle =  15*i;

         QTransform transform = QTransform().rotate(angle, Qt::ZAxis).translate(pix.width()/2,0);
         pix = pix.transformed(transform, Qt::SmoothTransformation); 
      
         pt.drawPixmap(boundingRect().toRect().adjusted((140 -pix.width())/2,
							   (150 -pix.height())/2 -15,
							   -(140 -pix.width())/2, 
							   -(150-pix.height())/2 -15)
							   , pix);
     }
     pt.end();
   }
   painter->drawPixmap(boundingRect().toRect(), basePixmap);
  
   //! manual graphics shadow (Qgraphics Effect cause refresh pb)
   drawShadow(painter,QPointF(0,0),basePixmap,boundingRect());  
  
   //! affichage nom de l'artist
   QColor color = opt.state & QStyle::State_HasFocus ? QColor("white") : QColor(0x315996);
   painter->setPen(color);
   painter->setFont(QFont("Arial", 8, QFont::Normal));
    
   QRect textBox = boundingRect().toRect().adjusted(0, 100, 0, 0);
   QFontMetrics fontmetric(painter->font());
   QRect fontrect = fontmetric.boundingRect (textBox,  Qt::AlignCenter | Qt::TextWordWrap, _name, 0, 0 );

   painter->drawText(textBox.adjusted(0, fontrect.height(), 0, 0), Qt::AlignCenter | Qt::TextWordWrap, _name);

   //! affichage des attributs playing ou favorite
   if(media->data[ARTIST::isPlaying].toBool())
     painter->drawPixmap(118, 0, QPixmap(":/images/media-playing.png"));
      
   if(media->data[ARTIST::isFavorite].toBool())
     painter->drawPixmap(121, 30, QPixmap(":/images/favorites-18x18.png"));
}




void ArtistGraphicItem::mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * event ) 
{
  Q_UNUSED(event)
  QMap<QString, QAction*> *actions = GLOBAL::globalActions();
  
  QVariant v; 
  v.setValue(static_cast<QGraphicsItem*>(this));
  
  (actions->value("itemDoubleClick"))->setData(v); 
  (actions->value("itemDoubleClick"))->trigger();  
} 

void ArtistGraphicItem::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
{
  Q_UNUSED(event)
  opt.state |= QStyle::State_HasFocus;
  opt.state |= QStyle::State_Selected;
  update(); 
}

void ArtistGraphicItem::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
{
  Q_UNUSED(event)
  opt.state &= ~QStyle::State_HasFocus;
  opt.state &= ~QStyle::State_Selected;
  update();
}

void ArtistGraphicItem::contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
{
  Q_UNUSED(event)
}


void ArtistGraphicItem::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
  Q_UNUSED(event)
}

void ArtistGraphicItem::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
{
    MediaMimeData* mimedata = new MediaMimeData();
    
    for(int i =0; i < this->media->childCount();i++) {
      MediaItem* albumItem = media->child(i);
      for(int j =0; j < albumItem->childCount();j++) 
      {
        if(!CollectionModel::instance()->isTrackFiltered(albumItem->child(j)) ) continue;
        mimedata->addMediaItem(albumItem->child(j));
      }
    }
    
    QDrag *drag = new QDrag(event->widget());
    drag->setMimeData(mimedata);
    
    //! set pixmap
    QPixmap basePixmap = QPixmap( 140, 150 );
    {
     basePixmap.fill( Qt::transparent );
     QPainter pt( &basePixmap );
     for (int i = 0; i < media->data[ARTIST::coverpathlist].toStringList().size(); i++) 
     {
       QPixmap pix;
       QPixmapCache::find(media->data[ARTIST::coverpathlist].toStringList()[i], pix);
       if(pix.isNull()) continue;
       
       pix = pix.scaled(QSize(85,85), Qt::KeepAspectRatio, Qt::SmoothTransformation);
       qreal angle =  15*i;
       
       QTransform transform = QTransform().rotate(angle, Qt::ZAxis).translate(pix.width()/2,0);
       pix = pix.transformed(transform, Qt::SmoothTransformation);
       
       pt.drawPixmap(boundingRect().toRect().adjusted((140 -pix.width())/2,
						      (150 -pix.height())/2 -15,
						      -(140 -pix.width())/2, 
						      -(150-pix.height())/2 -15)
                                                      , pix);
     }
    }
    drag->setPixmap(basePixmap);
    
    drag->start();  
}

/*
********************************************************************************
*                                                                              *
*    TrackGraphicWidget                                                        *
*                                                                              * 
********************************************************************************
*/
TrackGraphicItem::TrackGraphicItem()
{
   setAcceptsHoverEvents(true);
   setAcceptDrops(true);
   setFlag(QGraphicsItem::ItemIsSelectable, false);
   setFlag(QGraphicsItem::ItemIsMovable, false);
   setFlag(QGraphicsItem::ItemIgnoresTransformations, true);  
   setCursor(Qt::OpenHandCursor);
   
   //! option configuration
   QStyle *style =  QApplication::style();
   opt.widget = 0;
   opt.palette = QApplication::palette();
   opt.font = QApplication::font();
   opt.fontMetrics = QFontMetrics(opt.font);
    
   opt.showDecorationSelected = style->styleHint(QStyle::SH_ItemView_ShowDecorationSelected);
   int pm = style->pixelMetric(QStyle::PM_ListViewIconSize);
    
   opt.decorationSize = QSize(pm, pm);
   
   opt.decorationPosition = QStyleOptionViewItem::Left;
   opt.decorationAlignment = Qt::AlignCenter;
   opt.displayAlignment = Qt::AlignLeft|Qt::AlignVCenter;
   
   opt.locale.setNumberOptions(QLocale::OmitGroupSeparator);
   opt.state |= QStyle::State_Active;
   opt.state |= QStyle::State_Enabled;   
}

QRectF TrackGraphicItem::boundingRect() const
{
    return QRectF(0, 0, 450, 22);
}

void TrackGraphicItem::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget)
{
Q_UNUSED(option)
   //! get data 
   bool isTrack = (media->type == TYPE_TRACK) ? true : false;
   QString _tracknumber = isTrack ? media->data[TRACK::num].toString() : "" ;
   QString _trackname   = isTrack ? media->data[TRACK::title].toString() : media->data[STREAM::name].toString();
   bool isPlaying       = isTrack ? media->data[TRACK::isPlaying].toBool() : media->data[STREAM::isPlaying].toBool();
   bool isBroken        = isTrack ? media->data[TRACK::isBroken].toBool() :  media->data[STREAM::isBroken].toBool();

   QPixmap pixTemp(QSize(450,22));
   {
      pixTemp.fill(Qt::transparent);
      QPainter p;
      p.begin(&pixTemp);
      
      //! Draw frame for State_HasFocus item
      QStyle *style = widget ? widget->style() : QApplication::style();
      opt.rect = boundingRect().toRect();
      style->drawControl(QStyle::CE_ItemViewItem, &opt, &p, widget);
      
      
      //! paint track number & title
      QColor color = opt.state & QStyle::State_HasFocus ? QColor("white") : QColor(0x315996);
      p.setPen(color);
      p.setFont(QFont("Arial", 8, QFont::Normal));

      p.drawText(QRect(30, 0, 30, 22), Qt::AlignLeft | Qt::AlignVCenter, _tracknumber);
      QFontMetrics metric( p.font());
      QString _tracknameElided = metric.elidedText ( _trackname, Qt::ElideRight, 390);
      p.drawText(QRect(60, 0, 390, 22), Qt::AlignLeft | Qt::AlignVCenter,_tracknameElided);
	
      //! paint activated item
      if(isPlaying)
        p.drawPixmap(0, 0,QPixmap(":/images/media-playing.png"));
      else if(isBroken)
        p.drawPixmap(0, 0, QPixmap(":/images/media-broken.png"));
	
      p.end();
   }
   painter->drawPixmap(0, 0, pixTemp);   
}

void TrackGraphicItem::mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * event ) 
{
  Q_UNUSED(event)
  QMap<QString, QAction*> *actions = GLOBAL::globalActions();
  
  QVariant v; 
  v.setValue(static_cast<QGraphicsItem*>(this));
  
  (actions->value("itemDoubleClick"))->setData(v); 
  (actions->value("itemDoubleClick"))->trigger();  
} 


void TrackGraphicItem::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
{
  Q_UNUSED(event)
  opt.state |= QStyle::State_HasFocus;
  opt.state |= QStyle::State_Selected;
  update();
}

void TrackGraphicItem::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
{
  Q_UNUSED(event)
  opt.state &= ~QStyle::State_HasFocus;
  opt.state &= ~QStyle::State_Selected;
  update();
}

void TrackGraphicItem::contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
{
  Q_UNUSED(event)
}


void TrackGraphicItem::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
  Q_UNUSED(event)
}

void TrackGraphicItem::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
{
    MediaMimeData* mimedata = new MediaMimeData();
    mimedata->addMediaItem(this->media);
    
    QDrag *drag = new QDrag(event->widget());
    drag->setMimeData(mimedata);
    drag->start();    
}

/*
********************************************************************************
*                                                                              *
*    TrackGraphicItemFull                                                      *
*                                                                              * 
********************************************************************************
*/
TrackGraphicItemFull::TrackGraphicItemFull()
{
   
}


void TrackGraphicItemFull::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget)
{
Q_UNUSED(option)
   //! get data 
   bool isTrack = (media->type == TYPE_TRACK) ? true : false;
   
   if(!isTrack) {
     TrackGraphicItem::paint(painter, option, widget);
     return;
   }
   
   QString _tracknumber = media->data[TRACK::num].toString();
   QString _trackname   = media->data[TRACK::title].toString();
   QString _albumame    = media->data[TRACK::album].toString();
   QString _duree       = MEDIA::durationTextFromInt(media->data[TRACK::duration].toInt());

   bool isPlaying       = media->data[TRACK::isPlaying].toBool();
   bool isBroken        = media->data[TRACK::isBroken].toBool();

   QPixmap pixTemp(QSize(450,22));
   {
      pixTemp.fill(Qt::transparent);
      QPainter p;
      p.begin(&pixTemp);
      
      //! Draw frame for State_HasFocus item
      QStyle *style = widget ? widget->style() : QApplication::style();
      opt.rect = boundingRect().toRect();
      style->drawControl(QStyle::CE_ItemViewItem, &opt, &p, widget);
      
      
      //! paint track number 
      QColor color = opt.state & QStyle::State_HasFocus ? QColor("white") : QColor(0x315996);
      p.setPen(color);
      p.setFont(QFont("Arial", 8, QFont::Normal));

      p.drawText(QRect(30, 0, 30, 22), Qt::AlignLeft | Qt::AlignVCenter, _tracknumber);
      
      //! paint track name
      QFontMetrics metric( p.font());
      QString _tracknameElided = metric.elidedText ( _trackname, Qt::ElideRight, 200);
      p.drawText(QRect(60, 0, 200, 22), Qt::AlignLeft | Qt::AlignVCenter,_tracknameElided);

      //! paint album name
      QString _albumameElided = metric.elidedText ( _albumame, Qt::ElideRight, 140);
      p.drawText(QRect(260, 0, 140, 22), Qt::AlignLeft | Qt::AlignVCenter,_albumameElided);

      //! paint track duration
      QString _dureeElided = metric.elidedText ( _duree, Qt::ElideRight, 50);
      p.drawText(QRect(400, 0, 50, 22), Qt::AlignLeft | Qt::AlignVCenter,_dureeElided);

      
      //! paint activated item
      if(isPlaying)
        p.drawPixmap(0, 0,QPixmap(":/images/media-playing.png"));
      else if(isBroken)
        p.drawPixmap(0, 0, QPixmap(":/images/media-broken.png"));
	
      p.end();
   }
   painter->drawPixmap(0, 0, pixTemp);   
}


/*
********************************************************************************
*                                                                              *
*    HistoTrackGraphicWidget                                                   *
*                                                                              * 
********************************************************************************
*/
HistoTrackGraphicWidget::HistoTrackGraphicWidget()
{
   
}


void HistoTrackGraphicWidget::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget)
{
Q_UNUSED(option)
   //! get data 
   bool isTrack = (media->type == TYPE_TRACK) ? true : false;
   QString _trackname   = isTrack ? media->data[TRACK::name].toString() : media->data[STREAM::name].toString();
   bool isPlaying       = isTrack ? media->data[TRACK::isPlaying].toBool() : media->data[STREAM::isPlaying].toBool();
   bool isBroken        = isTrack ? media->data[TRACK::isBroken].toBool() :  media->data[STREAM::isBroken].toBool();

   QPixmap pixTemp(QSize(450,22));
   {
      pixTemp.fill(Qt::transparent);
      QPainter p;
      p.begin(&pixTemp);
      
      //! Draw frame for State_HasFocus item
      QStyle *style = widget ? widget->style() : QApplication::style();
      opt.rect = boundingRect().toRect();
      style->drawControl(QStyle::CE_ItemViewItem, &opt, &p, widget);
      
      //! paint track number & title
      QColor color = opt.state & QStyle::State_HasFocus ? QColor("white") : QColor(0x315996);
      p.setPen(color);
      p.setFont(QFont("Arial", 8, QFont::Normal));

      QFontMetrics metric( p.font());
      QString _tracknameElided = metric.elidedText ( _trackname, Qt::ElideRight, 390);
      p.drawText(QRect(60, 0, 390, 22), Qt::AlignLeft | Qt::AlignVCenter,_tracknameElided);
	
      //! paint activated item
      if(isPlaying)
        p.drawPixmap(0, 0,QPixmap(":/images/media-playing.png"));
      else if(isBroken)
        p.drawPixmap(0, 0, QPixmap(":/images/media-broken.png"));
	
      p.end();
   }
   painter->drawPixmap(0, 0, pixTemp);   
}


/*
********************************************************************************
*                                                                              *
*    PlaylistGraphicItem                                                       *
*                                                                              * 
********************************************************************************
*/
PlaylistGraphicItem::PlaylistGraphicItem()
{
   setAcceptsHoverEvents(true);
   setAcceptDrops(true);
   setFlag(QGraphicsItem::ItemIsSelectable, false);
   setFlag(QGraphicsItem::ItemIsMovable, false);
   setFlag(QGraphicsItem::ItemIgnoresTransformations, true);
   setCursor(Qt::OpenHandCursor);
   
   //! option configuration
   QStyle *style =  QApplication::style();
   opt.widget = 0;
   opt.palette = QApplication::palette();
   opt.font = QApplication::font();
   opt.fontMetrics = QFontMetrics(opt.font);
    
   opt.showDecorationSelected = style->styleHint(QStyle::SH_ItemView_ShowDecorationSelected);
   int pm = style->pixelMetric(QStyle::PM_IconViewIconSize);
    
   opt.decorationSize = QSize(pm, pm);
   opt.decorationPosition = QStyleOptionViewItem::Top;
   opt.displayAlignment = Qt::AlignCenter;
   
   opt.locale.setNumberOptions(QLocale::OmitGroupSeparator);
   opt.state |= QStyle::State_Active;
   opt.state |= QStyle::State_Enabled;
}

QRectF PlaylistGraphicItem::boundingRect() const
{
    return QRectF(0, 0, 140, 150);
}


void PlaylistGraphicItem::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget)
{
Q_UNUSED(option)
   //! get data
   QString _name         =  media->data[PLAYLIST::name].toString();
   QDateTime date        = QDateTime::fromTime_t(media->data[PLAYLIST::date].toUInt());
   QString icon          = media->data[PLAYLIST::icon].toString();
   QString _playlistDate = date.toString("dd.MM.yyyy");
   
   QPixmap pixTemp(QSize(140,150));
   {
      pixTemp.fill(Qt::transparent);
      QPainter p;
      p.begin(&pixTemp);

      //! Draw frame for State_HasFocus item
      QStyle *style = widget ? widget->style() : QApplication::style();
      opt.rect = boundingRect().toRect().adjusted(0,0,0,0);
      style->drawControl(QStyle::CE_ItemViewItem, &opt, &p, widget);
   
      QColor color = opt.state & QStyle::State_HasFocus ? QColor("white") : QColor(0x315996);
      p.setPen(color);
      p.setFont(QFont("Arial", 8, QFont::Normal));

      //! paint media playlist mime
      p.drawPixmap(opt.rect.adjusted(15,15,-15,-25),QPixmap(icon));

      //! paint playlist date
      if( media->data[PLAYLIST::type].toInt() == T_FILE || 
          media->data[PLAYLIST::type].toInt() == T_DATABASE) {
        p.drawText(boundingRect().toRect().adjusted(0, 0, 0, -120), Qt::AlignCenter, _playlistDate);
      }
      
      //! paint playlist name
      QRect textBox = boundingRect().toRect().adjusted(0, 100, 0,0);
      QFontMetrics fontmetric(p.font());
      QRect fontrect = fontmetric.boundingRect (textBox,  Qt::AlignCenter | Qt::TextWordWrap, _name, 0, 0 );
      
      p.drawText(textBox.adjusted(0, fontrect.height(), 0, 0), Qt::AlignCenter | Qt::TextWordWrap, _name);

      //! paint activated item
      if(media->data[PLAYLIST::isPlaying].toBool())
        p.drawPixmap(112, 20, QPixmap(":/images/media-playing.png"));

      //! paint favorites item
      if(media->data[PLAYLIST::isFavorite].toBool())  
        p.drawPixmap(114, 50, QPixmap(":/images/favorites-18x18.png"));

      //! paint media playlist type (internal/user)
      if(media->data[PLAYLIST::type].toInt() == T_DATABASE) 
        p.drawPixmap(114, 80, QPixmap(":/images/save-18x18.png"));
      
      p.end();
   }
   painter->drawPixmap(0, 0, pixTemp);  
}


void PlaylistGraphicItem::mouseDoubleClickEvent ( QGraphicsSceneMouseEvent *event ) 
{
  Q_UNUSED(event)
  QMap<QString, QAction*> *actions = GLOBAL::globalActions();
  
  QVariant v; 
  v.setValue(static_cast<QGraphicsItem*>(this));
  
  (actions->value("itemDoubleClick"))->setData(v); 
  (actions->value("itemDoubleClick"))->trigger();  
} 


void PlaylistGraphicItem::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
{
  Q_UNUSED(event)
  opt.state |= QStyle::State_HasFocus;
  opt.state |= QStyle::State_Selected;
  update();
}

void PlaylistGraphicItem::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
{
  Q_UNUSED(event)
  opt.state &= ~QStyle::State_HasFocus;
  opt.state &= ~QStyle::State_Selected;
  update();  
}

void PlaylistGraphicItem::contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
{
  Q_UNUSED(event)
}

void PlaylistGraphicItem::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
  Q_UNUSED(event)
}

void PlaylistGraphicItem::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
{
    MediaMimeData* mimedata = new MediaMimeData();
    
    
    //! rechercher dynamique pour les smart playlists
    if(media->data[PLAYLIST::type].toInt() == T_SMART_AUTO ||
       media->data[PLAYLIST::type].toInt() == T_SMART_USER)
    {
      mimedata->addMediaItemList(
           SMART_PLAYLIST::mediaItem( (T_PLAYLIST)media->data[PLAYLIST::type].toInt(),
                                      media->data[PLAYLIST::name].toString(),
                                      media->data[PLAYLIST::rules].toString()
                                      )
       );
    }
    //! playliste normale -> lecture dans el modele
    else { 
      
     for(int i =0; i < this->media->childCount();i++) {
       if(!PlsFileModel::instance()->isUrlFiltered(this->media->child(i)) ) continue;
       mimedata->addMediaItem(this->media->child(i));
      }
      
    }
    
    QDrag *drag = new QDrag(event->widget());
    drag->setMimeData(mimedata);
    
    //! set pixmap
    drag->setPixmap( QPixmap(media->data[PLAYLIST::icon].toString()) );

       
    drag->start();
}

/*
********************************************************************************
*                                                                              *
*    StreamGraphicItem                                                         *
*                                                                              * 
********************************************************************************
*/
StreamGraphicItem::StreamGraphicItem()
{
   setAcceptsHoverEvents(true);
   setAcceptDrops(true);
   setFlag(QGraphicsItem::ItemIsSelectable, false);
   setFlag(QGraphicsItem::ItemIsMovable, false);
   setFlag(QGraphicsItem::ItemIgnoresTransformations, true);  
   setCursor(Qt::OpenHandCursor);   
   
   //! option configuration
   QStyle *style =  QApplication::style();
   opt.widget = 0;
   opt.palette = QApplication::palette();
   opt.font = QApplication::font();
   opt.fontMetrics = QFontMetrics(opt.font);
    
   opt.showDecorationSelected = style->styleHint(QStyle::SH_ItemView_ShowDecorationSelected);
   int pm = style->pixelMetric(QStyle::PM_ListViewIconSize);
    
   opt.decorationSize = QSize(pm, pm);
   
   opt.decorationPosition = QStyleOptionViewItem::Left;
   opt.decorationAlignment = Qt::AlignCenter;
   opt.displayAlignment = Qt::AlignLeft|Qt::AlignVCenter;
   
   opt.locale.setNumberOptions(QLocale::OmitGroupSeparator);
   opt.state |= QStyle::State_Active;
   opt.state |= QStyle::State_Enabled;
}

QRectF StreamGraphicItem::boundingRect() const
{
    return QRectF(0, 0, 630, 22);
}


void StreamGraphicItem::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget *widget)
{
Q_UNUSED(option)
      
   //! get Model data
   if(!media) return;
   QString _streamname =  media->data[STREAM::name].toString();
   
   QPixmap pixTemp(QSize(630,22));
   {
      pixTemp.fill(Qt::transparent);
      QPainter p;
      p.begin(&pixTemp);
      
      //! Draw frame for State_HasFocus item
      QStyle *style = widget ? widget->style() : QApplication::style();
      opt.rect = boundingRect().toRect().adjusted(20,0,0,0);
      style->drawControl(QStyle::CE_ItemViewItem, &opt, &p, widget);
      
      //! paint stream name
      QColor color = opt.state & QStyle::State_HasFocus ? QColor("white") : QColor(0x315996);
      p.setPen(color);
      p.setFont(QFont("Arial", 8, QFont::Normal));
      
      QFontMetrics metric( painter->font());
      QString streamElided = metric.elidedText ( _streamname, Qt::ElideRight, 570);
      p.drawText(QRect(60, 0, 570, 22), Qt::AlignLeft | Qt::AlignVCenter,streamElided);
      
      //! paint activated item
      if(media->data[STREAM::isPlaying].toBool())
        p.drawPixmap(20, 0,QPixmap(":/images/media-playing.png"));
      else if(media->data[STREAM::isBroken].toBool())
        p.drawPixmap(22, 1, QPixmap(":/images/media-broken.png"));
         
      p.end();
   }
   
   painter->drawPixmap(0, 0, pixTemp);
}

void StreamGraphicItem::mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * event ) 
{
  Q_UNUSED(event)
  QMap<QString, QAction*> *actions = GLOBAL::globalActions();
  
  QVariant v; 
  v.setValue(static_cast<QGraphicsItem*>(this));
  
  (actions->value("itemDoubleClick"))->setData(v); 
  (actions->value("itemDoubleClick"))->trigger();  
} 


void StreamGraphicItem::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
{
  Q_UNUSED(event)
  opt.state |= QStyle::State_HasFocus;
  opt.state |= QStyle::State_Selected;
  update();
}

void StreamGraphicItem::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
{
  Q_UNUSED(event)
  opt.state &= ~QStyle::State_HasFocus;
  opt.state &= ~QStyle::State_Selected;
  update();
}

void StreamGraphicItem::contextMenuEvent(QGraphicsSceneContextMenuEvent *event) 
{
  Q_UNUSED(event)
}

void StreamGraphicItem::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
  Q_UNUSED(event)
}


void StreamGraphicItem::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
{
  MediaMimeData* mimedata = new MediaMimeData();
  mimedata->addMediaItem(this->media);
  
  QDrag *drag = new QDrag(event->widget());
  drag->setMimeData(mimedata);
  drag->start();
}



/*
********************************************************************************
*                                                                              *
*    AlbumChartGraphicItem                                                     *
*                                                                              * 
********************************************************************************
*/
AlbumChartGraphicItem::AlbumChartGraphicItem()
{
   setAcceptsHoverEvents(true);
   setAcceptDrops(true);
   setFlag(QGraphicsItem::ItemIsSelectable, false);
   setFlag(QGraphicsItem::ItemIsMovable, false);
   setFlag(QGraphicsItem::ItemIgnoresTransformations, true);
   setCursor(Qt::OpenHandCursor);
   
   //! option configuration
   QStyle *style =  QApplication::style();
   opt.widget = 0;
   opt.palette = QApplication::palette();
   opt.font = QApplication::font();
   opt.fontMetrics = QFontMetrics(opt.font);
    
   opt.showDecorationSelected = style->styleHint(QStyle::SH_ItemView_ShowDecorationSelected);
   int pm = style->pixelMetric(QStyle::PM_IconViewIconSize);
    
   opt.decorationSize = QSize(pm, pm);
   opt.decorationPosition = QStyleOptionViewItem::Top;
   opt.displayAlignment = Qt::AlignCenter;
   
   opt.locale.setNumberOptions(QLocale::OmitGroupSeparator);
   opt.state |= QStyle::State_Active;
   opt.state |= QStyle::State_Enabled;
   _width = 630;
}


QRectF AlbumChartGraphicItem::boundingRect() const
{
    return QRectF(0, 0, _width, 70);
}

void AlbumChartGraphicItem::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget)
{
Q_UNUSED(option)
   //! get Model data 
   QString _title     =  media->data[ALBUM::name].toString();
   int     _playcount =  media->data[ALBUM::playcount].toInt();
   QString _coverPath =  media->data[ALBUM::coverpath].toString();
      
   
   double longueur = static_cast<double>(_playcount)/_maxPlaycount;
   longueur = longueur*_width;
   if(longueur -120 < 1)
     longueur = 121;
   
   //! Draw frame for State_HasFocus item
   QStyle *style = widget ? widget->style() : QApplication::style();
   opt.rect = boundingRect().toRect().adjusted(0,0,0,0);  
   style->drawControl(QStyle::CE_ItemViewItem, &opt, painter, widget);
   
   QPixmap pixTemp(QSize(70,70));
   {
      pixTemp.fill(Qt::transparent);
      QPainter p;
      p.begin(&pixTemp);
    
              //! TEST 0.0.45c
              QPixmap pix;
              if(!QPixmapCache::find(_coverPath, pix)) {
                pix = QPixmap(UTIL::CONFIGDIR + "/albums/" + _coverPath);
                if(pix.isNull()) {
                  pix = QPixmap(":/images/emptycoverblue110x110.png");
                }
                else {
                  QPixmapCache::insert(_coverPath, pix);
                }
              }
              //! fin TEST 0.0.45c
      pix = pix.scaled(QSize(60,60), Qt::KeepAspectRatio, Qt::SmoothTransformation);
      p.drawPixmap(5,5, pix);

    p.end();
   }
   
   painter->drawPixmap(0, 0, pixTemp);
 
  
   //! paint title
   QColor color = opt.state & QStyle::State_HasFocus ? QColor("white") : QColor(0x315996);
   painter->setPen(color);
   painter->setFont(QFont("Arial", 8, QFont::Normal));
    
   QFontMetrics metric( painter->font());
   QString elided = metric.elidedText ( _title, Qt::ElideRight, 200);
   painter->drawText(QRect (80,10,200,20), Qt::AlignLeft | Qt::AlignVCenter,elided);

   //! paint playcount
   painter->setFont(QFont("Arial", 8, QFont::Bold));
   painter->drawText(QRect(longueur-30, 25, 20, 35), Qt::AlignTop | Qt::AlignCenter, QString::number(_playcount) );

   //! paint bar chart
   painter->setPen(QPen(QColor(50, 200, 200, 90), 0.5));
   painter->setBrush(QBrush( QColor( 50, 200, 200, 65) ,Qt::SolidPattern));
   painter->drawRect( 80, 25, longueur-120, 25 );

   //! paint playing or favorite attibute
   if(media->data[ALBUM::isPlaying].toBool())
     painter->drawPixmap(56, 5, QPixmap(":/images/media-playing.png"));
      
   if(media->data[ALBUM::isFavorite].toBool())
     painter->drawPixmap(60, 30, QPixmap(":/images/favorites-18x18.png"));
}




void AlbumChartGraphicItem::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
{
  Q_UNUSED(event)
  opt.state |= QStyle::State_HasFocus;
  opt.state |= QStyle::State_Selected;
  update();
}

void AlbumChartGraphicItem::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
{
  Q_UNUSED(event)
  opt.state &= ~QStyle::State_HasFocus;
  opt.state &= ~QStyle::State_Selected;
  update();
}

void AlbumChartGraphicItem::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
  Q_UNUSED(event)
}

void AlbumChartGraphicItem::contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
{
  Q_UNUSED(event) 
}

void AlbumChartGraphicItem::mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * event ) 
{
  Q_UNUSED(event)
  QMap<QString, QAction*> *actions = GLOBAL::globalActions();
  
  QVariant v; 
  v.setValue(static_cast<QGraphicsItem*>(this));
  
  (actions->value("itemDoubleClick"))->setData(v); 
  (actions->value("itemDoubleClick"))->trigger();
} 



void AlbumChartGraphicItem::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
{
    MediaMimeData* mimedata = new MediaMimeData();
    
    for(int i =0; i < this->media->childCount();i++) {
      if( CollectionModel::instance()->isTrackFiltered(this->media->child(i)) )
        mimedata->addMediaItem(this->media->child(i));
    }
    
    QDrag *drag = new QDrag(event->widget());
    drag->setMimeData(mimedata);
    
    //! set pixmap
    QPixmap pix;
    QPixmapCache::find(media->data[ALBUM::coverpath].toString(), pix);
    if(!pix.isNull())
      drag->setPixmap(pix);
    
    drag->start();
}


/*
********************************************************************************
*                                                                              *
*    ArtistChartGraphicItem                                                     *
*                                                                              * 
********************************************************************************
*/
ArtistChartGraphicItem::ArtistChartGraphicItem()
{
   setAcceptsHoverEvents(true);
   setAcceptDrops(true);
   setFlag(QGraphicsItem::ItemIsSelectable, false);
   setFlag(QGraphicsItem::ItemIsMovable, false);
   setFlag(QGraphicsItem::ItemIgnoresTransformations, true);
   setCursor(Qt::OpenHandCursor);  

   //! option configuration
   QStyle *style =  QApplication::style();
   opt.widget = 0;
   opt.palette = QApplication::palette();
   opt.font = QApplication::font();
   opt.fontMetrics = QFontMetrics(opt.font);
    
   opt.showDecorationSelected = style->styleHint(QStyle::SH_ItemView_ShowDecorationSelected);
   int pm = style->pixelMetric(QStyle::PM_IconViewIconSize);
    
   opt.decorationSize = QSize(pm, pm);
   opt.decorationPosition = QStyleOptionViewItem::Top;
   opt.displayAlignment = Qt::AlignCenter;
   
   opt.locale.setNumberOptions(QLocale::OmitGroupSeparator);
   opt.state |= QStyle::State_Active;
   opt.state |= QStyle::State_Enabled;
   _width = 630;
}

QRectF ArtistChartGraphicItem::boundingRect() const
{
    return QRectF(0, 0, _width, 70);
}



void ArtistChartGraphicItem::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget)
{
Q_UNUSED(option)
   //! get data 
   QString      _name          = media->data[ARTIST::name].toString();
   QStringList  _coverPathList = media->data[ARTIST::coverpathlist].toStringList();
   int     _playcount          = media->data[ARTIST::playcount].toInt();

   double longueur = static_cast<double>(_playcount)/_maxPlaycount;
   longueur = longueur*_width;
   if(longueur -120 < 1)
     longueur = 121;
   

   //! Draw frame for State_HasFocus item
   QStyle *style = widget ? widget->style() : QApplication::style();
   opt.rect = boundingRect().toRect().adjusted(0,0,0,0);  
   style->drawControl(QStyle::CE_ItemViewItem, &opt, painter, widget);
     
   QPixmap basePixmap = QPixmap( 70, 70 );
   {
     basePixmap.fill( Qt::transparent );
     QPainter pt( &basePixmap );
     
     //! affichage des covers
     for (int i = 0; i <_coverPathList.size(); i++) {
              //! TEST 0.0.45c
              QPixmap pix;
              if(!QPixmapCache::find(_coverPathList[i], pix)) {
                pix = QPixmap(UTIL::CONFIGDIR + "/albums/" + _coverPathList[i]);
                if(pix.isNull()) {
                  pix = QPixmap(":/images/emptycoverblue110x110.png");
                }
                else {
                  QPixmapCache::insert(_coverPathList[i], pix);
                }
              }
              //! fin TEST 0.0.45c

         pix = pix.scaled(QSize(50,50), Qt::KeepAspectRatio, Qt::SmoothTransformation);
         qreal angle =  15*i;

         QTransform transform = QTransform().rotate(angle, Qt::ZAxis).translate(pix.width()/2,0);
         pix = pix.transformed(transform, Qt::SmoothTransformation); 

         pt.drawPixmap(QRect(0,0,70,70).adjusted((70 -pix.width())/2 ,
						 (70 -pix.height())/2 ,
						 -(70 -pix.width())/2, 
						 -(70-pix.height())/2 )
	                                          , pix);
         
     }
     pt.end();
   }
   painter->drawPixmap(0,0, basePixmap);
  
   //! paint artiste name
   QColor color = opt.state & QStyle::State_HasFocus ? QColor("white") : QColor(0x315996);
   painter->setPen(color);
   painter->setFont(QFont("Arial", 8, QFont::Normal));
    
   QFontMetrics metric( painter->font());
   QString elided = metric.elidedText ( _name, Qt::ElideRight, 200);
   painter->drawText(QRect (80,10,200,20), Qt::AlignLeft | Qt::AlignVCenter,elided);
   
   //! paint playcount
   painter->setFont(QFont("Arial", 8, QFont::Bold));
   painter->drawText(QRect(longueur-30, 25, 20, 35), Qt::AlignTop | Qt::AlignCenter, QString::number(_playcount) );

   //! paint bar chart
   painter->setPen(QPen(QColor(50, 200, 200, 90), 0.5));
   painter->setBrush(QBrush( QColor( 50, 200, 200, 65) ,Qt::SolidPattern));
   painter->drawRect( 80, 25, longueur-120, 25 );
   
   //! affichage des attributs playing ou favorite
   if(media->data[ARTIST::isPlaying].toBool())
     painter->drawPixmap(58, 0, QPixmap(":/images/media-playing.png"));
      
   if(media->data[ARTIST::isFavorite].toBool())
     painter->drawPixmap(60, 30, QPixmap(":/images/favorites-18x18.png"));
}




void ArtistChartGraphicItem::mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * event ) 
{
  Q_UNUSED(event)
  QMap<QString, QAction*> *actions = GLOBAL::globalActions();
  
  QVariant v; 
  v.setValue(static_cast<QGraphicsItem*>(this));
  
  (actions->value("itemDoubleClick"))->setData(v); 
  (actions->value("itemDoubleClick"))->trigger();  
} 

void ArtistChartGraphicItem::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
{
  Q_UNUSED(event)
  opt.state |= QStyle::State_HasFocus;
  opt.state |= QStyle::State_Selected;
  update(); 
}

void ArtistChartGraphicItem::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
{
  Q_UNUSED(event)
  opt.state &= ~QStyle::State_HasFocus;
  opt.state &= ~QStyle::State_Selected;
  update();
}

void ArtistChartGraphicItem::contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
{
  Q_UNUSED(event)
}


void ArtistChartGraphicItem::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
  Q_UNUSED(event)
}

void ArtistChartGraphicItem::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
{
    MediaMimeData* mimedata = new MediaMimeData();
    
    for(int i =0; i < this->media->childCount();i++) {
      MediaItem* albumItem = media->child(i);
      for(int j =0; j < albumItem->childCount();j++) 
      {
        if(!CollectionModel::instance()->isTrackFiltered(albumItem->child(j)) ) continue;
        mimedata->addMediaItem(albumItem->child(j));
      }
    }
    
    QDrag *drag = new QDrag(event->widget());
    drag->setMimeData(mimedata);
    drag->start();  
}