/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _GRAPHICS_ITEM_H_
#define _GRAPHICS_ITEM_H_

#include <QtGui>

class MediaItem;


namespace GraphicsItem {
  enum GraphicsType {
      AlbumType             = QGraphicsItem::UserType + 1,
      ArtistType            = QGraphicsItem::UserType + 2,
      AlbumGenreType        = QGraphicsItem::UserType + 3,
      TrackType             = QGraphicsItem::UserType + 4,
      PlaylistType          = QGraphicsItem::UserType + 5,
      StreamType            = QGraphicsItem::UserType + 6
  };
}

/*******************************************************************************
    InfoGraphicItem
*******************************************************************************/
class InfoGraphicItem : public QGraphicsItem
{
  public:
    InfoGraphicItem();
    QString     _text;
    int         _width;

  protected:
    QRectF boundingRect() const;
    void paint(QPainter *p, const QStyleOptionGraphicsItem *, QWidget *);
};

/*******************************************************************************
    CategorieGraphicItem
*******************************************************************************/
class CategorieGraphicItem : public QGraphicsItem
{
  public:
    CategorieGraphicItem();
    QString     _catName;
    int         _width;
    
  protected:
    QRectF boundingRect() const;
    void paint(QPainter *p, const QStyleOptionGraphicsItem *, QWidget *);
};



/*******************************************************************************
    AlbumGraphicItem
*******************************************************************************/
class AlbumGraphicItem : public QGraphicsItem
{
  public:
    AlbumGraphicItem();
    MediaItem             *media;
    
    int type() const { return GraphicsItem::AlbumType; }

  protected:
    // Implement QGraphicsItem method
    QRectF boundingRect() const;
    void paint(QPainter *p, const QStyleOptionGraphicsItem *, QWidget *);

    void hoverEnterEvent(QGraphicsSceneHoverEvent *event);
    void hoverLeaveEvent(QGraphicsSceneHoverEvent *event);
    void mousePressEvent(QGraphicsSceneMouseEvent *event);
    void mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * event );
    void mouseMoveEvent(QGraphicsSceneMouseEvent *event);
    void contextMenuEvent(QGraphicsSceneContextMenuEvent *event);
  public:
     QStyleOptionViewItemV4 opt;    
};

/*******************************************************************************
    AlbumGenreGraphicItem
*******************************************************************************/
class AlbumGenreGraphicItem : public AlbumGraphicItem
{
  public:
    AlbumGenreGraphicItem();
    QString                 _genre;
    int type() const { return GraphicsItem::AlbumGenreType; }
    
  protected:
    // Implement QGraphicsItem method
    void mouseMoveEvent(QGraphicsSceneMouseEvent *event);
};

/*******************************************************************************
    AlbumChartGraphicItem
*******************************************************************************/
class AlbumChartGraphicItem : public QGraphicsItem
{
  public:
    AlbumChartGraphicItem();
    MediaItem             *media;
    int         _width;
    int         _maxPlaycount;

    int type() const { return GraphicsItem::AlbumType; }

  protected:
    // Implement QGraphicsItem method
    QRectF boundingRect() const;
    void paint(QPainter *p, const QStyleOptionGraphicsItem *, QWidget *);

    void hoverEnterEvent(QGraphicsSceneHoverEvent *event);
    void hoverLeaveEvent(QGraphicsSceneHoverEvent *event);
    void mousePressEvent(QGraphicsSceneMouseEvent *event);
    void mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * event );
    void mouseMoveEvent(QGraphicsSceneMouseEvent *event);
    void contextMenuEvent(QGraphicsSceneContextMenuEvent *event);
  public:
     QStyleOptionViewItemV4 opt;    
};



/*******************************************************************************
    ArtistGraphicItem 
*******************************************************************************/
class ArtistGraphicItem : public QGraphicsItem
{
  public:
    ArtistGraphicItem();
    MediaItem             *media;
    int type() const { return GraphicsItem::ArtistType; }
    
  protected:
    // Implement QGraphicsItem method
    QRectF boundingRect() const;
    void paint(QPainter *p, const QStyleOptionGraphicsItem *, QWidget *);

    void hoverEnterEvent(QGraphicsSceneHoverEvent *event);
    void hoverLeaveEvent(QGraphicsSceneHoverEvent *event);
    void mousePressEvent(QGraphicsSceneMouseEvent *event);
    void mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * event );
    void contextMenuEvent(QGraphicsSceneContextMenuEvent *event);
    void mouseMoveEvent(QGraphicsSceneMouseEvent *event);
  private:
     QStyleOptionViewItemV4 opt;    
};

/*******************************************************************************
    ArtistChartGraphicItem
*******************************************************************************/
class ArtistChartGraphicItem : public QGraphicsItem
{
  public:
    ArtistChartGraphicItem();
    MediaItem             *media;
    int         _width;
    int         _maxPlaycount;

    int type() const { return GraphicsItem::ArtistType; }

  protected:
    // Implement QGraphicsItem method
    QRectF boundingRect() const;
    void paint(QPainter *p, const QStyleOptionGraphicsItem *, QWidget *);

    void hoverEnterEvent(QGraphicsSceneHoverEvent *event);
    void hoverLeaveEvent(QGraphicsSceneHoverEvent *event);
    void mousePressEvent(QGraphicsSceneMouseEvent *event);
    void mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * event );
    void mouseMoveEvent(QGraphicsSceneMouseEvent *event);
    void contextMenuEvent(QGraphicsSceneContextMenuEvent *event);
  public:
     QStyleOptionViewItemV4 opt;    
};

/*******************************************************************************
    TrackGraphicWidget 
*******************************************************************************/
class TrackGraphicItem : public QGraphicsItem
{
  public:
    TrackGraphicItem();
    MediaItem             *media;
    int type() const { return GraphicsItem::TrackType; }
    
    //QStyleOptionViewItemV4 getOption() {return opt;}
    
  protected:
    // Implement QGraphicsItem method
    QRectF boundingRect() const;
    void paint(QPainter *p, const QStyleOptionGraphicsItem *, QWidget *);

    void hoverEnterEvent(QGraphicsSceneHoverEvent *event);
    void hoverLeaveEvent(QGraphicsSceneHoverEvent *event);
    void mousePressEvent(QGraphicsSceneMouseEvent *event);
    void mouseDoubleClickEvent ( QGraphicsSceneMouseEvent *event );
    void contextMenuEvent(QGraphicsSceneContextMenuEvent *event);
    void mouseMoveEvent(QGraphicsSceneMouseEvent *event);
  public:
     QStyleOptionViewItemV4 opt;    
};

/*******************************************************************************
    TrackGraphicItemFull 
*******************************************************************************/
class TrackGraphicItemFull  : public TrackGraphicItem
{
  public:
    TrackGraphicItemFull ();
    
  protected:
    void paint(QPainter *p, const QStyleOptionGraphicsItem *, QWidget *);
};


/*******************************************************************************
    HistoTrackGraphicWidget 
*******************************************************************************/
class HistoTrackGraphicWidget  : public TrackGraphicItem
{
  public:
    HistoTrackGraphicWidget ();
    
  protected:
    void paint(QPainter *p, const QStyleOptionGraphicsItem *, QWidget *);
};
/*******************************************************************************
    PlaylistGraphicItem 
*******************************************************************************/
class PlaylistGraphicItem : public QGraphicsItem
{
  public:
    PlaylistGraphicItem();
    MediaItem             *media;
    int type() const { return GraphicsItem::PlaylistType; }
        
  protected:
    // Implement QGraphicsItem method
    QRectF boundingRect() const;
    void paint(QPainter *p, const QStyleOptionGraphicsItem *, QWidget *);

    void hoverEnterEvent(QGraphicsSceneHoverEvent *event);
    void hoverLeaveEvent(QGraphicsSceneHoverEvent *event);
    void mousePressEvent(QGraphicsSceneMouseEvent *event);
    void mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * event );
    void mouseMoveEvent(QGraphicsSceneMouseEvent *event);
    void contextMenuEvent(QGraphicsSceneContextMenuEvent *event);
  private:
     QStyleOptionViewItemV4 opt;    
};

/*******************************************************************************
    StreamGraphicItem 
*******************************************************************************/
class StreamGraphicItem : public QGraphicsItem
{
  public:
    StreamGraphicItem();
    int type() const { return GraphicsItem::StreamType; }
    MediaItem             *media;

  protected:
    // Implement QGraphicsItem method
    QRectF boundingRect() const;
    void paint(QPainter *p, const QStyleOptionGraphicsItem *, QWidget *);

    void hoverEnterEvent(QGraphicsSceneHoverEvent *e);
    void hoverLeaveEvent(QGraphicsSceneHoverEvent *e);
    void mousePressEvent(QGraphicsSceneMouseEvent *e);
    void mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * event );
    void mouseMoveEvent(QGraphicsSceneMouseEvent *event);
    void contextMenuEvent(QGraphicsSceneContextMenuEvent *event);
  private:
     QStyleOptionViewItemV4 opt;
};

#endif // _GRAPHICS_ITEM_H_
