/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "CollectionView/userchangedialog.h"
#include "Database/database.h"
#include "MediaItem/mediaitem.h"
#include "utilities.h"

#include <QSqlField>
#include <QSqlQuery>
#include <QSqlDriver>
#include <QVariant>
#include <QString>

/*******************************************************************************
    AlbumChangeDialog 
*******************************************************************************/
AlbumChangeDialog::AlbumChangeDialog(int id)
{
    _albumBDId        = id;
    _albuminfochanged = false;
  
    //! create Gui
    CreateGui();
  
    //! Connection
    QObject::connect(buttonBox, SIGNAL(accepted()), this, SLOT(on_buttonBox_accepted()));
    QObject::connect(buttonBox, SIGNAL(rejected()), this, SLOT(on_buttonBox_rejected()));

    //! Get current album Info
    Database db;
    if (db.connect("AlbumItem")) {
      QSqlQuery tracksQuery("SELECT filename,album,interpret,year FROM view_tracks WHERE albumID="+QString::number(_albumBDId)+" ORDER BY track ASC;",
                            *db.sqlDb());

      tracksQuery.next();
      if (tracksQuery.isValid()) {
        _album  = tracksQuery.value(1).toString();
        _artist = tracksQuery.value(2).toString();
        _year   = tracksQuery.value(3).toString();
        lineEdit->setText(_year);   
        lineEdit_2->setText(_artist); 
        lineEdit_3->setText(_album); 
      }
    }
    else  {
      lineEdit->setText("Connection to Database failed");
      lineEdit_2->setText("Connection to Database failed");
      lineEdit_3->setText("Connection to Database failed");
      buttonBox->setStandardButtons(QDialogButtonBox::Cancel);
    }
}


AlbumChangeDialog::~AlbumChangeDialog()
{
}


void AlbumChangeDialog::CreateGui()
{
    this->resize(445, 157);
    this->setWindowTitle(QString::fromUtf8("YaRock - Change album Info"));

    QWidget     *gridLayoutWidget;
    QGridLayout *gridLayout;
    QLabel      *label, *label_2, *label_3;
 
    buttonBox = new QDialogButtonBox(this);
    buttonBox->setGeometry(QRect(140, 110, 291, 32));
    buttonBox->setOrientation(Qt::Horizontal);
    buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);
    
    gridLayoutWidget = new QWidget(this);
    gridLayoutWidget->setGeometry(QRect(10, 10, 431, 101));
        
    gridLayout = new QGridLayout(gridLayoutWidget);
    gridLayout->setContentsMargins(0, 0, 0, 0);

    label = new QLabel("Album name", gridLayoutWidget);
    gridLayout->addWidget(label, 0, 0, 1, 1);

    label_2 = new QLabel("Artist name", gridLayoutWidget);
    gridLayout->addWidget(label_2, 1, 0, 1, 1);

    label_3 = new QLabel("Year", gridLayoutWidget);
    gridLayout->addWidget(label_3, 2, 0, 1, 1);

    lineEdit = new QLineEdit(gridLayoutWidget);
    gridLayout->addWidget(lineEdit, 2, 1, 1, 1);

    lineEdit_2 = new QLineEdit(gridLayoutWidget);
    gridLayout->addWidget(lineEdit_2, 1, 1, 1, 1);

    lineEdit_3 = new QLineEdit(gridLayoutWidget);
    gridLayout->addWidget(lineEdit_3, 0, 1, 1, 1);
}



void AlbumChangeDialog::on_buttonBox_accepted()
{
    //! evaluate change
    if( _artist == lineEdit_2->text() && _album == lineEdit_3->text() && _year == lineEdit->text() ) {
      this->close();
      return;
    }
    
    _albuminfochanged = true;
    
    QList <MediaItem *> listMediaItem;
    QString             newCoverName;
    QString             oldCoverName; 

    Database db;
    if (!db.connect("AlbumItem")) return;
    QSqlQuery("BEGIN TRANSACTION;",*db.sqlDb());

    QSqlField data("col",QVariant::String);
    
    //! Select track from album
    QSqlQuery tracksQuery("SELECT `id`, `filename`,`trackname`,`track`,`length`,`album`,`interpret`,`genre`,`year`,`playcount` "\
                          "FROM `view_tracks` WHERE `albumID`="+QString::number(_albumBDId)+";",*db.sqlDb());
 
    while(tracksQuery.next())
    {
      //! Get filename
      QString filename = tracksQuery.value(0).toString();

      //! Get tags from DB
      MediaItem *mi = new MediaItem();
      mi->type = TYPE_TRACK;
      mi->data[TRACK::id]        = tracksQuery.value(0).toInt();    //! = database id
      mi->data[TRACK::url]       = tracksQuery.value(1).toString(); //! = filename
      mi->data[TRACK::title]     = tracksQuery.value(2).toString();
      mi->data[TRACK::num]       = tracksQuery.value(3).toInt();
      mi->data[TRACK::duration]  = tracksQuery.value(4).toInt();
      mi->data[TRACK::album]     = tracksQuery.value(5).toString();
      mi->data[TRACK::artist]    = tracksQuery.value(6).toString();
      mi->data[TRACK::genre]     = tracksQuery.value(7).toString();
      mi->data[TRACK::year]      = tracksQuery.value(8).toUInt();
      mi->data[TRACK::playcount] = tracksQuery.value(9).toUInt();

      oldCoverName = MEDIA::getCoverName(mi);

      //! update mediaItem with new values
      data.setValue( lineEdit_2->text() );
      mi->data[TRACK::artist] = db.sqlDb()->driver()->formatValue(data,false);

      data.setValue( lineEdit_3->text() );
      mi->data[TRACK::album] = db.sqlDb()->driver()->formatValue(data,false);

      data.setValue( mi->data[TRACK::genre].toString() );
      mi->data[TRACK::genre] = db.sqlDb()->driver()->formatValue(data,false);
    
      mi->data[TRACK::year]   = (lineEdit->text()).toUInt();

      newCoverName  = MEDIA::getCoverName(mi);

      listMediaItem.append(mi);
      
      //! delete track from Database      
      data.setValue( mi->data[TRACK::url].toString());
      QString filenameDB = db.sqlDb()->driver()->formatValue(data,false);

      QSqlQuery query("DELETE FROM `tracks` WHERE `filename`="+filenameDB+";", *db.sqlDb());
      query.next();
    }

    
    //! keep album and interpret playcount 
    int oldArtistPlaycount = 0;
    int oldAlbumPlaycount  = 0;
    
    QSqlQuery artistquery("SELECT `id`,`playcount` FROM `interprets` WHERE `interpret`="+_artist+";", *db.sqlDb());
    artistquery.next();
    if (artistquery.isValid()) {
      oldArtistPlaycount = artistquery.value(1).toInt();
      
      QSqlQuery albumQuery("SELECT `id`,`playcount` FROM `albums` WHERE `album`="+_album+" AND `interpretID`="+artistquery.value(0).toString()+";", *db.sqlDb());
      albumQuery.next();
      if (albumQuery.isValid())
        oldAlbumPlaycount = albumQuery.value(1).toInt();
    }
   
    //! DATABASE MODIFICATION : YEAR part in database
    QString s_year;
    data.setValue(listMediaItem.first()->data[TRACK::year].toUInt());
    s_year = db.sqlDb()->driver()->formatValue(data,false);   
    {
      QSqlQuery query("SELECT `id` FROM `years` WHERE `year`="+s_year+";", *db.sqlDb());
      query.next();
      if (!query.isValid())
        QSqlQuery query("INSERT INTO `years`(`year`) VALUES("+s_year+");", *db.sqlDb());
    }

    //! DATABASE MODIFICATION : ARTIST part in database
    {
      QSqlQuery query("SELECT `id` FROM `interprets` WHERE `interpret`="+listMediaItem.first()->data[TRACK::artist].toString()+";", *db.sqlDb());
      query.next();
      if (!query.isValid())
        QSqlQuery query("INSERT INTO `interprets`(`interpret`, `playcount`) " \
                        "VALUES("+listMediaItem.first()->data[TRACK::artist].toString()+","+QString::number(oldArtistPlaycount)+");", *db.sqlDb());	      
    }

    //! DATABASE MODIFICATION : ALBUM part in database
    data.setValue(newCoverName);
    QString coverName_Db = db.sqlDb()->driver()->formatValue(data,false);

    QString interpretID;
    QString albumID;
    {
      QSqlQuery artistquery("SELECT `id` FROM `interprets` WHERE `interpret`="+listMediaItem.first()->data[TRACK::artist].toString()+";", *db.sqlDb());
      artistquery.next();
      data.setValue(artistquery.value(0).toString());
      interpretID = db.sqlDb()->driver()->formatValue(data,false);
      
      qDebug() << "interpret ID found " << interpretID;

      QSqlQuery query("SELECT `id`,`album`,`interpretID` FROM `albums` WHERE `album`="+listMediaItem.first()->data[TRACK::album].toString()+" AND `interpretID`="+interpretID+";", *db.sqlDb());
      query.next();
      if (!query.isValid()) {
        qDebug() << " ----new album /ARTIST ident---- " << listMediaItem.first()->data[TRACK::album].toString() + " " + interpretID;  
      
        QSqlQuery query("INSERT INTO `albums`(`album`,`interpretID`,`cover`,`year`,`favorite`,`playcount`) " \
                        "VALUES("+listMediaItem.first()->data[TRACK::album].toString()+","+interpretID+","+coverName_Db+","+s_year+",0,"+QString::number(oldAlbumPlaycount)+");", *db.sqlDb());
      
        //qDebug() << "insert album artist " << query.exec();
      }
      else {
        data.setValue(query.value(0).toString());
        albumID = db.sqlDb()->driver()->formatValue(data,false);
        QSqlQuery query("UPDATE `albums` " \
                        "SET  `year`="+s_year+" " \
                        "WHERE `id`="+albumID+";", *db.sqlDb());
      }
    } 
      
    foreach (MediaItem* mi, listMediaItem) 
    {    
      uint mtime = QFileInfo(mi->data[TRACK::url].toString()).lastModified().toTime_t();

      //! DATABASE MODIFICATION : TRACK part in database
      QSqlQuery query(*db.sqlDb());
      query.prepare("INSERT INTO `tracks`(`filename`,`trackname`,`track`,`length`,`interpret`,`album`,`year`,`genre`,`mtime`,`playcount`)" \
                      "VALUES(?," \
                      "       ?," \
                      "       ?," \
                      "       ?," \
                      "       (SELECT `id` FROM `interprets` WHERE `interpret`="+mi->data[TRACK::artist].toString()+")," \
                      "       (SELECT `id` FROM `albums` WHERE `album`="+mi->data[TRACK::album].toString()+" AND `interpretID`="+interpretID+")," \
                      "       (SELECT `id` FROM `years` WHERE `year`="+s_year+")," \
                      "       (SELECT `id` FROM `genres` WHERE `genre`="+mi->data[TRACK::genre].toString()+")," \
                      "       ?," \
                      "       ?);");
        query.addBindValue(mi->data[TRACK::url].toString());
        query.addBindValue(mi->data[TRACK::title].toString());
        query.addBindValue(mi->data[TRACK::num].toUInt());
        query.addBindValue(mi->data[TRACK::duration].toInt());
        query.addBindValue(mtime);
        query.addBindValue(mi->data[TRACK::playcount].toInt());
        query.exec();
    }

    //! Just once , If old cover exist -> take it !!
    recupCoverArt(newCoverName,oldCoverName);
  
    //! DATABASE MODIFICATION : CLEAN 
    QSqlQuery queryClean1("DELETE FROM `albums` WHERE `id` NOT IN (SELECT `album` FROM `tracks` GROUP BY `album`);", *db.sqlDb());
    QSqlQuery queryClean2("DELETE FROM `genres` WHERE `id` NOT IN (SELECT `genre` FROM `tracks` GROUP BY `genre`);", *db.sqlDb());
    QSqlQuery queryClean3("DELETE FROM `interprets` WHERE `id` NOT IN (SELECT `interpret` FROM `tracks` GROUP BY `interpret`);", *db.sqlDb());
    QSqlQuery queryClean4("DELETE FROM `years` WHERE `id` NOT IN (SELECT `year` FROM `tracks` GROUP BY `year`);", *db.sqlDb());

    
    QSqlQuery("COMMIT TRANSACTION;",*db.sqlDb());
    if(_albuminfochanged)
      emit signal_AlbumInfoChanged();
  
    this->close();
}


void AlbumChangeDialog::on_buttonBox_rejected()
{
    this->close();
}


/*******************************************************************************
    ArtistChangeDialog 
*******************************************************************************/
ArtistChangeDialog::ArtistChangeDialog(int id)
{
    _artistBDId = id;
    _artistinfochanged = false;
  
    //! create Gui
    CreateGui();
  
    //! Connection
    QObject::connect(buttonBox, SIGNAL(accepted()), this, SLOT(on_buttonBox_accepted()));
    QObject::connect(buttonBox, SIGNAL(rejected()), this, SLOT(on_buttonBox_rejected()));

    //! Get current album Info
    Database db;
    if (db.connect("ArtistItem")) {
      QSqlQuery artistQuery("SELECT interpret FROM interprets WHERE id="+QString::number(_artistBDId)+";",*db.sqlDb());

      artistQuery.next();
      if (artistQuery.isValid()) {
        _artist = artistQuery.value(0).toString();
        lineEdit->setText(_artist); 
      }
    }
    else  {
      lineEdit->setText("Connection to Database failed");
      buttonBox->setStandardButtons(QDialogButtonBox::Cancel);
    }
}


ArtistChangeDialog::~ArtistChangeDialog()
{
}


void ArtistChangeDialog::CreateGui()
{
    this->resize(445, 157);
    this->setWindowTitle(QString::fromUtf8("YaRock - Change artist Info"));
    
    QWidget     *gridLayoutWidget;
    QGridLayout *gridLayout;
    QLabel      *label;
 
    buttonBox = new QDialogButtonBox(this);
    buttonBox->setGeometry(QRect(140, 110, 291, 32));
    buttonBox->setOrientation(Qt::Horizontal);
    buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);
    
    gridLayoutWidget = new QWidget(this);
    gridLayoutWidget->setGeometry(QRect(10, 10, 431, 101));
        
    gridLayout = new QGridLayout(gridLayoutWidget);
    gridLayout->setContentsMargins(0, 0, 0, 0);

    label = new QLabel("Artist name", gridLayoutWidget);
    gridLayout->addWidget(label, 0, 0, 1, 1);

    lineEdit = new QLineEdit(gridLayoutWidget);
    gridLayout->addWidget(lineEdit, 0, 1, 1, 1);
}


void ArtistChangeDialog::on_buttonBox_accepted()
{
  Database db;
  if (!db.connect("AlbumItem")) return;
  
  QSqlField data("col",QVariant::String);
  
  qDebug() << " ----on_buttonBox_accepted ---- ";  

  QSqlQuery("BEGIN TRANSACTION;",*db.sqlDb());
  
  //! ARTIST HAS CHANGED ?
  data.setValue(lineEdit->text());
  QString artistBD = db.sqlDb()->driver()->formatValue(data,false);

  data.setValue(_artistBDId);
  QString artistBDId = db.sqlDb()->driver()->formatValue(data,false);

  if(_artist != lineEdit->text()) {
     qDebug() << " ----on_buttonBox_accepted Artist different---- ";  
      
     //! check if artist already exist
     QSqlQuery artistQuery("SELECT `id` FROM `interprets` WHERE `interpret`="+artistBD+";",*db.sqlDb());
     artistQuery.next();
     if (!artistQuery.isValid()) {
       qDebug() << " ----Artist n'existe pas ---- " << lineEdit->text();  
       QSqlQuery query("INSERT INTO `interprets`(`interpret`) VALUES("+artistBD+");",*db.sqlDb());
     }
     
     
     QSqlQuery artistQuery2("SELECT `id` FROM `interprets` WHERE `interpret`="+artistBD+";",*db.sqlDb());
     artistQuery2.next();
     data.setValue(artistQuery2.value(0).toString());
     QString newartistBDId = db.sqlDb()->driver()->formatValue(data,false);
  
     //! Update Album table
     QSqlQuery albumQuery("SELECT `id`,`album`,`cover` FROM `albums` WHERE `interpretID`="+artistBDId+";",*db.sqlDb());
     while(albumQuery.next()) {
     
       QString oldCoverName = albumQuery.value(2).toString();
       //qDebug() << " ----oldCoverName ---- " <<oldCoverName;  

       QString newCoverName = MEDIA::getCoverName(lineEdit->text(), albumQuery.value(1).toString());
       //qDebug() << " ----newCoverName ---- " <<newCoverName;  

       data.setValue(newCoverName);
       QString coverpathBD = db.sqlDb()->driver()->formatValue(data,false);
       
       QSqlQuery query("UPDATE `albums` "                  \
                     "SET `cover`="+coverpathBD+","        \
                     "    `interpretID`="+newartistBDId+" " \
                     "WHERE `id`="+albumQuery.value(0).toString()+";",*db.sqlDb());
    
       //! If old cover exist -> take it !!
       recupCoverArt(newCoverName, oldCoverName);
       
     } //! WHILE album query
     
    //! Update Tracks table
    QSqlQuery query2("UPDATE `tracks` SET `interpret`="+newartistBDId+" WHERE `interpret`="+artistBDId+" ;",*db.sqlDb());
    
    //! CLEAN 
    QSqlQuery query3("DELETE FROM `interprets` WHERE `id` NOT IN (SELECT `interpret` FROM `tracks` GROUP BY `interpret`);", *db.sqlDb());
    _artistinfochanged = true;
  }

  QSqlQuery("COMMIT TRANSACTION;",*db.sqlDb());
  
  if(_artistinfochanged)
    emit signal_ArtistInfoChanged();
  
  this->close();
}


void ArtistChangeDialog::on_buttonBox_rejected()
{
    this->close();
}


//! -------------- recupCoverArt -----------------------------------------------
void recupCoverArt(const QString &newCoverName, const QString &oldCoverName)
{
  qDebug() << " ----recupCoverArt newCoverName + oldCoverName---- " << newCoverName + " - " + oldCoverName;  

  QString storageLocation = UTIL::CONFIGDIR + "/albums/";
  
  //! quit if same filename
  if(newCoverName == oldCoverName) return;
   
  //! remove new cover if already exist
  QFile newCoverFile(storageLocation + newCoverName);
  if(newCoverFile.exists()) newCoverFile.remove();

  //! copy old into new
  QFile oldCoverFile(storageLocation + oldCoverName);
  if(oldCoverFile.exists()) { 
    qDebug() << " ----recupCoverArt oldCoverName exists COPY ---- ";  

    oldCoverFile.rename(storageLocation + newCoverName);
  }
}


/*******************************************************************************
    GenreChangeDialog 
*******************************************************************************/
GenreChangeDialog::GenreChangeDialog(int id, QString genre)
{
    _albumBDId = id;
    _genre = genre;
    
    //! create Gui
    CreateGui();
  
    //! Connection
    QObject::connect(buttonBox, SIGNAL(accepted()), this, SLOT(on_buttonBox_accepted()));
    QObject::connect(buttonBox, SIGNAL(rejected()), this, SLOT(on_buttonBox_rejected()));

    //! Genre is set with Current AlbumGenreItem value
    lineEdit->setText(_genre);   
}


GenreChangeDialog::~GenreChangeDialog()
{
}


void GenreChangeDialog::CreateGui()
{
    this->resize(445, 157);
    this->setWindowTitle(QString::fromUtf8("YaRock - Change album Info"));

    QWidget     *gridLayoutWidget;
    QGridLayout *gridLayout;
    QLabel      *label;
 
    buttonBox = new QDialogButtonBox(this);
    buttonBox->setGeometry(QRect(140, 110, 291, 32));
    buttonBox->setOrientation(Qt::Horizontal);
    buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);
    
    gridLayoutWidget = new QWidget(this);
    gridLayoutWidget->setGeometry(QRect(10, 10, 431, 101));
        
    gridLayout = new QGridLayout(gridLayoutWidget);
    gridLayout->setContentsMargins(0, 0, 0, 0);

    label = new QLabel("Genre", gridLayoutWidget);
    gridLayout->addWidget(label, 0, 0, 1, 1);

    lineEdit = new QLineEdit(gridLayoutWidget);
    gridLayout->addWidget(lineEdit, 0, 1, 1, 1);
}

void GenreChangeDialog::on_buttonBox_accepted()
{
    //! check if there is user modification
    QString newGenre = lineEdit->text(); 
    if(_genre == newGenre) {
      this->close();
      return;
    }  

    //qDebug() << " ----GenreChangeDialog -> genre is different ----";

    Database db;
    if (!db.connect("AlbumGenreItem")) return;
  
    QSqlField data("col",QVariant::String);  
    QSqlQuery("BEGIN TRANSACTION;",*db.sqlDb());

    data.setValue(newGenre);
    newGenre = db.sqlDb()->driver()->formatValue(data,false);

    data.setValue(_genre);
    _genre = db.sqlDb()->driver()->formatValue(data,false);
  
    //! DATABASE MODIFICATION : GENRE part in database
    {
      QSqlQuery query("SELECT `id` FROM `genres` WHERE `genre`="+newGenre+";", *db.sqlDb());
      query.next();
    
      if (!query.isValid())
        QSqlQuery query("INSERT INTO `genres`(`genre`) VALUES("+newGenre+");", *db.sqlDb());
    }

    QSqlQuery genreQuery("SELECT `id` FROM `genres` WHERE `genre`="+newGenre+";", *db.sqlDb());
    genreQuery.next();

    int newGenreID = genreQuery.value(0).toInt();
  
    //! DATABASE MODIFICATION : TRACKS part in database
    QSqlQuery tracksQuery("SELECT `id` FROM `view_tracks` WHERE `albumID`="+QString::number(_albumBDId)+" AND `genre`="+_genre+";",
                          *db.sqlDb());

    while(tracksQuery.next())
    {
      int trackID = tracksQuery.value(0).toInt();
    
      QSqlQuery query("UPDATE `tracks` "                  \
                      "SET `genre`="+QString::number(newGenreID)+" " \
                      "WHERE `id`="+QString::number(trackID)+";",*db.sqlDb());
      
    }
      
    //! DATABASE MODIFICATION : CLEAN 
    {
    QSqlQuery query("DELETE FROM `genres` WHERE `id` NOT IN (SELECT `genre` FROM `tracks` GROUP BY `genre`);", *db.sqlDb());
    }
  
    QSqlQuery("COMMIT TRANSACTION;",*db.sqlDb());
    emit signal_GenreInfoChanged();
  
    this->close();
}


void GenreChangeDialog::on_buttonBox_rejected()
{
    this->close();
}







/*******************************************************************************
    PlaylistChangeDialog 
*******************************************************************************/
PlaylistChangeDialog::PlaylistChangeDialog(int id)
{
    _playlistBDId = id;
    _playlistinfochanged = false;
    //qDebug() << " ----PlaylistChangeDialog _playlistBDId---- " << _playlistBDId;  

    //! create Gui
    CreateGui();
  
    //! Connection
    QObject::connect(buttonBox, SIGNAL(accepted()), this, SLOT(on_buttonBox_accepted()));
    QObject::connect(buttonBox, SIGNAL(rejected()), this, SLOT(on_buttonBox_rejected()));

    //! Get current album Info
    Database db;
    if (db.connect("PlaylistItem")) {
      QSqlQuery playlistQuery("SELECT name FROM playlists WHERE id="+QString::number(_playlistBDId)+";",*db.sqlDb());

      playlistQuery.next();
      if (playlistQuery.isValid()) {
        _playlist = playlistQuery.value(0).toString();
        lineEdit->setText(_playlist); 
      }
    }
    else
    {
      lineEdit->setText("Connection to Database failed");
      buttonBox->setStandardButtons(QDialogButtonBox::Cancel);
    }
}


PlaylistChangeDialog::~PlaylistChangeDialog()
{
}


void PlaylistChangeDialog::CreateGui()
{
    this->resize(445, 157);
    this->setWindowTitle(QString::fromUtf8("YaRock - Change playlist Info"));
    
    QWidget     *gridLayoutWidget;
    QGridLayout *gridLayout;
    QLabel      *label;
 
    buttonBox = new QDialogButtonBox(this);
    buttonBox->setGeometry(QRect(140, 110, 291, 32));
    buttonBox->setOrientation(Qt::Horizontal);
    buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);
    
    gridLayoutWidget = new QWidget(this);
    gridLayoutWidget->setGeometry(QRect(10, 10, 431, 101));
        
    gridLayout = new QGridLayout(gridLayoutWidget);
    gridLayout->setContentsMargins(0, 0, 0, 0);

    label = new QLabel(tr("Playlist name"), gridLayoutWidget);
    gridLayout->addWidget(label, 0, 0, 1, 1);

    lineEdit = new QLineEdit(gridLayoutWidget);
    gridLayout->addWidget(lineEdit, 0, 1, 1, 1);
}


void PlaylistChangeDialog::on_buttonBox_accepted()
{
    Database db;
    if (!db.connect("PlaylistItem")) return;
  
    QSqlField data("col",QVariant::String);
  
    QSqlQuery("BEGIN TRANSACTION;",*db.sqlDb());
  
    //! ARTIST HAS CHANGED ?
    data.setValue(lineEdit->text());
    QString playlistNameBD = db.sqlDb()->driver()->formatValue(data,false);

    if(_playlist != lineEdit->text()) 
    {    
       QSqlQuery query("UPDATE `playlists` "         \
                     "SET `name`="+playlistNameBD+" " \
                     "WHERE `id`="+QString::number(_playlistBDId)+";",*db.sqlDb());
    
      _playlistinfochanged = true;
    }

    QSqlQuery("COMMIT TRANSACTION;",*db.sqlDb());
  
    if(_playlistinfochanged)
      emit signal_PlaylistInfoChanged();
  
    this->close();
}


void PlaylistChangeDialog::on_buttonBox_rejected()
{
    this->close();
}
