/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "CollectionView/virtualplaylist.h"
#include "utilities.h" //! pour randomInt

#include <QDebug>


/*******************************************************************************
    VirtualPlaylist 
*******************************************************************************/
VirtualPlaylist::VirtualPlaylist(QWidget *parent)
{
Q_UNUSED(parent)

    _PlayingIdx    = -1;
    _isPlaying     = false;
    _repeatMode    = RepeatOff;
    _shuffleMode   = ShuffleOff;
}


//! ----- add files methods -----------------------------------------------------
void VirtualPlaylist::AppendFileInPlaylist(const QString &file)
{
    //qDebug() << "-- VirtualPlaylist::AppendFileInPlaylist" << file;

    if (file.isEmpty()) return;
    _listTrack.clear();
    _listTrack.append(file);
    _listisPlayed.clear();
    _listisPlayed.append(false);
    
    setActiveTrack(0);
    emit signal_collection_playTrack();
}

void VirtualPlaylist::AppendFilesInPlaylist(const QStringList& list)
{
    //qDebug() << "-- VirtualPlaylist::AppendFilesInPlaylist" << list;
      
    if (list.isEmpty()) return;
    
    _listTrack.clear();
    _listisPlayed.clear();
    
    _listTrack.append(list);

    for (int i = 0; i< _listTrack.size(); i++)
      _listisPlayed.append(false);
      
    setActiveTrack(0);
    emit signal_collection_playTrack();
}


void VirtualPlaylist::AppendFilesInPlaylist(const QStringList& list, int startIdx) 
{
    //qDebug() << "-- VirtualPlaylist::AppendFilesInPlaylist startIdx=" << startIdx;
      
    if (list.isEmpty()) return;
    
    if(startIdx >= 0 && startIdx < list.size()) {
      
      _listTrack.clear();
      _listisPlayed.clear();
      
      _listTrack.append(list);

      for (int i = 0; i< _listTrack.size(); i++)
        _listisPlayed.append(false);
    
      setActiveTrack(startIdx);
      emit signal_collection_playTrack();
    }
}
   
    

void VirtualPlaylist::ClearPlaylist()
{
    _listTrack.clear();
    _listisPlayed.clear();

    _PlayingIdx = -1;
    _isPlaying = false;
}

QString VirtualPlaylist::GetNextTrack()
{
    QString file;
    //qDebug() << "-- VirtualPlaylist::GetNextTrack";
       
    //! normal mode get next track
    if(_repeatMode == RepeatOff && _shuffleMode == ShuffleOff) {
      if(_PlayingIdx < _listTrack.size()- 1) {
         setActiveTrack(++_PlayingIdx);
         file = GetActiveTrack();
         return (file); 
      }
    }
  
    //! repeat track mode
    if(_repeatMode == RepeatTrack) {
      if(_PlayingIdx < _listTrack.size()- 1) {
         setActiveTrack(_PlayingIdx);
         file = GetActiveTrack();
        return (file);     
      }
    }
  
    //! repeat all mode
    if(_repeatMode == RepeatAll && _shuffleMode == ShuffleOff) {
      if( _PlayingIdx == _listTrack.size()- 1) {
	setActiveTrack(0);
	file = GetActiveTrack();
	return (file);     
      }
      else if(_PlayingIdx < _listTrack.size()- 1) {
	setActiveTrack(++_PlayingIdx);
	file = GetActiveTrack();
        return (file); 
      }
    }

  //! shuffle ==> random next until all is played
  if(_shuffleMode == ShuffleOn) {
    QStringList list = unplayedTrack();
    if(list.isEmpty()) return QString();
    
    int index = UTIL::randomInt(0, list.size() -1);
    QString track = list.at(index);
    
    setActiveTrack(_listTrack.indexOf(track));    
    
    file = GetActiveTrack();
    return (file); 
  }

  return QString();
}


QString VirtualPlaylist::GetPrevTrack()
{
    QString file;

    //! normal mode get prev track
    if(_repeatMode == RepeatOff && _shuffleMode == ShuffleOff) {
      if(_PlayingIdx > 0) {
        setActiveTrack(--_PlayingIdx);
	file = GetActiveTrack();
        return (file);
      }
    }

    //! repeat track mode
    if(_repeatMode == RepeatTrack) {
      if(_PlayingIdx < _listTrack.size()- 1) {
	setActiveTrack(_PlayingIdx);
	file = GetActiveTrack();
        return (file);     
      }
    }
  
    //! repeat all mode
    if(_repeatMode == RepeatAll && _shuffleMode == ShuffleOff) {
      if( _PlayingIdx == 0) {
	setActiveTrack(_listTrack.size()- 1);
	file = GetActiveTrack();
        return (file);     
      }
      else if(_PlayingIdx > 0) {
	setActiveTrack(--_PlayingIdx);
	file = GetActiveTrack();
        return (file);
      }
    }
    
    //! shuffle ==> random next until all is played
    if(_shuffleMode == ShuffleOn) {
      QStringList list = unplayedTrack();
      if(list.isEmpty()) return QString();
      
      int index = UTIL::randomInt(0, list.size() -1);
      QString track = list.at(index);
      
      setActiveTrack(_listTrack.indexOf(track));    
      
      file = GetActiveTrack();
      return (file); 
    }  
  
    return QString();
}


void VirtualPlaylist::setActiveTrack(int idx)
{
    //qDebug() << "-- VirtualPlaylist::setActiveTrack idx=" << idx;
    
    if( (idx > -1 && idx < _listTrack.size()) ||
        (idx == 0 && _listTrack.size() == 1 ))
    {
      _listisPlayed[idx]  = true;
      _isPlaying     = true;  
      _PlayingIdx    = idx;
    }
}



QStringList VirtualPlaylist::unplayedTrack()
{
  QStringList list;
  

  for (int i = 0; i< _listisPlayed.size(); i++)
    if(_listisPlayed.at(i) == false)
      list << _listTrack.at(i);
  
  return list;
}
    
    
QString VirtualPlaylist::GetActiveTrack()
{
    //qDebug() << "-- VirtualPlaylist::GetActiveTrack ";

    QString file;
    if( (_PlayingIdx > -1 && _PlayingIdx < _listTrack.size()) ||
        (_PlayingIdx == 0 && _listTrack.size() ==1))
    {
      file = _listTrack.at(_PlayingIdx);
    }
    
    return (file);   
}


void  VirtualPlaylist::StopPlaying()
{
    _PlayingIdx = -1;
    _isPlaying  = false;
}


