/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "ContextView/contextgraphicitem.h"
#include <QString>
#include <QObject>

/*******************************************************************************
    Class TitleGraphicItem
*******************************************************************************/
TitleGraphicItem::TitleGraphicItem() 
{
   setAcceptsHoverEvents(false);
   setAcceptDrops(false);
   setFlag(QGraphicsItem::ItemIsSelectable, false);
   setFlag(QGraphicsItem::ItemIsMovable, false);
   setFlag(QGraphicsItem::ItemIgnoresTransformations, true);
   _width = 630;
}

void TitleGraphicItem::setPixmap(const QPixmap& p)
{
   _pix   = p.scaled(QSize(24,24), Qt::KeepAspectRatio, Qt::SmoothTransformation);
}

    
QRectF TitleGraphicItem::boundingRect() const
{
    return QRectF(0, 0, _width, 50);
}

void TitleGraphicItem::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * )
{
Q_UNUSED(option)

   //! draw categorie text
  QFontDatabase * fontdb = new QFontDatabase();
  painter->setFont(fontdb->font("Segoe UI", "Light", 14) );
  
  QColor outlineColor = QColor ( 0, 0, 0, 50);
  QColor inlineColor = QColor ( 0, 0, 0, 255);

  painter->drawPixmap(10, 10, _pix);
  painter->setPen(inlineColor);
  painter->drawStaticText(40,5, QStaticText(_name));
 
  //BEGIN: horizontal line
  {
    QPoint start(boundingRect().toRect().left()+40, boundingRect().toRect().top()+28);
    start.rx() += 3;
    QPoint horizontalGradTop(boundingRect().toRect().topLeft());
    horizontalGradTop.rx() += boundingRect().toRect().width() - 6;
    QLinearGradient gradient(start, horizontalGradTop);
    gradient.setColorAt(0, inlineColor);
    gradient.setColorAt(1, outlineColor);	
    painter->fillRect(QRect(start, QSize(boundingRect().toRect().width() - 6, 1)), gradient);
  }
  //END: horizontal line
}



/*******************************************************************************
    Class Button
*******************************************************************************/
Button::Button()
{
   setAcceptsHoverEvents(true);
   setAcceptDrops(false);
   setFlag(QGraphicsItem::ItemIsSelectable, false);
   setFlag(QGraphicsItem::ItemIsMovable, false);
   setFlag(QGraphicsItem::ItemIgnoresTransformations, true);
   
   //! option configuration
   QPalette palette;
   palette.setColor(QPalette::Background, Qt::white);
   palette.setColor(QPalette::Window , Qt::white);
   palette.setColor(QPalette::Foreground, Qt::white);
   palette.setColor(QPalette::Base, Qt::white);
   palette.setColor(QPalette::AlternateBase, Qt::white);
   palette.setColor(QPalette::Button,  QColor(240,240,240));
   
   opt.palette = palette;
   opt.fontMetrics = QApplication::fontMetrics();
   opt.features |= QStyleOptionButton::None;
   
   //opt.features |= QStyleOptionButton::DefaultButton | QStyleOptionButton::Flat;
   opt.state |= QStyle::State_Enabled;
   opt.state &= ~QStyle::State_HasFocus;
}

void Button::setIcon(QIcon i)
{
   opt.icon = i;
   opt.iconSize = QSize(14,14);
}

void Button::setText(QString t)
{
  opt.text = t;
}
    
    

QRectF Button::boundingRect() const
{
   QFontMetrics fm(QFont("Arial", 8, QFont::Normal));
   int width = fm.width(opt.text);

   return QRectF(0, 0, width + opt.iconSize.width() + 25, 30);

}

void Button::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget)
{
  Q_UNUSED(option)

     QStyle *style = widget ? widget->style() : QApplication::style();
     opt.rect = boundingRect().toRect();  
     painter->setFont(QFont("Arial", 8, QFont::Normal));
     
     style->drawControl(QStyle::CE_PushButton, &opt, painter);
}



void Button::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
{
  Q_UNUSED(event)
  //opt.state |= QStyle::State_MouseOver;
  opt.state |= QStyle::State_HasFocus;
  //opt.state |= QStyle::State_Selected;  
  update();
}

void Button::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
{
  Q_UNUSED(event)
  //opt.state &= ~QStyle::State_MouseOver;
  opt.state &= ~QStyle::State_HasFocus;
  //opt.state &= ~QStyle::State_Selected;
  update();
}

void Button::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
  Q_UNUSED(event)
  action->trigger();    
}


/*******************************************************************************
    Class ButtonLink
*******************************************************************************/
ButtonLink::ButtonLink()
{
   setAcceptsHoverEvents(true);
   setAcceptDrops(false);
   setFlag(QGraphicsItem::ItemIsSelectable, false);
   setFlag(QGraphicsItem::ItemIsMovable, false);
   setFlag(QGraphicsItem::ItemIgnoresTransformations, true);
   
   //! option configuration
   QPalette palette;
   palette.setColor(QPalette::Background, Qt::white);
   palette.setColor(QPalette::Window , Qt::white);
   palette.setColor(QPalette::Foreground, Qt::white);
   palette.setColor(QPalette::Base, Qt::white);
   palette.setColor(QPalette::AlternateBase, Qt::white);
   palette.setColor(QPalette::Button,  QColor(240,240,240));
   
   opt.palette = palette;
   opt.fontMetrics = QApplication::fontMetrics();
   opt.features |= QStyleOptionButton::None;
   
   //opt.features |= QStyleOptionButton::DefaultButton | QStyleOptionButton::Flat;
   opt.state |= QStyle::State_Enabled;
   opt.state &= ~QStyle::State_HasFocus;
}

void ButtonLink::clear()
{
   link.clear();
}


void ButtonLink::setIcon(QIcon i)
{
   opt.icon = i;
   opt.iconSize = QSize(14,14);
}

void ButtonLink::setText(QString t)
{
  opt.text = t;
}
    
    

QRectF ButtonLink::boundingRect() const
{
   QFontMetrics fm(QFont("Arial", 8, QFont::Normal));
   int width = fm.width(opt.text);

   return QRectF(0, 0, width + opt.iconSize.width() + 25, 30);

}

void ButtonLink::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget)
{
  Q_UNUSED(option)

     QStyle *style = widget ? widget->style() : QApplication::style();
     opt.rect = boundingRect().toRect();  
     painter->setFont(QFont("Arial", 8, QFont::Normal));
     
     //style->drawControl(QStyle::CE_PushButton, &opt, painter, widget);
     style->drawControl(QStyle::CE_PushButton, &opt, painter);
}



void ButtonLink::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
{
  Q_UNUSED(event)
  //opt.state |= QStyle::State_MouseOver;
  opt.state |= QStyle::State_HasFocus;
  //opt.state |= QStyle::State_Selected;  
  update();
}

void ButtonLink::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
{
  Q_UNUSED(event)
  //opt.state &= ~QStyle::State_MouseOver;
  opt.state &= ~QStyle::State_HasFocus;
  //opt.state &= ~QStyle::State_Selected;
  update();
}

void ButtonLink::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
  Q_UNUSED(event)
  QDesktopServices::openUrl(QUrl(link));
}



/*******************************************************************************
    Class ArtistInfoGraphicItem
*******************************************************************************/
ArtistInfoGraphicItem::ArtistInfoGraphicItem()
{
   setAcceptsHoverEvents(false);
   setAcceptDrops(false);
   setFlag(QGraphicsItem::ItemIsSelectable, false);
   setFlag(QGraphicsItem::ItemIsMovable, false);
   setFlag(QGraphicsItem::ItemIgnoresTransformations, true);
   _width = 630;
}

void ArtistInfoGraphicItem::clear()
{
   _artistBio.clear();
   _readMoreUrl.clear();
   _artistPix = QPixmap();
   _similarArtist.clear();
}

void ArtistInfoGraphicItem::updateItem()
{
  prepareGeometryChange();
  update();
}

QRectF ArtistInfoGraphicItem::boundingRect() const
{
    int YPos      =  0;
    int XPos      =  0;
    QTextDocument doc;
    doc.setDefaultFont(QFont("Arial", 8, QFont::Normal)); 
      
    XPos = _artistPix.width() + 10 + 30;

    // artist biographie 
    if(!_artistBio.isEmpty() || !_similarArtist.isEmpty()) {
      doc.setHtml(_artistBio + _similarArtist);
      doc.setTextWidth(_width - XPos - 40);
      YPos += doc.size().height();
    } 

    // Y position correction
    if (_artistPix.height() > YPos) 
      YPos = _artistPix.height();
    
    return QRectF(0, 0, _width, YPos);
}


void ArtistInfoGraphicItem::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * )
{
Q_UNUSED(option)
    QFont normalFont   = QFont("Arial", 8, QFont::Normal);
    
    int YPos      =  0;
    int XPos      =  0;
    QTextDocument doc;
    doc.setDefaultFont(normalFont); 
      
    if(!_artistPix.isNull()) 
      painter->drawPixmap(15,YPos + 10, _artistPix);
        
    XPos = _artistPix.width() + 10 + 30;

    // artist biographie 
    if(!_artistBio.isEmpty() || !_similarArtist.isEmpty()) {
      doc.setHtml(_artistBio);
      doc.setTextWidth(_width - XPos - 40);
      painter->translate(XPos , YPos);
      doc.drawContents(painter);
      painter->translate(-XPos,-YPos);
        
      YPos += doc.size().height();
      
      doc.setHtml(_similarArtist);
      doc.setTextWidth(_width - XPos);
      painter->translate(XPos , YPos);
      doc.drawContents(painter);
      painter->translate(-XPos,-YPos);
       
      YPos += doc.size().height();
    } 
}


/*******************************************************************************
    Class AlbumInfoGraphicItem
*******************************************************************************/
AlbumInfoGraphicItem::AlbumInfoGraphicItem()
{
   setAcceptsHoverEvents(false);
   setAcceptDrops(false);
   setFlag(QGraphicsItem::ItemIsSelectable, false);
   setFlag(QGraphicsItem::ItemIsMovable, false);
   setFlag(QGraphicsItem::ItemIgnoresTransformations, false);
   _width = 630;
}

void AlbumInfoGraphicItem::clear()
{
   _albumBio.clear();
   _albumPix = QPixmap();
}

void AlbumInfoGraphicItem::updateItem()
{
  prepareGeometryChange();
  //update();
}

QRectF AlbumInfoGraphicItem::boundingRect() const
{
    int YPos      =  0;
    int XPos      =  0;
    
    QTextDocument doc;
    doc.setDefaultFont(QFont("Arial", 8, QFont::Normal)); 
      
    XPos = 190;
      
    if(!_albumBio.isEmpty()) {
      doc.setHtml(_albumBio);
      doc.setTextWidth(_width - XPos -40);
      YPos +=  doc.size().height();
    }      

    // Y position correction
    if (_albumPix.height() > YPos) 
      YPos = _albumPix.height();
      
    return QRectF(0, 0, _width, YPos);
}


void AlbumInfoGraphicItem::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * )
{
Q_UNUSED(option)
    QFont normalFont   = QFont("Arial", 8, QFont::Normal);
    
    int YPos      = 0;
    int XPos      = 0;
    QTextDocument doc;
    doc.setDefaultFont(normalFont); 
      
    // album pixmap
    if(!_albumPix.isNull()) {
      painter->drawPixmap(25,YPos, _albumPix);
    }

    // album wiki
    //if(XPos < _albumPix.width()) 
    XPos = 190;
      
    if(!_albumBio.isEmpty()) {
      doc.setHtml(_albumBio);
      doc.setTextWidth(_width - XPos -40);
      painter->translate(XPos , YPos);
      doc.drawContents(painter);
      painter->translate(-XPos,-YPos);
    }     
    if(doc.size().height() < _albumPix.height())
      YPos += _albumPix.height();
    else
      YPos +=  doc.size().height();
}

/*******************************************************************************
    Class LyricsInfoGraphicItem
*******************************************************************************/
LyricsInfoGraphicItem::LyricsInfoGraphicItem()
{
   setAcceptsHoverEvents(false);
   setAcceptDrops(false);
   setFlag(QGraphicsItem::ItemIsSelectable, false);
   setFlag(QGraphicsItem::ItemIsMovable, false);
   setFlag(QGraphicsItem::ItemIgnoresTransformations, true);
   _width = 630;
}

void LyricsInfoGraphicItem::clear()
{
   _songLyrics.clear();
}

void LyricsInfoGraphicItem::updateItem()
{
  prepareGeometryChange();
  update();
}

QRectF LyricsInfoGraphicItem::boundingRect() const
{
    int YPos      =  0;
    int XPos      =  0;
    QTextDocument doc;
    doc.setDefaultFont(QFont("Arial", 8, QFont::Normal)); 
      
    XPos = 190;
    doc.setHtml(_songLyrics);
    doc.setTextWidth(_width - XPos);
    YPos += doc.size().height();

    return QRectF(0, 0, _width, YPos);
}


void LyricsInfoGraphicItem::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * )
{
Q_UNUSED(option)
    QFont normalFont   = QFont("Arial", 8, QFont::Normal);
    
    int YPos      = 0;
    int XPos      = 0;
    QTextDocument doc;
    doc.setDefaultFont(normalFont); 
      
    XPos = 190;
    doc.setHtml(_songLyrics);
    doc.setTextWidth(_width - XPos);
    painter->translate(XPos , YPos);
    doc.drawContents(painter);
    painter->translate(-XPos,-YPos);
     
    YPos += doc.size().height();
}
