/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _CONTEXT_GRAPHICS_ITEM_H_
#define _CONTEXT_GRAPHICS_ITEM_H_

#include <QtGui>
#include <QStyleOptionButton>

/*******************************************************************************
    TitleGraphicItem
*******************************************************************************/
class TitleGraphicItem : public QGraphicsItem
{
  public:
    TitleGraphicItem();
    void setPixmap(const QPixmap &p);
    QString     _name;
    QPixmap     _pix;
    int         _width;
    
  protected:
    QRectF boundingRect() const;
    void paint(QPainter *p, const QStyleOptionGraphicsItem *, QWidget *);
};


/*******************************************************************************
    Button
*******************************************************************************/
class Button : public QGraphicsItem
{
  public:
    Button();
    void               setIcon(QIcon i);
    void               setText(QString t);
    int height() {return boundingRect().toRect().height();}
    QAction            *action;

  protected:
    // Implement QGraphicsItem method
    QRectF boundingRect() const;
    void paint(QPainter *p, const QStyleOptionGraphicsItem *, QWidget *);    
    void hoverEnterEvent(QGraphicsSceneHoverEvent *event);
    void hoverLeaveEvent(QGraphicsSceneHoverEvent *event);
    void mousePressEvent(QGraphicsSceneMouseEvent *event);
    
  private:
    QStyleOptionButton opt;
};

/*******************************************************************************
    ButtonLink
*******************************************************************************/
class ButtonLink : public QGraphicsItem
{
  public:
    ButtonLink();
    void               setIcon(QIcon i);
    void               setText(QString t);
    QString            link;
    void clear();

    int height() {return boundingRect().toRect().height();}

  protected:
    // Implement QGraphicsItem method
    QRectF boundingRect() const;
    void paint(QPainter *p, const QStyleOptionGraphicsItem *, QWidget *);    
    void hoverEnterEvent(QGraphicsSceneHoverEvent *event);
    void hoverLeaveEvent(QGraphicsSceneHoverEvent *event);
    void mousePressEvent(QGraphicsSceneMouseEvent *event);
    
  private:
    QStyleOptionButton opt;
};

/*******************************************************************************
    ArtistInfoGraphicItem
*******************************************************************************/
class ArtistInfoGraphicItem : public QGraphicsItem
{
  public:
    ArtistInfoGraphicItem();
    void updateItem();
    void clear();
    int height() {return boundingRect().toRect().height();}

    QString            _artistBio;
    QString            _readMoreUrl;
    QPixmap            _artistPix;
    QString            _similarArtist;

    int                _width;
    
  protected:
    // Implement QGraphicsItem method
    QRectF boundingRect() const;
    void paint(QPainter *p, const QStyleOptionGraphicsItem *, QWidget *);
};


/*******************************************************************************
    AlbumInfoGraphicItem
*******************************************************************************/
class AlbumInfoGraphicItem : public QGraphicsItem
{
  public:
    AlbumInfoGraphicItem();
    void updateItem();
    void clear();
    int height() {return boundingRect().toRect().height();}
    
    QString            _albumBio;
    QPixmap            _albumPix;
    int                _width;

  protected:
    // Implement QGraphicsItem method
    QRectF boundingRect() const;
    void paint(QPainter *p, const QStyleOptionGraphicsItem *, QWidget *);
};


/*******************************************************************************
    LyricsInfoGraphicItem
*******************************************************************************/
class LyricsInfoGraphicItem : public QGraphicsItem
{
  public:
    LyricsInfoGraphicItem();
    void updateItem();
    void clear();
    int height() {return boundingRect().toRect().height();}
    
    QString            _songLyrics;
    int                _width;

  protected:
    // Implement QGraphicsItem method
    QRectF boundingRect() const;
    void paint(QPainter *p, const QStyleOptionGraphicsItem *, QWidget *);
};

#endif // _CONTEXT_GRAPHICS_ITEM_H_
