/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef _CONTEXT_SCENE_H_
#define _CONTEXT_SCENE_H_


#include "ContextView/contextgraphicitem.h"

#include <QtGui>
#include <QtCore>

class InfoResolver;
class InfoDownloader;


/*******************************************************************************
    ContextScene
*******************************************************************************/
class ContextScene : public QGraphicsScene 
{
Q_OBJECT
  public:
    ContextScene(QWidget* parent);
    void setActivate(bool a) {_activated = a;}
    bool isActivated() {return _activated;}
    void startContextScene();
        
  private slots:
    void updateDraw();
    void updateContextScene();
    
    //! artist
    void artistBioReady(int id, QString provider, QString bio);
    void artistImageReady(int id, QByteArray bytes);
    void artistSimilarReady(int id,QStringList list);
    
    //! album
    void albumBioReady(int id, QString bio);
    void albumImageReady(int id, QByteArray bytes);
    
    //! lyrics
    void songLyricsReady(int id, const QString& provider, const QString& lyrics);
    void addLyrics();
    void removeLyrics();
    
  private :
    QWidget                   *_parentWidget;
    bool                      _activated;
    InfoResolver              *_infoResolver;
    InfoDownloader            *_infoDownloader;
    int                       _currentId;
    QString                   _url;
    QString                   _lyrics;
    
    TitleGraphicItem          *_titleArtistGraphicItem;
    TitleGraphicItem          *_titleAlbumGraphicItem;
    TitleGraphicItem          *_titleSongGraphicItem;
    ArtistInfoGraphicItem     *_artistInfoGraphicItem;
    AlbumInfoGraphicItem      *_albumInfoGraphicItem;
    LyricsInfoGraphicItem     *_lyricsInfoGraphicItem;
    
    ButtonLink                *_linkLastFm;
    Button                    *_addLyrics;
    Button                    *_removeLyrics;
    bool                      isLocalLyrics;
};


#endif // _CONTEXT_SCENE_H_
