/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef _INFO_DOWNLOADER_H_
#define _INFO_DOWNLOADER_H_

//! local
#include "MediaItem/mediaitem.h"
#include "ultimatelyricsreader.h"

//! Qt
#include <QMap>
#include <QObject>
#include <QByteArray>
#include <QNetworkReply>

struct EchoNestBiographiesRequest;

typedef QList<UltimateLyricsProvider*> ProviderList;

/*******************************************************************************
    InfoDownloader
*******************************************************************************/
class InfoDownloader : public QObject 
{
Q_OBJECT
  public:
    InfoDownloader(QObject* parent = 0);
    
    int newRequest();
    void startDownload(MediaItem *mi);
    
    
  private:
    int                 _requestId;
    QMap<QObject*, int> _listRequests;
     
    QMap<QNetworkReply*, EchoNestBiographiesRequest*> requests_;
    
    MediaItem             *_currentMi;
    UltimateLyricsReader  *_ultimateLyricsReader;
    ProviderList          _providerList;
    ProviderList          _providerTemp;
    

  private slots:
    void ultimateLyricsParsed();

    //! artist Bio 
    void fetchArtistBio(const int id);
    void requestArtistBioFinished();
    void signalArtistImage(QByteArray bytes);

    //! artist Image
    void fetchArtistImage(const int id);
    void requestArtistImageFinished();
    
    //! similar artist info 
    void fetchArtistSimilar(const int id);
    void requestArtistSimilarFinished();

    //! album info 
    void fetchAlbumInfo(const int id);
    void parseAlbumInfo(QByteArray bytes);
    void signalAlbumImage(QByteArray bytes);
    
    //! lyrics info 
    void searchLyricsInfo(const int id);
    void fetchLyricsInfo(const int id);
    
    QString getLocale() const;

  signals:
    void ArtistBioOK(int id, QString provider, QString bio);
    void ArtistImageOK(int id, QByteArray bytes);
    void AlbumBioOK(int id, QString bio);
    void AlbumImageOK(int id, QByteArray bytes);
    void ArtistSimilarOK(int id, QStringList similarArtist);
    void LyricsOK(int id, const QString& provider, const QString& lyrics);
};

#endif // _INFO_DOWNLOADER_H_
