/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "DashBoard/dashboard.h"
#include "CollectionModel/collectionmodel.h"

#include "CollectionView/graphicitem.h"
#include "CollectionView/virtualplaylist.h"

#include "MediaItem/mediaitem.h"
#include <QDebug>



namespace GLOBAL {
    QMap<QString, QAction*>* globalActions();
}

/*******************************************************************************
    DashBoardScene
*******************************************************************************/
DashBoardScene::DashBoardScene(QWidget* parent)
{
    _parentWidget = parent;
    this->setParent(parent);
    this->setItemIndexMethod(QGraphicsScene::NoIndex);
    
    connect(CollectionModel::instance(), SIGNAL(signalPlaycountChanged()), this, SLOT(updateScene()));
    
    isGridView   = true;
    chartView    = new QAction(QIcon(), tr("Use chart view"), this);
    gridView     = new QAction(QIcon(), tr("Use grid view"), this);
    chartView->setCheckable(true);
    gridView->setCheckable(true);
    QObject::connect(chartView, SIGNAL(triggered()), this, SLOT(setChartView()));
    QObject::connect(gridView, SIGNAL(triggered()), this, SLOT(setGridView()));
    
    
    //! gestion du double click par action globale et data
    QMap<QString, QAction*> *actions = GLOBAL::globalActions();  
    QObject::connect(actions->value("itemDoubleClick"), SIGNAL(triggered()), this, SLOT(dashboardDoubleClick()), Qt::DirectConnection);
}

void DashBoardScene::setVirtualPlaylist(VirtualPlaylist  *playlist)
{
  _virtualPlaylist = playlist;
}




void DashBoardScene::updateScene()
{
    if(isActivated())
      populateScene();
}



//! ------------ startContextScene ---------------------------------------------
void DashBoardScene::populateScene()
{
    qDebug() << " ---- DashBoardScene -> populateScene ----";
    int YPos = 0;
    clear();
    if(isGridView) {
      populateMostPlayedAlbum(YPos); 
      populateMostPlayedArtist(itemsBoundingRect().height() + 40); 
    }
    else {
      populateMostPlayedAlbumChart(YPos); 
      populateMostPlayedArtistChart(itemsBoundingRect().height() + 40); 
    }
    
    //! we need to ajust SceneRect 
    int diff = _parentWidget->height() - itemsBoundingRect().height() + 40;
    int ajust  = ( diff > 0) ? diff : 0; 
    setSceneRect ( itemsBoundingRect().adjusted(-10, 0, +10, +10 + ajust) );    
}


void DashBoardScene::populateMostPlayedAlbum(int YPos)
{
   int albumRow  = 0;
   int Column    = 0;
   int nbItem = (_parentWidget->width()/160 > 2) ? _parentWidget->width()/160 : 2;

   CategorieGraphicItem *cat = new CategorieGraphicItem();
   cat->_catName = QString("Most played albums");
   cat->_width   = _parentWidget->width();
   cat->setPos( 0 ,YPos);
   addItem(cat);
    
    //! get Album List from CollectionModel
    QList<MediaItem*> albumList = CollectionModel::instance()->albumItemList;

    //! sort 
    qSort(albumList.begin(), albumList.end(), MEDIA::compareAlbumItemPlaycount);
    int i =0;
    for (; i < albumList.size(); i++) {
      
      if(albumList.at(i)->data[ALBUM::playcount].toInt() == 0 || i == 10) break;
      
      AlbumGraphicItem *_album = new AlbumGraphicItem();
      _album->media = albumList.at(i);
       _album->setToolTip(
	QString(tr("<b>playcount :</b>%1")).arg(_album->media->data[ALBUM::playcount].toInt())
	      );

      _album->setPos(4+160*Column, YPos + 50 + albumRow*170);
      addItem(_album);
      
      if(Column < (nbItem-1)) {
        Column++;
      } 
      else {
        Column = 0;
        albumRow++;
      }
    }
    
    //! si liste vide --> message
    if(i==0) {
      InfoGraphicItem *item = new InfoGraphicItem();
      item->_text = tr("No entry found");
      item->_width   = _parentWidget->width();
      item->setPos( 0 , YPos + 50);
      addItem(item);
    }
    
}


void DashBoardScene::populateMostPlayedAlbumChart(int YPos)
{
   int chartRow     = 0;
   int maxPlaycount = 0;;

   CategorieGraphicItem *cat = new CategorieGraphicItem();
   cat->_catName = QString("Most played albums");
   cat->_width   = _parentWidget->width();
   cat->setPos( 0 ,YPos);
   addItem(cat);
    
    //! get Album List from CollectionModel
    QList<MediaItem*> albumList = CollectionModel::instance()->albumItemList;

    //! sort 
    qSort(albumList.begin(), albumList.end(), MEDIA::compareAlbumItemPlaycount);
    int i =0;
    for (;i < albumList.size(); i++) {
      
      if(albumList.at(i)->data[ALBUM::playcount].toInt() == 0 || i == 10) break;
      if(i == 0)
        maxPlaycount = albumList.at(i)->data[ALBUM::playcount].toInt();

      AlbumChartGraphicItem *_album = new AlbumChartGraphicItem();
      _album->media = albumList.at(i);
      _album->_maxPlaycount = maxPlaycount;
      _album->setPos(4, YPos + 50 + chartRow*80);
      addItem(_album);

      chartRow++;
    }
    
    //! si liste vide --> message
    if(i==0) {
      InfoGraphicItem *item = new InfoGraphicItem();
      item->_text = tr("No entry found");
      item->_width   = _parentWidget->width();
      item->setPos( 0 , YPos + 50);
      addItem(item);
    }
    
}



void DashBoardScene::populateMostPlayedArtist(int YPos)
{
    int artistRow  = 0;
    int Column     = 0;
    int nbItem = (_parentWidget->width()/160 > 2) ? _parentWidget->width()/160 : 2;
    QStringList coverPathList;

    CategorieGraphicItem *cat = new CategorieGraphicItem();
    cat->_catName = QString("Most played artists");
    cat->_width   = _parentWidget->width();
    cat->setPos( 0 ,YPos);
    addItem(cat);
    
    //! get artists List from CollectionModel
    QList<MediaItem*> artistList;
    for (int j=0 ; j < CollectionModel::instance()->rootItem()->childCount(); j++ ) {
       artistList << CollectionModel::instance()->rootItem()->child(j);
     }

    //! sort 
    qSort(artistList.begin(), artistList.end(), MEDIA::compareArtistItemPlaycount);

    int i =0;
    for (;i < artistList.size(); i++) {
      
      if(artistList.at(i)->data[ARTIST::playcount].toInt() == 0 || i == 10) break;
      
      ArtistGraphicItem *_artist = new ArtistGraphicItem();
      _artist->media = artistList.at(i);
      _artist->setToolTip(
        QString(tr("<b>playcount :</b>%1")).arg(_artist->media->data[ARTIST::playcount].toInt())
       );

      _artist->setPos(4+160*Column, YPos + 50 + artistRow*170);
      addItem(_artist);
      
      
      //! ALBUM LOOP -> set Album cover path list
      for(int k = _artist->media->childCount()-1 ; k >= 0; k--) {
	MediaItem* albumItem = _artist->media->child(k);
	coverPathList.prepend(albumItem->data[ALBUM::coverpath].toString());
	if(coverPathList.size() >=6) break;
      }
      _artist->media->data[ARTIST::coverpathlist] = coverPathList;
      coverPathList.clear();
      
      if(Column < (nbItem-1)) {
        Column++;
      } 
      else {
        Column = 0;
        artistRow++;
      }
    }
    
    //! si liste vide --> message
    if(i==0) {
      InfoGraphicItem *item = new InfoGraphicItem();
      item->_text = tr("No entry found");
      item->_width   = _parentWidget->width();
      item->setPos( 0 , YPos + 50);
      addItem(item);
    }
}


void DashBoardScene::populateMostPlayedArtistChart(int YPos)
{
    int chartRow  = 0;
    int maxPlaycount;
    QStringList coverPathList;
    
    
    CategorieGraphicItem *cat = new CategorieGraphicItem();
    cat->_catName = QString("Most played artists");
    cat->_width   = _parentWidget->width();
    cat->setPos( 0 ,YPos);
    addItem(cat);
    
    //! get artists List from CollectionModel
    QList<MediaItem*> artistList;
    for (int j=0 ; j < CollectionModel::instance()->rootItem()->childCount(); j++ ) {
       artistList << CollectionModel::instance()->rootItem()->child(j);
     }

    //! sort 
    qSort(artistList.begin(), artistList.end(), MEDIA::compareArtistItemPlaycount);

    int i =0;
    for (; i < artistList.size(); i++) {
      
      if(artistList.at(i)->data[ARTIST::playcount].toInt() == 0 || i == 10) break;
      
      if(i == 0)
        maxPlaycount = artistList.at(i)->data[ARTIST::playcount].toInt();

      ArtistChartGraphicItem *_artist = new ArtistChartGraphicItem();
      _artist->media = artistList.at(i);
      _artist->_maxPlaycount = maxPlaycount;
      _artist->setToolTip(
        QString(tr("<b>playcount :</b>%1")).arg(_artist->media->data[ARTIST::playcount].toInt())
       );

      _artist->setPos(4, YPos + 50 + chartRow*80);
      addItem(_artist);

      
    //! ALBUM LOOP -> set Album cover path list
      for(int k = _artist->media->childCount()-1 ; k >= 0; k--) {
        MediaItem* albumItem = _artist->media->child(k);
        coverPathList.prepend(albumItem->data[ALBUM::coverpath].toString());
        if(coverPathList.size() >=6) break;
      }
      _artist->media->data[ARTIST::coverpathlist] = coverPathList;
      coverPathList.clear();
      
      chartRow++;
    }
    
    //! si liste vide --> message
    if(i==0) {
      InfoGraphicItem *item = new InfoGraphicItem();
      item->_text = tr("No entry found");
      item->_width   = _parentWidget->width();
      item->setPos( 0 , YPos + 50);
      addItem(item);
    }
}


//! --------------------- contextMenuEvent ------------------------------------- 
void DashBoardScene::contextMenuEvent(QGraphicsSceneContextMenuEvent *event) 
{
  _mouseGrabbedItem = this->mouseGrabberItem();
   qDebug() << " ---- DashBoardScene::contextMenuEvent";
  
  //if(!_mouseGrabbedItem) {
      chartView->setChecked(!isGridView);
      gridView->setChecked(isGridView);

      QMenu *albumMenu = new QMenu();
      albumMenu->addAction(chartView);
      albumMenu->addAction(gridView);
      albumMenu->exec(event->screenPos());
      return;
  //}
}

void DashBoardScene::setChartView()
{
   isGridView = !chartView->isChecked();
   updateScene();
}

void DashBoardScene::setGridView()
{
   isGridView = gridView->isChecked();
   updateScene();
}






//! --------------------- itemDoubleClick ------------------------------------- 
//! récupération du double click sur un item par une action globale
void DashBoardScene::dashboardDoubleClick()
{
   qDebug() << " ---- DashBoardScene:: itemDoubleClick";   

   QList<QGraphicsView *> view = this->views();
   if(view.isEmpty()) {
     //qDebug() << " ---- DashBoardScene:: View Not used";   
     return;
   }
   
   //qDebug() << " ---- DashBoardScene::itemDoubleClick";
   QMap<QString, QAction*> *actions = GLOBAL::globalActions();
  
   QGraphicsItem *gItem = qvariant_cast<QGraphicsItem *>( (actions->value("itemDoubleClick"))->data() );
   _mouseGrabbedItem = gItem;
   if(!_mouseGrabbedItem) return;
   
   switch(gItem->type()) {
     case GraphicsItem::AlbumType: playAlbum();break;
     case GraphicsItem::ArtistType: playArtist();break;
     default:break;    
   }
}



//! --------------------- playing method ---------------------------------------
void DashBoardScene::playAlbum() 
{
  qDebug() << " ---- playAlbum";
  if(!_mouseGrabbedItem) return;
  AlbumGraphicItem *item = static_cast<AlbumGraphicItem*>(_mouseGrabbedItem);
  
  QStringList _list = CollectionModel::instance()->getItemChildrenTracks(item->media);

  if(_list.isEmpty()) return;
  _virtualPlaylist->AppendFilesInPlaylist(_list);
}


void DashBoardScene::playArtist() 
{
  qDebug() << " ---- playArtist";
  if(!_mouseGrabbedItem) return;
  ArtistGraphicItem *item = static_cast<ArtistGraphicItem*>(_mouseGrabbedItem);
  
  QStringList _list = CollectionModel::instance()->getItemChildrenTracks(item->media);

  if(_list.isEmpty()) return;
  _virtualPlaylist->AppendFilesInPlaylist(_list);
}