/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh(sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _DATABASE_BUILDER_H_
#define _DATABASE_BUILDER_H_

#include <QRunnable>
#include <QObject>
#include <QStringList>
#include <QSqlDatabase>
#include <QString>
#include <QHash>

/*******************************************************************************
    DataBaseBuilder
*******************************************************************************/
// Thread thread that : 
//   - parse collection directory 
//   - read track file metada (using Taglib)
//   - write sql database with track information
class DataBaseBuilder :  public QObject, public QRunnable
{
  Q_OBJECT
  public:
    DataBaseBuilder();
    void run();
    bool isRunning() {return _isRunning;}
  
  public slots:
    void rebuildFolder(QStringList folder);
  
  private:
    QHash<QString,uint>  dbMediaFiles;
    QStringList          _folders;
    
    void insertTrack(const QString& filename, QSqlDatabase *sqlDb);
    void updateTrack(const QString& filename, QSqlDatabase *sqlDb);
    void removeTrack(const QString& filename, QSqlDatabase *sqlDb);
    
    void insertPlaylist(const QString& filename, QSqlDatabase *sqlDb);
    void updatePlaylist(const QString& filename, QSqlDatabase *sqlDb);
    void removePlaylist(const QString& filename, QSqlDatabase *sqlDb);
    
    void cleanUpDatabase(QSqlDatabase *sqlDb);
    
    void storeCoverArt(const QString& coverFilePath, const QString& trackFilename);
    void recupCoverArtFromDir(const QString& coverFilePath, const QString& trackFilename);
    
    bool                 _flagRecupCover;
    bool                 _isRunning;
    
  signals:
    void buildingFinished();
    void buildingProgress(int);
};

#endif // _DATABASE_BUILDER_H_
