/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "FileViewer/fileviewer.h"
#include "MediaItem/mediamimedata.h"

#include <QtCore>
#include <QtGui>

/*******************************************************************************
    FileViewer
*******************************************************************************/
FileViewer::FileViewer(QWidget *parent) : QWidget(parent)
{
    //! Widget Layout
    this->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);

    QVBoxLayout *vl = new QVBoxLayout(this);
    vl->setSpacing(0);
    vl->setContentsMargins(0, 0, 0, 0);
    
    //! top Frame
 /*   QWidget *topFrame = new QFrame(this);
    topFrame->setMaximumHeight(28);
    topFrame->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum);

    QHBoxLayout* topFrameLayout = new QHBoxLayout(topFrame);
    topFrameLayout->setSpacing(0);
    topFrameLayout->setContentsMargins(0, 0, 0, 0);*/
    
    QToolBar *toolBar = new QToolBar();
    //!topFrameLayout->addWidget(toolBar);
    
    
    goPrevA = new QAction(QIcon::fromTheme("go-previous", QIcon(":/images/go-previous.png")),tr("Go back"),this);
    goNextA = new QAction(QIcon::fromTheme("go-next", QIcon(":/images/go-next.png")),tr("Go forward"),this);     
    goUpA   = new QAction(QIcon::fromTheme("go-up", QIcon(":/images/go-up.png")),tr("Go Up"),this);     
    connect(goPrevA, SIGNAL(triggered()), this, SLOT(goPrev()));
    connect(goNextA, SIGNAL(triggered()), this, SLOT(goNext()));
    connect(goUpA, SIGNAL(triggered()), this, SLOT(goUp()));
    goPrevA->setEnabled( false );
    goNextA->setEnabled( false );
    goUpA->setEnabled( false );
    

    QWidget *exp1 = new QWidget(this);
    exp1->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Minimum);
    QWidget *exp2 = new QWidget(this);
    exp2->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Minimum);

    toolBar->addWidget( exp1 );
    toolBar->addAction( goPrevA );
    toolBar->addAction( goUpA );
    toolBar->addAction( goNextA );
   // toolBar->addWidget( pathString );
    toolBar->addWidget( exp2 );
    
    //! Label
    QHBoxLayout* hl1 = new QHBoxLayout();
    hl1->setSpacing(0);
    hl1->setContentsMargins(0, 0, 0, 0);

    QWidget *exp3 = new QWidget(this);
    exp3->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Minimum);
    QWidget *exp4 = new QWidget(this);
    exp4->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Minimum);
    
    pathString = new QLabel();
    hl1->addWidget( exp3 );
    hl1->addWidget( pathString );
    hl1->addWidget( exp4 );
     
    //! main Frame
    listView = new QListView();
    listView->setFrameShape(QFrame::NoFrame);
  
    QPalette palette;
    palette.setColor(QPalette::Background, Qt::white);
    palette.setColor(QPalette::Window , Qt::white);
    palette.setColor(QPalette::Foreground, Qt::white);
    palette.setColor(QPalette::Base, Qt::white);
    palette.setColor(QPalette::AlternateBase, Qt::white);
    palette.setColor(QPalette::Button, QColor(240,240,240));
  
    palette.setColor(QPalette::Text, QColor(0x315996));
    palette.setColor(QPalette::WindowText, QColor(0x315996));
    palette.setColor(QPalette::HighlightedText, Qt::white);
  
    listView->setPalette(palette);
    listView->setAutoFillBackground(true);
  
    //! widget
    listView->setEnabled(true);
    listView->setAlternatingRowColors(false);
    listView->setEditTriggers(QAbstractItemView::NoEditTriggers);
    listView->setSelectionMode(QAbstractItemView::SingleSelection); 
    listView->setUniformItemSizes(true); //!optim
    listView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    listView->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    
    //!vl->addWidget(topFrame);
    vl->addWidget(toolBar);
    vl->addLayout(hl1);
    vl->addWidget(listView);
    
    //! load indicator
    _loaded = false;

    _currentBrowserIdx = 0;
}



void FileViewer::load() 
{
    //! file model
    fileSystemModel = new QFileSystemModel();
    fileSystemModel->setReadOnly( true );
    fileSystemModel->setFilter(QDir::AllDirs | QDir::NoDotAndDotDot);
    fileSystemModel->setRootPath(QDir::rootPath());

    //! QListView
    listView->setDragEnabled(true);
    listView->setDragDropMode(QAbstractItemView::DragOnly);
    listView->setSelectionMode(QAbstractItemView::SingleSelection);
    listView->setVerticalScrollMode(QAbstractItemView::ScrollPerItem);
    listView->setModel(fileSystemModel);
    
    rootIdx = fileSystemModel->index(QDir::rootPath());
    listView->setRootIndex( rootIdx );

    listView->setViewMode(QListView::IconMode);
    listView->setGridSize(QSize(120,120)); 
    listView->setIconSize(QSize(75,75)); 
    listView->setFlow(QListView::LeftToRight);
    listView->setWordWrap(1);
 
    connect(listView, SIGNAL(activated(const QModelIndex &)), SLOT(itemActivated(const QModelIndex &)));
    _loaded = true;
    
    updateCurrentIndex(rootIdx);
}


void FileViewer::mouseMoveEvent(QMouseEvent *event) 
{
    Q_UNUSED(event)

    QString filepath = fileSystemModel->filePath(listView->currentIndex());
    
    QDir dirlist(filepath);
    QStringList filters = QStringList() << "*.mp3" << "*.ogg" << "*.flac";
    dirlist.setNameFilters(filters);
  
    QDirIterator dirIterator(dirlist,QDirIterator::Subdirectories);


    QList<QUrl> listUrl;
    QMimeData *mimedata=new QMimeData; 
  
    while (dirIterator.hasNext()) {
      dirIterator.next();
      QFileInfo fileInfo = dirIterator.fileInfo();
      if (fileInfo.isFile()) {
        //listUrl << QUrl(fileInfo.filePath().toUtf8());
        listUrl << QUrl::fromLocalFile(fileInfo.canonicalFilePath());
      }
    }
  
    if(!listUrl.isEmpty()) {
      mimedata->setUrls(listUrl);
      QDrag *drag=new QDrag(this);
      drag->setMimeData(mimedata);
      drag->start();     
    }
}

void FileViewer::itemActivated(const QModelIndex &index)
{
    qDebug() << "- FileViewer -> itemActivated idx" << index;
    
    QFileInfo name = fileSystemModel->fileInfo(index);
    if(!name.exists()) return;
    
    //! check if dir contains subdir 
    QDirIterator dirIterator(name.filePath(),QDir::Dirs|QDir::NoSymLinks|QDir::NoDotAndDotDot,QDirIterator::Subdirectories);
    if(!dirIterator.hasNext()) {
      return;
    }
    
    if(index.isValid())
      updateCurrentIndex( index );
}

void FileViewer::setPlace(const QString & place)
{
    qDebug() << "- FileViewer -> setPlace place" << place;

    if(!_loaded) load();
    
    QModelIndex idx = fileSystemModel->index(place);
    if(idx.isValid() && idx != _browserList.at(_currentBrowserIdx))
      updateCurrentIndex( idx );
}

void FileViewer::updateCurrentIndex(const QModelIndex &idx)
{
    qDebug() << "- FileViewer -> updateCurrentIndex";

    //! nouveau déclenchement --> on supprime les next
    for(int i = 0; i < _currentBrowserIdx; i++) {
      _browserList.removeAt(i);
    }
    
    listView->setRootIndex(idx);
    _browserList.prepend(idx);
    _currentBrowserIdx = 0;
    pathString->setText(fileSystemModel->fileInfo(idx).filePath());
    
    goPrevA->setEnabled(_currentBrowserIdx < _browserList.size() -1);
    goNextA->setEnabled(_currentBrowserIdx > 0);
    goUpA->setEnabled( idx !=  rootIdx);
  
    //! limite de la taille de l'historique de navigation
    if(_browserList.size() >= 20 && _currentBrowserIdx != _browserList.size() -1)
      _browserList.takeLast();
}

void FileViewer::goPrev() 
{
    if(_currentBrowserIdx < _browserList.size() -1) {
      _currentBrowserIdx++;
      listView->setRootIndex( _browserList.at(_currentBrowserIdx) );
      pathString->setText(fileSystemModel->fileInfo(_browserList.at(_currentBrowserIdx)).filePath());

      goPrevA->setEnabled(_currentBrowserIdx < _browserList.size() -1);
      goNextA->setEnabled(_currentBrowserIdx > 0);
      goUpA->setEnabled( _browserList.at(_currentBrowserIdx) !=  rootIdx);
    }
}

void FileViewer::goNext()
{
    if(_currentBrowserIdx > 0) {
      _currentBrowserIdx--;

      listView->setRootIndex( _browserList.at(_currentBrowserIdx) );
      pathString->setText(fileSystemModel->fileInfo(_browserList.at(_currentBrowserIdx)).filePath());
      
      goPrevA->setEnabled(_currentBrowserIdx < _browserList.size() -1);
      goNextA->setEnabled(_currentBrowserIdx > 0);
      goUpA->setEnabled( _browserList.at(_currentBrowserIdx) !=  rootIdx);

    }
}


void FileViewer::goUp()
{
    //listView->setRootIndex(fileSystemModel->parent(listView->rootIndex()));
    QModelIndex parent = fileSystemModel->parent(listView->rootIndex()) ;
    if( parent.isValid() )
      updateCurrentIndex( parent );
}
