/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _GRAPHIC_VIEW_WIDGET_H_
#define _GRAPHIC_VIEW_WIDGET_H_

#include "viewmode.h"

#include <QGraphicsView>
#include <QHash>
#include <QString>

class CollectionScene;
class RadioScene;
class ContextScene;
class DashBoardScene;

/*******************************************************************************
    GraphicViewWidget
*******************************************************************************/
class GraphicViewWidget : public QGraphicsView
{
  Q_OBJECT
public: 
    GraphicViewWidget(QWidget *parent);
    void setCollectionScene(CollectionScene *scene);
    void setRadioScene(RadioScene *scene);
    void setContextScene(ContextScene *scene);
    void setDashBoardScene(DashBoardScene *scene);

    void saveLastView();
    void restoreLastView();
    void updateView();

    QString             _radioFile;
    void setSearch(QString search);
        
private :
    //! all scene
    CollectionScene     *_collectionScene;
    RadioScene          *_radioScene;
    ContextScene        *_contextScene;
    DashBoardScene      *_dashboardScene;
    
    MAIN::ViewMode       _viewMode;
    QHash<MAIN::ViewMode, int>  _scrollPosition;

protected:
    void resizeEvent ( QResizeEvent * event );
  
private slots:
    void checkSlider(int);
    void switchView();
    void updateSearch(QString);
    void updateStatusBar();
    void navigateGoUp();
    void navigateGoDown();    
};

#endif // _GRAPHIC_VIEW_WIDGET_H_
