/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "histomanager.h"
#include "Database/database.h"
#include "Player/inforesolver.h"
#include "MediaItem/mediaitem.h"
#include "CollectionModel/collectionmodel.h"

#include <QDateTime>
#include <QSqlField>
#include <QSqlQuery>
#include <QSqlDriver>
#include <QVariant>

#include <QDebug>


HistoManager* HistoManager::INSTANCE = 0;
HistoManager* _HistoManager() {return HistoManager::instance();}

/*******************************************************************************
    HistoManager
*******************************************************************************/
HistoManager::HistoManager()
{
    INSTANCE  = this;

    //! set Info resolver
    _infoResolver = InfoResolver::instance();
    connect(_infoResolver, SIGNAL(playingInfoChanged()), this, SLOT(addEntry()));  

    _timer = new QTimer();
    connect(_timer, SIGNAL(timeout()), this, SLOT(addToDatabase()));  
    
    //! check database histo table 
    checkHisto();
}



//! ---------------- addEntry ------------------------------------------------------
void HistoManager::addEntry() 
{
    _timer->stop();
    
    if(_infoResolver->activeUrl().isEmpty()) return;

    _timer->start(20000);
}

//! ---------------- addToDatabase ---------------------------------------------------
void HistoManager::addToDatabase() 
{
    _timer->stop();

    MediaItem *mi     =  _infoResolver->mediaItem();
    QString   _date   =  QDateTime::currentDateTime().toString(Qt::ISODate);
    QString   _url    =  _infoResolver->activeUrl();
    QString   _name;

    if(_url.isEmpty()) return;

    if(mi->type == TYPE_TRACK) {
      _name = mi->data[TRACK::artist].toString() + " - " +
              mi->data[TRACK::album].toString() + " - " +
              mi->data[TRACK::title].toString();
    }
    else {
     _name = mi->data[STREAM::name].toString();
    }
    
    Database db;
    if (!db.connect("Histo")) return;
  
    QSqlField data("col",QVariant::String);
    QSqlQuery("BEGIN TRANSACTION;",*db.sqlDb());

    data.setValue(_url);
    _url = db.sqlDb()->driver()->formatValue(data,false);

    data.setValue(_date);
    _date = db.sqlDb()->driver()->formatValue(data,false);

    data.setValue(_name);
    _name = db.sqlDb()->driver()->formatValue(data,false);
    
    
    QSqlQuery histoQuery("SELECT `id`,`url` FROM `histo` WHERE `url`="+_url+";", *db.sqlDb());
    histoQuery.next();

    //! update histo table
    if (!histoQuery.isValid()) 
    {
      qDebug() << " ---- HISTO --> add a new entry" << _url;  
      
      QSqlQuery query("INSERT INTO `histo`(`url`,`name`,`date`) " \
                      "VALUES("+_url+","+_name+","+_date+");", *db.sqlDb()); 
      
      { //limit histo table
      QSqlQuery query("DELETE FROM `histo` WHERE `id` <= (SELECT MAX(`id`) FROM `histo`) - 250;", *db.sqlDb());
      }
    }
    else 
    {
      qDebug() << " ---- HISTO --> update an existing entry" << _url;
      
      data.setValue(histoQuery.value(0).toString());
      QString histoId = db.sqlDb()->driver()->formatValue(data,false);
      
      QSqlQuery query("UPDATE `histo` " \
                      "SET  `date`="+_date+" " \
                      "WHERE `id`="+histoId+";", *db.sqlDb());
    }
    
    //! update playcount of track is track exist
    QSqlQuery trackQuery("SELECT `id`,`interpretID`,`albumID` FROM `view_tracks` WHERE `filename`="+_url+" LIMIT 1;",*db.sqlDb());
    //QSqlQuery trackQuery("SELECT id FROM tracks WHERE filename="+_url+" LIMIT 1;",*db.sqlDb());
    trackQuery.next();
    
    if (trackQuery.isValid()) {
      int trackId  = trackQuery.value(0).toInt();
      int artistId = trackQuery.value(1).toInt();
      int albumId  = trackQuery.value(2).toInt();
      
      QSqlQuery query1("UPDATE `tracks` " \
                       "SET `playcount`=`playcount`+1 " \
                       "WHERE `id`="+QString::number(trackId)+";", *db.sqlDb());

      QSqlQuery query2("UPDATE `albums` " \
                       "SET `playcount`=`playcount`+1 " \
                       "WHERE `id`="+QString::number(albumId)+";", *db.sqlDb());

      QSqlQuery query3("UPDATE `interprets` " \
                       "SET `playcount`=`playcount`+1 " \
                       "WHERE `id`="+QString::number(artistId)+";", *db.sqlDb());
      
      //! update media Item into collection model 
      for ( int i = 0; i < CollectionModel::instance()->rootItem()->childCount(); i++ ) {
	    MediaItem* artistItem = CollectionModel::instance()->rootItem()->child(i);
	    if(artistItem->data[ARTIST::id].toInt() == artistId) {
	      artistItem->data[ARTIST::playcount] = artistItem->data[ARTIST::playcount].toInt() + 1;
	      
	      for (int j = 0; j < artistItem->childCount(); j++) {
		MediaItem* albumItem = artistItem->child(j);
   	        if(albumItem->data[ALBUM::id].toInt() == albumId) {
		  albumItem->data[ALBUM::playcount] = albumItem->data[ALBUM::playcount].toInt() + 1;
		  break;
		}
	      }
	      CollectionModel::instance()->sendPlaycountUpdated();
	      break;
	    }
      } 
    }
    
    
    QSqlQuery("COMMIT TRANSACTION;",*db.sqlDb());
}
 
 
//! ---------------- checkHisto -----------------------------------------------------
//! At startup fix de limit of the size of the histo table
void HistoManager::checkHisto()
{
    Database db;
    if (!db.connect("Histo")) return;
    
    QSqlQuery query("SELECT MAX(`id`) FROM `histo`;", *db.sqlDb());
    query.next();

    if (query.isValid()) {
      int maxId = query.value(0).toInt();
      //qDebug() << " ---- HISTO --> check table max id" << maxId;  
      if(maxId >= 100000) {
        QSqlQuery query("UPDATE `histo` SET `id` = `id` - (SELECT MIN(`id`) -1 FROM `histo`) ;", *db.sqlDb());
        query.exec();
      }
    }
}


//! ---------------- clearHistory ---------------------------------------------------
void HistoManager::clearHistory()
{
    Database db;
    if (!db.connect("Histo")) return;
    
    QSqlQuery query("DELETE FROM `histo`;", *db.sqlDb());
    query.exec();
}



