/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "mediaplaylist.h"
#include "mediaitem.h"


const QStringList m3uFilter = QStringList() << "m3u" << "m3u8";
const QStringList plsFilter = QStringList() << "pls";
const QStringList xspfFilter = QStringList() << "xspf";



//! -------- MEDIA::readM3uPlaylist --------------------------------------------
QList<MediaItem*> MEDIA::readPlaylistFile(QString filename)
{
    qDebug() << "---- MEDIA::readPlaylistFile" << QUrl(filename).toLocalFile();

    QList<MediaItem*>  list;
    if((m3uFilter).contains(QFileInfo(filename).suffix().toLower())) {
      //list = MEDIA::readM3uPlaylist(QUrl(filename).toLocalFile());
      list = MEDIA::readM3uPlaylist( QFileInfo(filename).canonicalFilePath() );
    }
    else if((plsFilter).contains(QFileInfo(filename).suffix().toLower())) {
      //list = MEDIA::readPlsPlaylist(QUrl(filename).toLocalFile());
      list = MEDIA::readPlsPlaylist( QFileInfo(filename).canonicalFilePath() );
    }
    else if((xspfFilter).contains(QFileInfo(filename).suffix().toLower())) {
      //list = MEDIA::readXspfPlaylist(QUrl(filename).toLocalFile());
      list = MEDIA::readXspfPlaylist( QFileInfo(filename).canonicalFilePath() );
    }
    
    return list;
}


//! -------- MEDIA::readM3uPlaylist --------------------------------------------
QList<MediaItem*>  MEDIA::readM3uPlaylist(QString filename)
{
    QList<MediaItem*>   list;
  
    QFile file(filename);
    if (!file.open(QFile::ReadOnly)) {
      qDebug() << "Failed to open read only file " << filename;
      return list;
    }

    QString data = QString::fromUtf8(file.readAll());
    if(data.isEmpty()) return list;
    data.replace('\r', '\n');
    QByteArray bytes = data.toUtf8();
    QBuffer buffer(&bytes);
    buffer.open(QIODevice::ReadOnly);
  
    //! metadata
    int       lenght;
    QString   title;
    QString   artist;
    
    //! main reading loop
    QString line;
    while(!buffer.atEnd()) {
      line = QString::fromUtf8(buffer.readLine()).trimmed();

      if(line.startsWith("#EXT")) 
      {
	QString info = line.section(':', 1);
	QString l = info.section(',', 0, 0);
	bool ok = false;
	int length = l.toInt(&ok);
	if (!ok) {lenght = -1; continue;}
	
	QString track_info = info.section(',', 1);
	QStringList list = track_info.split('-');

	if (list.size() <= 1) {
	  title   = track_info;
	  continue;
	}
	
	artist = list[0].trimmed();
	title = list[1].trimmed();
      }
      else if (line.startsWith('#') || line.isEmpty()) 
      {
        continue;
      }
      else 
      {
        //qDebug() << "---- readM3uPlaylist -> line " << line << "\n";
        MediaItem* mi = new MediaItem();

        //! Find the Track location
        if (line.contains(QRegExp("^[a-z]+://"))) {
          QUrl url(line);
          if (url.isValid()) {
            //qDebug() << "---- readM3uPlaylist -> url.isValid()" << url;
	    mi->type               = TYPE_STREAM;
	    mi->data[STREAM::url]  = url.toString();
	    if(!title.isEmpty()) 
	      mi->data[STREAM::name]  = title;
	    else
	      mi->data[STREAM::name]  =  mi->data[STREAM::url];
	    
          }
        }
        else if (QDir::isAbsolutePath(line)) {
          //! qDebug() << "---- readM3uPlaylist -> Absolute path : It's OK\n";
	  mi->type               = TYPE_TRACK;
	  mi->data[TRACK::url]  = QDir::fromNativeSeparators(line);

          if(!title.isEmpty()) mi->data[TRACK::title]   = title;
          if(!artist.isEmpty()) mi->data[TRACK::title]  = artist;
          if(lenght!=-1) mi->data[TRACK::duration]      = lenght;

	}
        else {
          //! qDebug() << "---- readM3uPlaylist -> Relative path : make it absolute\n";    
          QString relativePath = QDir::fromNativeSeparators(line);
          QDir playlistDir(QFileInfo(filename).absoluteDir());
          QString absolutePath = playlistDir.absoluteFilePath(relativePath);

          mi->type              = TYPE_TRACK;
          mi->data[TRACK::url]  = absolutePath;

          if(!title.isEmpty()) mi->data[TRACK::title]   = title;
          if(!artist.isEmpty()) mi->data[TRACK::title]  = artist;
          if(lenght!=-1) mi->data[TRACK::duration]      = lenght;
        } 
        
        list.append(mi);
	
	lenght   = -1;
	title    = "";
	artist   = "";
      }
    } // end while
    
  return list;
}
    
//! -------- MEDIA::readPlsPlaylist --------------------------------------------
QList<MediaItem*>  MEDIA::readPlsPlaylist(QString filename) 
{
    QList<MediaItem*>  list;

    QFile file(filename);
    if (!file.open(QFile::ReadOnly)) {
      qDebug() << "Failed to open read only file " << filename;
      return list;
    }

    MediaItem* mi = 0;
    
    while (!file.atEnd()) {
      QString line = QString::fromUtf8(file.readLine()).trimmed();
      int equals = line.indexOf('=');
      QString key = line.left(equals).toLower();
      QString value = line.mid(equals + 1);

      if (key.startsWith("file")) 
      {
        mi = new MediaItem();
	list.append(mi);

	//! Find the Track location
        if (value.contains(QRegExp("^[a-z]+://"))) {
          QUrl url(value);
          if (url.isValid()) {
            //! qDebug() << "---- readPlsPlaylist -> url.isValid()" << url;
	    mi->type               = TYPE_STREAM;
	    mi->data[STREAM::url]  = value;
	    mi->data[STREAM::name] = value;
          }
        }
        else if (QDir::isAbsolutePath(value)) {
          //! qDebug() << "---- readPlsPlaylist -> Absolute path : It's OK\n";
          mi->type               = TYPE_TRACK;
	  mi->data[TRACK::url]  = QDir::fromNativeSeparators(value);
        }
        else {
          //! qDebug() << "---- readPlsPlaylist -> Relative path : make it absolute \n";           
          QString relativePath = QDir::fromNativeSeparators(value);
          QDir playlistDir(QFileInfo(filename).absolutePath());
          QString absolutePath = playlistDir.absoluteFilePath(relativePath);
	  
	  mi->type              = TYPE_TRACK;
          mi->data[TRACK::url]  = absolutePath;        
        }       
      } // key is filename
      else if (key.startsWith("title")) 
      {
        if(mi->type == TYPE_TRACK) mi->data[TRACK::title] = value;
        else mi->data[STREAM::name] = value;
      } 
      else if (key.startsWith("length"))
      {
        if(mi->type == TYPE_TRACK) mi->data[TRACK::duration] = value.toInt();
      }
    } // fin while
    
    
    return list;
}  
//! -------- MEDIA::readXspfPlaylist -------------------------------------------
QList<MediaItem*>  MEDIA::readXspfPlaylist(QString filename) 
{
    QList<MediaItem*>  list;

    //! open file to read
    QFile file(filename);  
    if(!file.open(QIODevice::ReadOnly))
      return list;
  
    QXmlStreamReader xml(&file);  
      MediaItem* mi = 0;
    while (!xml.atEnd()) {
      QXmlStreamReader::TokenType type = xml.readNext();

      if ((type == QXmlStreamReader::StartElement) && (xml.name() == "track")) 
      {
	    mi = new MediaItem();	
      }
      else if ((type == QXmlStreamReader::StartElement) && (xml.name() == "location")) 
      {
            QString value = QString(xml.readElementText());
	    
            qDebug() << "---- readXspfPlaylist -> Find the Track location\n" << value;
            if (value.contains(QRegExp("^[a-z]+://"))) {
              QUrl url(value);
              if (url.isValid()) {
                qDebug() << "---- readXspfPlaylist -> url.isValid()" << url;
                mi->type               = TYPE_STREAM;
		mi->data[STREAM::url]  = value;
		mi->data[STREAM::name] = value;
              }
            }
            else if (QDir::isAbsolutePath(value)) {
              qDebug() << "---- readXspfPlaylist -> Absolute path : It's OK \n";
              mi->type               = TYPE_TRACK;
	      mi->data[TRACK::url]   = QDir::fromNativeSeparators(value);
	    }
	    else {
	      qDebug() << "---- readXspfPlaylist -> Relative path : make it absolute \n";           
              QString relativePath = QDir::fromNativeSeparators(value);
              QDir playlistDir(QFileInfo(filename).absolutePath());
              QString absolutePath = playlistDir.absoluteFilePath(relativePath);
              
	      mi->type              = TYPE_TRACK;
	      mi->data[TRACK::url]  = absolutePath;
	    }
        } // end location
        else if ((type == QXmlStreamReader::StartElement) && (xml.name() == "title")) 
	{
	    if(mi->type == TYPE_TRACK) mi->data[TRACK::title] = QString(xml.readElementText());
	    else mi->data[STREAM::name] = QString(xml.readElementText());
        } 
      else if ((type == QXmlStreamReader::EndElement) && (xml.name() == "track")) {
     qDebug() << "---- readXspfPlaylist -> list.append(mi)" << mi;

	if(mi) 
	  list.append(mi);
	mi = 0;
      }
    }  // End while xml end

    return list;
}  




//! ----------------------- MEDIA::savePlaylistFile ----------------------------
void MEDIA::savePlaylistFile(QString filename, QList<MediaItem*> _listMediaItem) 
{
    if((m3uFilter).contains(QFileInfo(filename).suffix().toLower())) {
      MEDIA::saveM3uPlaylist(filename, _listMediaItem);
    }
    else if((plsFilter).contains(QFileInfo(filename).suffix().toLower())) {
      MEDIA::savePlsPlaylist(filename, _listMediaItem);
    }
    else if((xspfFilter).contains(QFileInfo(filename).suffix().toLower())) {
      MEDIA::saveXspfPlaylist(filename, _listMediaItem);
    }
}


//! ----------------------- MEDIA::saveM3UPlaylist -----------------------------
void MEDIA::saveM3uPlaylist(QString filename, QList<MediaItem*> _listMediaItem) 
{
  //! get the playlist file directory
  QDir playlistDir(QFileInfo(filename).absolutePath());

  //! open file to write
  QFile file(filename);  
  if(!file.open(QIODevice::WriteOnly)) return;  
  
  file.write("#EXTM3U\n");

  foreach (MediaItem* mi, _listMediaItem) 
  {
    if (!mi) continue;
    if (mi->type != TYPE_TRACK && mi->type != TYPE_STREAM) continue;

    QString info;
    QString mediaFilePath; 
    if (mi->type == TYPE_TRACK ) {
       info = QString("#EXTINF:%1,%2 - %3\n").arg(mi->data[TRACK::duration].toString())
                                             .arg(mi->data[TRACK::artist].toString())
                                             .arg(mi->data[TRACK::album].toString());
       file.write(info.toUtf8());
    
       //! Get the path to MediaItem relative to the Playlist File directory
       mediaFilePath = mi->data[TRACK::url].toString();
       mediaFilePath = playlistDir.relativeFilePath(mediaFilePath);
    }
    else {
       info = QString("#EXTINF:\n");
       info = QString("#EXTINF:%1,%2 - %3\n").arg(0)
                                             .arg("")
                                             .arg(mi->data[STREAM::name].toString());
					     
       
       file.write(info.toUtf8());
       mediaFilePath = mi->data[STREAM::url].toString();
    }
     
    file.write(mediaFilePath.toUtf8());
    file.write("\n");
  }
  
  file.close();
}


//! ----------------------- MEDIA::savePlsPlaylist --------------------
void MEDIA::savePlsPlaylist(QString filename, QList<MediaItem*> _listMediaItem) 
{
  //! get the playlist file directory
  QDir playlistDir(QFileInfo(filename).absolutePath());
  
  //! open file to write
  QFile file(filename);  
  if(!file.open(QIODevice::WriteOnly)) return;  
  
  QTextStream textStream(&file);
  textStream << "[playlist]" << endl;
  textStream << "Version=2" << endl;
  textStream << "NumberOfEntries=" << _listMediaItem.size() << endl;

  int n = 1;
  foreach (MediaItem* mi, _listMediaItem) 
  {
    if (!mi) continue;
    if (mi->type != TYPE_TRACK && mi->type != TYPE_STREAM) continue;
        
    // write metadata
    if (mi->type == TYPE_TRACK ) {
        QString mediaFilePath = mi->data[TRACK::url].toString();
        mediaFilePath = playlistDir.relativeFilePath(mediaFilePath).toUtf8();
        
        textStream << "File" << n << "=" << mediaFilePath << endl;
        textStream << "Title" << n << "=" << mi->data[TRACK::title].toString() << endl;
        textStream << "Length" << n << "=" << mi->data[TRACK::duration].toInt()  << endl;
        ++n;
    }
    else { // TYPE_STREAM
        textStream << "File" << n << "=" << mi->data[STREAM::url].toString().toUtf8() << endl;
        textStream << "Title" << n << "=" << mi->data[STREAM::name].toString().toUtf8() << endl;
        textStream << "Length" << n << "=" << endl;
        ++n;
    }
  } // fin Foreach
}

//! ----------------------- MEDIA::saveXspfPlaylist --------------------
void MEDIA::saveXspfPlaylist(QString filename, QList<MediaItem*> _listMediaItem) 
{
  //! get the playlist file directory
  QDir playlistDir(QFileInfo(filename).absolutePath());
  
  //! open file to write
  QFile file(filename);  
  if(!file.open(QIODevice::WriteOnly)) return;  
  
  QXmlStreamWriter xml(&file);
  xml.writeStartDocument();
  xml.writeStartElement("playlist");
  xml.writeAttribute("version", "1");
  xml.writeDefaultNamespace("http://xspf.org/ns/0/");

  xml.writeStartElement("trackList");
  foreach (MediaItem* mi, _listMediaItem) 
  {
    if (!mi) continue;
    if (mi->type != TYPE_TRACK && mi->type != TYPE_STREAM) continue;
          
    xml.writeStartElement("track");
    
    QString mediaFilePath;
    // write metadata
    if (mi->type == TYPE_TRACK ) {
        mediaFilePath = mi->data[TRACK::url].toString();
        mediaFilePath = playlistDir.relativeFilePath(mediaFilePath).toUtf8();
        
        xml.writeTextElement("location", mediaFilePath);        
        xml.writeTextElement("title", mi->data[TRACK::title].toString());
        xml.writeTextElement("creator", mi->data[TRACK::artist].toString());
        xml.writeTextElement("album", mi->data[TRACK::album].toString());
        xml.writeTextElement("duration", QString::number(mi->data[TRACK::duration].toInt() * 1000));
    }
    else { // TYPE_STREAM
        mediaFilePath = mi->data[STREAM::url].toString();

        xml.writeTextElement("location", mediaFilePath);        
        xml.writeTextElement("title", mi->data[STREAM::name].toString());
    }
    
    xml.writeEndElement(); //track
  }    
  xml.writeEndElement(); //trackList
  xml.writeEndElement(); //playlist
  xml.writeEndDocument(); 
}
