/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "NowPlaying/nowplayingview.h" 
#include "Player/inforesolver.h"
#include "MediaItem/mediaitem.h"

#include <QPainter>
#include <QFontDatabase>

//! -------- NowPlayingView class ----------------------------------------------
NowPlayingView::NowPlayingView(QWidget *parent) : QFrame(parent)
{
    QPalette palette;
    palette.setColor(QPalette::Background, Qt::white);
    this->setPalette(palette);
    this->setFrameShape(QFrame::NoFrame);
    this->setFixedHeight (130);
    this->setFocusPolicy( Qt::NoFocus );
    this->setAutoFillBackground(true);  
  
    //! set Info resolver
    _infoResolver = InfoResolver::instance();
    connect(_infoResolver, SIGNAL(playingInfoChanged()), this, SLOT(update()));
    connect(_infoResolver, SIGNAL(streamInfoChanged()), this, SLOT(update()));  

    //! first init
    update();
}


//! -------- NowPlayingView::handleStateChanged ---------------------------------
void NowPlayingView::handleStateChanged() 
{
    if(_infoResolver->state() == PLAYING) {
     update();  
    }
}


//! -------- NowPlayingView::paintEvent ----------------------------------------
void NowPlayingView::paintEvent(QPaintEvent *)
{
  QPainter painter(this);
  const int width = this->width();
  QRectF rect(130, 15, width-135, 20);
	
  //! Get data 
  MediaItem *mi = _infoResolver->mediaItem();
  QPixmap   pix = _infoResolver->mediaPix();
    
  //! font
  QFontDatabase * fontdb = new QFontDatabase();
  QFont boldFont   = fontdb->font( "Segoe UI", "Bold", 10);
  QFont normalFont = fontdb->font( "Segoe UI", "Normal", 10);

    
  //! -- Playing State
  if(_infoResolver->state() == PLAYING) {
    if(mi->type == TYPE_TRACK) {
        //! paint title
        painter.setFont(boldFont);
        QFontMetrics fm( boldFont);   
        QString titleTruncated = fm.elidedText ( mi->data[TRACK::title].toString(), Qt::ElideRight, width-135 );
        painter.drawText(rect, Qt::AlignVCenter | Qt::AlignLeft,titleTruncated );

        //! paint album    
        painter.setFont(normalFont);
        QFontMetrics fm2( normalFont);   
        QString albumTruncated = fm2.elidedText ( mi->data[TRACK::album].toString(), Qt::ElideRight, width-135 );
        painter.drawText(rect.adjusted(0,25,0,25), Qt::AlignVCenter | Qt::AlignLeft,albumTruncated );

        //! paint artist    
        QString artistTruncated = fm2.elidedText ( mi->data[TRACK::artist].toString(), Qt::ElideRight, width-135 );
        painter.drawText(rect.adjusted(0,50,0,50), Qt::AlignVCenter | Qt::AlignLeft,artistTruncated );
    
        //! paint duration    
        if(mi->data[TRACK::duration].toInt() > 0) {
          QString durationText = MEDIA::durationTextFromInt(mi->data[TRACK::duration].toInt());
          QString dureeTruncated = fm2.elidedText ( durationText, Qt::ElideRight, width-135 );
          painter.drawText(rect.adjusted(0,75,0,75), Qt::AlignVCenter | Qt::AlignLeft,dureeTruncated );
        }

        //! paint pixmap (coverart)
        painter.drawPixmap(10,10,110,110, pix);
    }
    else if(mi->type == TYPE_STREAM) {
        //! paint stream name
        painter.setFont(boldFont);
        QFontMetrics fm( boldFont);   
        QString streamTruncated = fm.elidedText ( mi->data[STREAM::name].toString(), Qt::ElideRight, width-135 );
        painter.drawText(rect, Qt::AlignVCenter | Qt::AlignLeft,streamTruncated );

        //! paint title    
        painter.setFont(normalFont);
        QFontMetrics fm2( normalFont);   
        QString titleTruncated = fm2.elidedText ( mi->data[STREAM::title].toString(), Qt::ElideRight, width-135 );
        painter.drawText(rect.adjusted(0,25,0,25), Qt::AlignVCenter | Qt::AlignLeft,titleTruncated );

        //! paint album    
        QString albumTruncated = fm2.elidedText ( mi->data[STREAM::album].toString(), Qt::ElideRight, width-135 );
        painter.drawText(rect.adjusted(0,50,0,50), Qt::AlignVCenter | Qt::AlignLeft,albumTruncated );

        //! paint artist    
        QString artistTruncated = fm2.elidedText ( mi->data[STREAM::artist].toString(), Qt::ElideRight, width-135 );
        painter.drawText(rect.adjusted(0,75,0,75), Qt::AlignVCenter | Qt::AlignLeft,artistTruncated );

        //! paint pixmap (coverart)
        painter.drawPixmap(8,8,110,110, pix); 
    }
  }
  //! -- Stopped State
  else 
  {
    painter.setFont(boldFont);
    QFontMetrics fm( boldFont);   
    QString text = fm.elidedText ( tr("Playlist"), Qt::ElideRight, width-135 );
    painter.drawText(rect, Qt::AlignVCenter | Qt::AlignLeft,text );
    
    painter.setFont(normalFont);
    QFontMetrics fm2( normalFont);   
    QString text2 = fm2.elidedText ( tr("Playing Stopped"), Qt::ElideRight, width-135 );
    painter.drawText(rect.adjusted(0,25,0,25), Qt::AlignVCenter | Qt::AlignLeft,text2 );
    
    painter.drawPixmap(10,10,64,64, QPixmap(":/icon/64x64/yarock.png"));
   }
   painter.end();
}