/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "Player/inforesolver.h"
#include "MediaItem/mediaitem.h"
#include "Player/player.h"
#include "utilities.h"

#include <QMap>
#include <QAction>

namespace GLOBAL {
    QMap<QString, QAction*>* globalActions();
}

InfoResolver* InfoResolver::INSTANCE = 0;
InfoResolver* _InfoResolver() {return InfoResolver::instance();}
/*******************************************************************************
    InfoResolver
*******************************************************************************/
InfoResolver::InfoResolver(QObject* parent)
{
    //! global instance
    INSTANCE = this;

    //! set player
    _player = Player::instance();
    connect(_player, SIGNAL(trackChanged(QString)), this, SLOT(updateSource(QString)));  
    connect(_player, SIGNAL(metaDataChanged()), this, SLOT(updateMetaData())); 
    
    connect(_player, SIGNAL(stateChanged(Phonon::State,Phonon::State)), 
	    this, SLOT(stateChanged(Phonon::State,Phonon::State)), Qt::QueuedConnection);
  
  
    connect(GLOBAL::globalActions()->value("lastfmloveNowPlaying"), SIGNAL(triggered()), this, SLOT(sendLastFmLove()));

    //! init
    _mediaItem       = new MediaItem();
    _mediaItem->type = TYPE_UNKWON;
    _url             = "";
    _state           = STOPPED;
    _pix             = QPixmap();
}

//! -------- InfoResolver::stateChanged ----------------------------------------
void InfoResolver::stateChanged(Phonon::State newState, Phonon::State /*oldState*/)
{
    E_InfoState _oldState = _state;
    switch (newState) {
      case Phonon::LoadingState:_state          = STOPPED;break;
      case Phonon::StoppedState:_state          = STOPPED;break;
      case Phonon::PlayingState:_state          = PLAYING;break;
      case Phonon::BufferingState :_state       = STOPPED;break;
      case Phonon::PausedState:_state           = STOPPED;break;
      case Phonon::ErrorState: _state           = STOPPED;break;
      default:_state           = STOPPED;break;
    }  
    
    if(_oldState != PLAYING && _state == PLAYING) {
      emit playingInfoChanged();
      //qDebug() << "InfoResolver -> emit playingInfoChanged";
    }
}

//! -------- InfoResolver::updateSource ----------------------------------------
void InfoResolver::updateSource(QString filename)
{
   //qDebug() << "InfoResolver -> updateSource" << filename;

  _url        = filename;
  _pix        = QPixmap();
    
  if(_url.isEmpty()) {
      _state           = STOPPED;
      _mediaItem->type = TYPE_UNKWON;
      emit playingInfoChanged();
      return;
    }
  
  //! ----- Local File Case ----------------------------------------------------
  if(MEDIA::isUrlLocal(filename)) 
  {
    _mediaItem->type = TYPE_TRACK;
    
    _mediaItem = MEDIA::FromDataBase("NowPlayingConnection", _url);
    if(!_mediaItem)
      _mediaItem     = MEDIA::FromLocalFile(QFileInfo(_url).absoluteFilePath());

    QString coverpath = UTIL::CONFIGDIR + "/albums/" + MEDIA::getCoverName(_mediaItem);
    _pix = QPixmap::fromImage(QImage(coverpath));
    
    if (_pix.isNull()) _pix = MEDIA::getPixmapFromMediaUrl(filename, QSize(110,110));
    if (_pix.isNull()) _pix = QPixmap(":/images/emptycoverblue110x110.png");
  }
  //! ----- Remote Url  --------------------------------------------------------
  else  
  {
    _mediaItem->type = TYPE_STREAM;
        
    //! for stream item, stream short name = _mediaItem->filename
    _mediaItem->data[STREAM::url]    = _url;
    _mediaItem->data[STREAM::name]   = _url;
    _mediaItem->data[STREAM::title]  = "";
    _mediaItem->data[STREAM::album]  = "";
    _mediaItem->data[STREAM::artist] = "";
    _pix                 = QPixmap(":/images/media-url-110x110.png");
  }
  
  if(_player->playerState() == Phonon::PlayingState) {
    emit playingInfoChanged();
    //qDebug() << "InfoResolver -> emit playingInfoChanged";
  }
}



//! -------- InfoResolver::updateMetaData --------------------------------------
void InfoResolver::updateMetaData()
{
  if(_state == STOPPED || _mediaItem->type != TYPE_STREAM) return;

  QMap<QString, QString> metaData = _player->currentMetaData();

  _mediaItem->data[STREAM::title]  = metaData.value("TITLE");
  _mediaItem->data[STREAM::album]  = metaData.value("ALBUM");
  _mediaItem->data[STREAM::artist] = metaData.value("ARTIST");

  emit streamInfoChanged();        
}


//!---------- InfoResolver::sendLastFmLove -------------------------------------
void InfoResolver::sendLastFmLove() 
{
    if(_state == PLAYING && _mediaItem->type == TYPE_TRACK) {
        emit signal_lastFmLove(_mediaItem->data[TRACK::artist].toString(), _mediaItem->data[TRACK::title].toString() );
    }
}
