/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
 
#include "player.h"

#include <QFileInfo>
#include <phonon/mediaobject.h>
#include <phonon/path.h>
#include <phonon/audiooutput.h>
#include <phonon/mediasource.h>
#include <phonon/backendcapabilities.h>

Player* Player::INSTANCE = 0;
Player* _Player() {return Player::instance();}
/*******************************************************************************
    Player
*******************************************************************************/
Player::Player()
{
    INSTANCE = this;
  
    _mediaObject = new Phonon::MediaObject(this);
    _audioOutput = new Phonon::AudioOutput(Phonon::MusicCategory, this);

    // By default tick every 1 second
    _mediaObject->setTickInterval(250);
    
    //! TEST 0.0.50c
    // Get the next track when there is 2 seconds left on the current one
    // in case of playing track from track view
    _mediaObject->setPrefinishMark( 2000 );
    _mediaObject->setTransitionTime(0); //! GAPLESS/CROSSFADE
    
    connect(_mediaObject,SIGNAL(finished()),this,SIGNAL(trackFinished()));
    connect(_mediaObject,SIGNAL(aboutToFinish()),this,SIGNAL(trackAboutToFinish()));
    
    connect(_mediaObject,SIGNAL(stateChanged(Phonon::State,Phonon::State)),
    this,SIGNAL(stateChanged(Phonon::State,Phonon::State)),Qt::DirectConnection);
    
    connect(_mediaObject,SIGNAL(tick(qint64)),this,SIGNAL(trackPositionChanged(qint64)));
    connect(_mediaObject,SIGNAL(totalTimeChanged(qint64)),this,SIGNAL(trackTotalTimeChanged(qint64)));
    
    connect(_mediaObject,SIGNAL(currentSourceChanged(Phonon::MediaSource)),this,SLOT(emitTrackChanged()));
    connect(_mediaObject, SIGNAL(metaDataChanged()), this,SIGNAL(metaDataChanged()));

    _phononPath     = Phonon::createPath(_mediaObject,_audioOutput);
    _audioEqualizer = 0;
}

Player::~Player()
{
    delete _audioOutput;
    delete _mediaObject;
}


//! ------ set source method ---------------------------------------------------
bool Player::setTrack(const QString &fileName)
{
   if (!QFileInfo(fileName).isFile() && !fileName.isEmpty()) return false;
  qDebug() << "Player -> setTrack " << fileName;

  _mediaObject->stop();
  _mediaObject->clearQueue();
  
  _mediaObject->setCurrentSource( Phonon::MediaSource(fileName) );
  _mediaObject->play();  
  return true;
}

bool Player::setTrack(const QUrl &url)
{
  if (!url.isValid() && !url.isEmpty() ) return false;
  qDebug() << "Player -> setTrack " << url.toString();
  
  _mediaObject->stop();
  _mediaObject->clearQueue();
  _mediaObject->setCurrentSource( Phonon::MediaSource(url) );
  _mediaObject->play();
  return true;
}

//! ------ MEdia Source Queue management ---------------------------------------
void Player::setNextTrack(const QString &fileName)
{
    if (!QFileInfo(fileName).isFile() && !fileName.isEmpty()) return;
    qDebug() << "Player -> setNextTrack " << fileName;
    _mediaObject->enqueue( Phonon::MediaSource(fileName) );
}

void Player::setNextTrack(const QUrl &url)
{
    if (!url.isValid() && !url.isEmpty() ) return;
    qDebug() << "Player -> setNextTrack " << url.toString();
    _mediaObject->enqueue( Phonon::MediaSource(url) );
}


//! ------ stop ----------------------------------------------------------------
void Player::stop()
{
    //qDebug() << "Player -> stop";
    //_mediaObject->stop();
    _mediaObject->clear();
}


//! ------ current source & metadata -------------------------------------------
QString Player::currentMediaSource()
{
 /* if (_mediaObject->currentSource().type()==Phonon::MediaSource::Invalid) {
    qDebug() << "Player -> currentMediaSource MediaSource::Invalid";
    return QString();
  }*/
 
  if (_mediaObject->currentSource().type() == Phonon::MediaSource::LocalFile)
    return _mediaObject->currentSource().url().toLocalFile();
  else 
    return _mediaObject->currentSource().url().toString();
}

const QMap<QString, QString> & Player::currentMetaData() const
{
    return _mediaObject->metaData();
}

//! ------ emitTrackChanged ----------------------------------------------------
void Player::emitTrackChanged()
{
    //qDebug() << "Player -> emitTrackChanged" << currentMediaSource();
    emit trackChanged(currentMediaSource());
}


//! ------ Equalizer method ----------------------------------------------------
bool Player::isEqualizerAvailable()
{
    QList<Phonon::EffectDescription> effects = Phonon::BackendCapabilities::availableAudioEffects();
    
    foreach (Phonon::EffectDescription effect, effects) {
      if(effect.name()=="KEqualizer")
	return true;
     }
     return false; 
}

void Player::addEqualizer()
{
  QList<Phonon::EffectDescription> descriptions = Phonon::BackendCapabilities::availableAudioEffects();
  
  foreach (Phonon::EffectDescription description, descriptions)  {
    if(description.name() == "KEqualizer")
    {
      _audioEqualizer = new Phonon::Effect(description, this);
      _phononPath.insertEffect(_audioEqualizer);
    }
  }
}
  
  
void Player::removeEqualizer()
{
  if(_audioEqualizer != 0) {
      _phononPath.removeEffect(_audioEqualizer);
      _audioEqualizer = 0;
   }
}
  
