/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef _PLAYER_H_
#define _PLAYER_H_

#include <QObject>
#include <QUrl>
#include <QString>
#include <phonon/mediasource.h>
#include <phonon/audiooutput.h>
#include <phonon/mediaobject.h>
#include <phonon/effect.h>


/*******************************************************************************
    Player
*******************************************************************************/
class Player: public QObject
{ 
Q_OBJECT
    static Player         *INSTANCE;

  public:
    Player();
    ~Player();
    static Player* instance() { return INSTANCE; }
    
    //! method
    QString  currentMediaSource();
    const QMap<QString, QString> &currentMetaData() const;
        
    Phonon::MediaSource currentSource() { return _mediaObject->currentSource(); }
    Phonon::State playerState() { return _mediaObject->state(); }
    QString errorString() { return _mediaObject->errorString(); }
    Phonon::MediaObject* mediaObject() { return _mediaObject; }
    Phonon::AudioOutput* audioOutput() { return _audioOutput; }
    
    //! equalizer method 
    bool isEqualizerAvailable();
    void addEqualizer();
    void removeEqualizer();
    Phonon::Effect* equalizer() {return _audioEqualizer;}

  public slots:
    void play() { _mediaObject->play(); }
    void pause() { _mediaObject->pause(); }
    void stop();
    bool setTrack(const QString &fileName);
    bool setTrack(const QUrl &url);
    void setNextTrack(const QString &fileName);
    void setNextTrack(const QUrl &url);    

  private slots:
    void emitTrackChanged();

  private:
    Phonon::MediaObject   *_mediaObject;
    Phonon::AudioOutput   *_audioOutput;
    Phonon::Path          _phononPath;
    Phonon::Effect        *_audioEqualizer;

  signals:
    void trackChanged(QString filename);    // before loading when current soure is set
    void metaDataChanged(); // after loading when metadata resolved
    void trackFinished();
    void trackFinished(const QString&);
    void trackAboutToFinish();
    void stateChanged(Phonon::State newState, Phonon::State oldState);
    void trackPositionChanged(qint64 newPos);
    void trackTotalTimeChanged(qint64 newTotalTime);
};

#endif // _PLAYER_H_
