/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "playlistdbwriter.h"
#include "playlistmodel.h"
#include "Database/database.h"
#include "MediaItem/mediaitem.h"

#include <QtCore>
#include <QtSql/QSqlDatabase>
#include <QtSql/QSqlQuery>
#include <QtSql/QSqlDriver>
#include <QtSql/QSqlField>
#include <QtSql/QSqlError>
#include <QtSql/QSqlResult>


#include <QStringList>


//! ----------------------- PlaylistDbWriter -----------------------------------
PlaylistDbWriter::PlaylistDbWriter()
{
    _playlistModel = PlaylistModel::instance();
    _isRunning     = false;
}

//! ----------------------- run ------------------------------------------------
void PlaylistDbWriter::run()
{
    _isRunning     = true;
    
    //! playlist data for Database
    QDateTime date   = QDateTime::currentDateTime();
    uint mtime       = date.toTime_t();
    QString pname    = "playlist-" + date.toString("dd-MM-yyyy-hh:mm:ss"); 
    QString fname    = QString(QCryptographicHash::hash(pname.toUtf8().constData(), QCryptographicHash::Sha1).toHex());
    int favorite     = 0;
    
    QSqlField data("col",QVariant::String);

    Database db;
    if (!db.connect("PlaylistDbWriter")) return;
    
    qDebug() << "--- PlaylistDbWriter -> insert playlist" << pname;
    QSqlQuery("BEGIN TRANSACTION;",*db.sqlDb());
    
    //! PLAYLIST part in database
    {
      QSqlQuery query(*db.sqlDb());
      query.prepare("INSERT INTO `playlists`(`filename`,`name`,`icon`,`rules`,`type`,`favorite`,`mtime`)" \
                    "VALUES(?," \
                    "       ?," \
                    "       ?," \
                    "       ?," \
                    "       ?," \
                    "       ?," \
                    "       ?);");
      query.addBindValue(fname);
      query.addBindValue(pname);
      query.addBindValue(":/images/media-playlist-110x110.png"); //! icon
      query.addBindValue(""); //! rules
      query.addBindValue((int) T_DATABASE);
      query.addBindValue(favorite);
      query.addBindValue(mtime);
      query.exec();
    }
    
    data.setValue(fname);
    QString fnameDB = db.sqlDb()->driver()->formatValue(data,false);
	
    //! get Url from playlist 
    QStringList listUrl;
    for (int i = 0; i < _playlistModel->rowCount(QModelIndex()); i++) {
       listUrl << _playlistModel->MediaItemAt(i)->data[TRACK::url].toString(); //! TRACK or STREAM
    }
    
    foreach (const QString &url, listUrl) {
      if(url.isEmpty()) continue;
      QString itemUrlName   = url; //QFileInfo(url).filePath().toUtf8();
      QString itemName      = QFileInfo(url).baseName();

      //! Playlist Item part in database
      {
        QSqlQuery itemQuery(*db.sqlDb());
        qDebug() << "--- PlaylistDbWriter -> insert playlistitem itemUrlName:" << itemUrlName;

        itemQuery.prepare("INSERT INTO `playlistitem`(`url`,`name`,`playlist`)" \
                          "VALUES(?," \
                          "       ?," \
                          "       (SELECT `id` FROM `playlists` WHERE `filename`="+fnameDB+"));");
        itemQuery.addBindValue(itemUrlName);
        itemQuery.addBindValue(itemName);
        itemQuery.exec();
      }  

    } // end foreach url into playlist
    
    
    QSqlQuery("COMMIT TRANSACTION;",*db.sqlDb());
    _isRunning     = false;    
    emit playlistSaved();
}
