/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "playlistmodel.h"
#include "MediaItem/mediaitem.h"
#include "MediaItem/mediamimedata.h"

PlaylistModel* PlaylistModel::INSTANCE = 0;
PlaylistModel* _PlaylistModel() {return PlaylistModel::instance();}

/*******************************************************************************
    PlaylistModel
*******************************************************************************/
PlaylistModel::PlaylistModel(QWidget *parent) : QAbstractListModel(parent) 
{
    INSTANCE             = this;
    _playingMediaItem    =  0;
    _requestedMediaItem  =  0;
    _requestedRow        = -1;
}


//!------------PlaylistModel::rowCount -----------------------------------------
// QAbstractListModel implementation
int PlaylistModel::rowCount(const QModelIndex & /* parent */) const 
{
    return _listMediaItem.size();
}

//!------------PlaylistModel::data ---------------------------------------------
// QAbstractListModel implementation
QVariant PlaylistModel::data(const QModelIndex &index, int role) const
{
Q_UNUSED(index)
Q_UNUSED(role)

    return QVariant();
}

//!------------PlaylistModel::updateMediaItemBroken ----------------------------
void PlaylistModel::updateMediaItemBroken(const QString & filename)
{
  qDebug() << " --- PlaylistModel->updateMediaItemBroken" << filename;
  if(_requestedMediaItem) {
      if(MEDIA::urlForMediaItem(_requestedMediaItem) == filename) {
        _requestedMediaItem->data[TRACK::isBroken]  = true;
        _requestedMediaItem->data[TRACK::isPlaying] = false;

        int row = _listMediaItem.indexOf(_requestedMediaItem);
        QModelIndex newIndex = index(row, 0, QModelIndex());
        emit dataChanged(newIndex, newIndex);   
        return;
      }
  }

  if(_playingMediaItem) {
    if(MEDIA::urlForMediaItem(_playingMediaItem) == filename) {
      _playingMediaItem->data[TRACK::isBroken]  = true;
      _playingMediaItem->data[TRACK::isPlaying] = false;

      int row = _listMediaItem.indexOf(_playingMediaItem);
      QModelIndex newIndex = index(row, 0, QModelIndex());
      emit dataChanged(newIndex, newIndex);  
      return;
    }
  }

  foreach(MediaItem *mi, _listMediaItem) {
    if(MEDIA::urlForMediaItem(mi) == filename) {
      mi->data[TRACK::isBroken]  = true;
      mi->data[TRACK::isPlaying] = false;    
      
      int row = _listMediaItem.indexOf(mi);
      QModelIndex newIndex = index(row, 0, QModelIndex());
      emit dataChanged(newIndex, newIndex);  
      break;
    }
  }
}



void PlaylistModel::updateMediaItemPlaying(const QString & filename)
{
    //qDebug() << " --- PlaylistModel->updateMediaItemPlaying";

    //! unset previous playing item
    stopPlayingMediaItem();
    
    //! set new playing item
    if(_requestedMediaItem != 0) {
      //qDebug() << " --- PlaylistModel->updateMediaItemPlaying _requestedMediaItem" << _requestedMediaItem;
      
      QString url = MEDIA::urlForMediaItem(_requestedMediaItem);
      qDebug() << " --- PlaylistModel->updateMediaItemPlaying url" << url;

      if(url == filename) {
        _playingMediaItem = _requestedMediaItem;
        _playingMediaItem->data[TRACK::isPlaying] = true;
        _playingMediaItem->data[TRACK::isPlayed]  = true;

        int row = _listMediaItem.indexOf(_playingMediaItem);
        QModelIndex newIndex = index(row, 0, QModelIndex());
        emit dataChanged(newIndex, newIndex);   
        return;
      }
    }

    foreach(MediaItem *mi, _listMediaItem) {
      QString url = MEDIA::urlForMediaItem(mi);
      if(url == filename) {
        _playingMediaItem = mi;
        _playingMediaItem->data[TRACK::isPlaying] = true;
        _playingMediaItem->data[TRACK::isPlayed]  = true;

        int row = _listMediaItem.indexOf(_playingMediaItem);
        QModelIndex newIndex = index(row, 0, QModelIndex());
        emit dataChanged(newIndex, newIndex);   
        break;
      }
    }
}

void PlaylistModel::stopPlayingMediaItem()
{
      //! unset previous playing item
    if(_playingMediaItem) {
      _playingMediaItem->data[TRACK::isPlaying] = false;
      
      int row = _listMediaItem.indexOf(_playingMediaItem);
      QModelIndex newIndex = index(row, 0, QModelIndex());
      emit dataChanged(newIndex, newIndex);   

      _playingMediaItem =  0;
    }
  
}






//!------------PlaylistModel::setRequestedMediaItem ------------------------------------
void PlaylistModel::setRequestedMediaItem(int row)
{
  if (!rowExists(row)) return;

  _requestedMediaItem = _listMediaItem.at(row);
  _requestedRow = row;
}

QString PlaylistModel::getRequestedUrl() const 
{
    if(!_requestedMediaItem) return QString();
    
    return MEDIA::urlForMediaItem(_requestedMediaItem);
}


//!------------PlaylistModel::skipBackward -------------------------------------
// Specific method
void PlaylistModel::skipBackward()
{
   int prevRow = _requestedRow - 1;
   if (rowExists(prevRow))
     setRequestedMediaItem(prevRow);
}

//!------------PlaylistModel::skipForward --------------------------------------
// Specific method
void PlaylistModel::skipForward() 
{
   int nextRow = _requestedRow + 1;
   if (rowExists(nextRow))
       setRequestedMediaItem(nextRow);
}

//!------------PlaylistModel::MediaItemAt --------------------------------------
// Specific method
MediaItem* PlaylistModel::MediaItemAt(int row) const 
{
    if (rowExists(row)) {
      return _listMediaItem.at(row);
    }
    return 0;
}

//!------------PlaylistModel::getActiveMediaItem -------------------------------
// Specific method
MediaItem* PlaylistModel::getPlayingMediaItem() const
{
    if(_playingMediaItem)
      return _playingMediaItem;
    return 0;
}

int PlaylistModel::getPlayingRow() 
{
    if(_playingMediaItem)
      return _listMediaItem.indexOf(_playingMediaItem);
    else
      return -1;
}



//!------------PlaylistModel::addMediaItem -------------------------------------
// Specific method
void PlaylistModel::addMediaItem(MediaItem* mediaitem) 
{
    //! no duplicates
    bool found = false;
    foreach (const MediaItem *mi, _listMediaItem) {
      if(mediaitem->data[TRACK::url].toString() == mi->data[TRACK::url].toString()) {
        found = true;
        break;
      }
    }
    
    if (!found) {
        _mutex.lock();
        beginInsertRows(QModelIndex(), _listMediaItem.size(), _listMediaItem.size());
        _listMediaItem << mediaitem;
        endInsertRows();
       _mutex.unlock();
     
        emit layoutChanged();
    }
}


//!------------PlaylistModel::getListDuration ----------------------------------
QString PlaylistModel::getListDuration()
{ 
  return MEDIA::durationTextFromMediaList(_listMediaItem); 
}

//!------------PlaylistModel::clear --------------------------------------------
void PlaylistModel::clear()
{
    qDeleteAll(_listMediaItem.constBegin(), _listMediaItem.constEnd());
      
    _listMediaItem.clear();
    _playingMediaItem     =  0;
    _requestedMediaItem   =  0;
    _requestedRow         = -1;
  
    emit layoutChanged();
    emit dataChanged(QModelIndex(), QModelIndex());  
}



//!------------PlaylistModel::removeRows ---------------------------------------
bool PlaylistModel::removeRows(int position, int count, const QModelIndex & /*parent*/) 
{
    //! version 0.0.43 suppression des beginRemoveRows & endRemoveRows car crash
    //qDebug() << "--- PlaylistModel-->removeRow position rows" << position;
    //qDebug() << "--- PlaylistModel-->removeRow position count" << count;
    if( position + count - 1 >= _listMediaItem.size()) return false;
    
    _mutex.lock();
    //beginRemoveRows(QModelIndex(), position, position + count - 1);
    for (int row = 0; row < count; row++) {
      delete _listMediaItem.takeAt(position);
    }
    //endRemoveRows();
    _mutex.unlock();

    emit dataChanged(QModelIndex(), QModelIndex());  
    return true;
}




//! DRAG AND DROP
//!------------PlaylistModel::supportedDropActions -----------------------------
Qt::DropActions PlaylistModel::supportedDropActions() const 
{
    return Qt::CopyAction | Qt::MoveAction; // deplace les donn�es de la source vers la cible
    //return Qt::CopyAction;
}

Qt::ItemFlags PlaylistModel::flags(const QModelIndex &index) const
{
    Qt::ItemFlags defaultFlags = QAbstractListModel::flags(index);
    if (index.isValid()) {
        //return ( defaultFlags | Qt::ItemIsDropEnabled | Qt::ItemIsDragEnabled );
        return ( defaultFlags | Qt::ItemIsDropEnabled);

    } else
        return Qt::ItemIsDropEnabled | defaultFlags;
}

//!------------PlaylistModel::mimeTypes ----------------------------------------
// Encoding Exported Data with mimeTypes
QStringList PlaylistModel::mimeTypes() const
{
    QStringList types;
    types << MEDIA_MIME;
    types << "text/uri-list"; //we do accept urls
    return types;
}

//!------------PlaylistModel::mimeData -----------------------------------------
QMimeData* PlaylistModel::mimeData( const QModelIndexList &indexes ) const 
{
    MediaMimeData* mime = new MediaMimeData();

    foreach( const QModelIndex &index, indexes ) {
        int row = index.row();
        
        if (row >= 0 && row < _listMediaItem.size())
            mime->addMediaItem( _listMediaItem.at(row) );
    }

    return mime;
}


//!------------PlaylistModel::dropMimeData -------------------------------------
// --> Inserting Dropped Data into a Model
bool PlaylistModel::dropMimeData(const QMimeData *data,
                                 Qt::DropAction action, int row, int column,
                                 const QModelIndex &parent) 
{
    //qDebug() << "--- PlaylistModel --> start Drop MimeData";
    QAbstractListModel::dropMimeData(data, action, row, column, parent);
    
    if (action == Qt::IgnoreAction) return true;
    
    if (data->hasFormat(MEDIA_MIME))
    {
      //qDebug() << "--- PlaylistModel::dropMimeData --> data->hasFormat(MEDIA_MIME)";
      const MediaMimeData* mediaMimeData = dynamic_cast<const MediaMimeData*>( data );
      if(!mediaMimeData ) return false;

      foreach(MediaItem *mi, mediaMimeData->getMediaItem())
      {
        if(MEDIA::isPlaylistFile(mi->data[STREAM::url].toString()) ||
           MEDIA::isShoutCastUrl(mi->data[STREAM::url].toString()) )
        {
          QList<QUrl> listUrl;
          listUrl  << QUrl( mi->data[STREAM::url].toString() );
          //qDebug() << "--- PlaylistModel::dropMimeData --> data->hasUrls()";
          emit signalAddUrls(listUrl);
        }
        else {
          addMediaItem(MEDIA::miCopy(mi));
        }
      }

      emit layoutChanged();
    }
    else if( data->hasUrls() )
    {
      QList<QUrl> listUrl = data->urls();
      //qDebug() << "--- PlaylistModel::dropMimeData --> data->hasUrls()";
      emit signalAddUrls(listUrl);
    }
    else {
     return false; 
    }
    
    return true;
}


//!------------PlaylistModel::rowForMediaItem ----------------------------------
int PlaylistModel::rowForMediaItem(MediaItem* mediaitem)
{
    return _listMediaItem.indexOf(mediaitem);
}



//!------------PlaylistModel::move ---------------------------------------------
void PlaylistModel::move(QModelIndexList &indexes, bool up) 
{
    //qDebug() << "--- PlaylistModel::move";

    QList<MediaItem*> movedMediaItems;

    foreach (QModelIndex index, indexes) {
        int row = index.row();
        qDebug() << "--- PlaylistModel::move    index row" << row;
        MediaItem *mi = MediaItemAt(row);
        if (mi)
            movedMediaItems << mi;
    }

    foreach (MediaItem *mi, movedMediaItems) {

        int row = rowForMediaItem(mi);
        // qDebug() << "track row" << row;
	
	//! COPY
	MediaItem* newMi = new MediaItem();
	newMi->type = mi->type;
	for(int i=0;i< MAX_FIELD; i++)
	  newMi->data[i] = mi->data[i];
	
	//! Item is deleted
        removeRows(row, 1, QModelIndex()); 

        if (up) row--;
        else row++;

	beginInsertRows(QModelIndex(), row, row);
        _listMediaItem.insert(row, newMi);
        endInsertRows();
    }
    emit layoutChanged();
}


//!------------PlaylistModel::unplayedMedia --------------------------------------
QList<MediaItem*>  PlaylistModel::unplayedMediaItem()
{
    //qDebug() << "--- PlaylistModel::unplayedMediaItem";

    QList<MediaItem*>  list;
    foreach (MediaItem *mi, _listMediaItem) {
      if(mi->data[TRACK::isPlayed].toBool() == false) //! ok for STREAM 
        list << mi;
    }  
    
    return list;
}



