/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _PLAYLISTMODEL_H
#define _PLAYLISTMODEL_H

#include <QtGui>
#include <QMutex>

class MediaItem;


/*******************************************************************************
    PlaylistModel
*******************************************************************************/
class PlaylistModel : public QAbstractListModel 
{
Q_OBJECT
    static PlaylistModel* INSTANCE;  

  public:
    PlaylistModel(QWidget *parent);
    
    //! return global instance
    static PlaylistModel* instance() { return INSTANCE; }    
    
    //! inherited from QAbstractListModel
    int rowCount(const QModelIndex &parent = QModelIndex()) const;
    QVariant data(const QModelIndex &index, int role) const;
    bool removeRows(int position, int count, const QModelIndex &parent);

    //! drag and drop
    Qt::ItemFlags flags(const QModelIndex &index) const;
    QStringList mimeTypes() const;
    Qt::DropActions supportedDropActions() const;
    QMimeData* mimeData( const QModelIndexList &indexes ) const;
    bool dropMimeData(const QMimeData *data,
                      Qt::DropAction action, int row, int column,
                      const QModelIndex &parent);

    //! custom methods
    bool rowExists( int row ) const { return (( row >= 0 ) && ( row < _listMediaItem.size() ) ); }
    int rowForMediaItem(MediaItem* _mediaItem);
    void move(QModelIndexList &indexes, bool up);

    MediaItem* MediaItemAt( int row ) const;
    QString getListDuration();
    
    //! get Unplayed mediaItem
    QList<MediaItem*> unplayedMediaItem();
    
    //! broken MediaItem method
    void updateMediaItemBroken(const QString&);
    void updateMediaItemPlaying(const QString&);
    
    //! Requested MediaItem method
    void setRequestedMediaItem(int row);
    QString getRequestedUrl() const;
    MediaItem* getPlayingMediaItem() const;
    int getPlayingRow();
    void stopPlayingMediaItem();
  
  public slots:
    void addMediaItem(MediaItem* mediaitem);
    void clear();
    void skipBackward();
    void skipForward();
  
  signals:
    void signalAddUrls(const QList<QUrl> listUrl );

  private:
    int                  _requestedRow;
    MediaItem*           _playingMediaItem;
    MediaItem*           _requestedMediaItem;
    QList<MediaItem*>    _listMediaItem;
    QMutex               _mutex;
};

#endif // _PLAYLISTMODEL_H
