/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "Playlist/playlistpopulator.h"
#include "Playlist/playlistmodel.h"
#include "MediaItem/mediaitem.h"
#include "MediaItem/mediaplaylist.h"
#include "Web/networkaccess.h"
#include "utilities.h"


#include <QDataStream>
#include <QFile>
#include <QByteArray>
#include <QDirIterator>
#include <QMutexLocker>
#include <QDebug>
#include <QFileInfo>


namespace GLOBAL {
    NetworkAccess* http();
}


//! ------------ PlaylistPopulator class ---------------------------------------
PlaylistPopulator::PlaylistPopulator()
{
    _playlistModel  = PlaylistModel::instance();
    _files.clear();
    _isRunning = false;
}


//! ------------ PlaylistPopulator::run()---------------------------------------
void PlaylistPopulator::run()
{
  _isRunning = true;

  while (!_files.isEmpty()) 
  {
      //! ---- cas des dossiers ---- 
      if (QFileInfo(_files.first()).isDir()) {
        QString dirName = _files.takeFirst();
        const QStringList dirFilter  = QStringList() << "*.mp3" << "*.ogg" << "*.flac" << "*.wav";
        QDirIterator dirIterator(dirName, dirFilter ,QDir::Files | QDir::NoSymLinks, QDirIterator::Subdirectories);

        QStringList readfiles; 
        while(dirIterator.hasNext()) {
          readfiles << dirIterator.next(); 
        }
        
        readfiles.append(_files);
        _files = readfiles;
      }
     
      //! ---- cas des playlists remote ---- 
      if (_files.size()>0) {
        if( !MEDIA::isUrlLocal(_files.first()) && 
            (MEDIA::isPlaylistFile(_files.first()) ||
             MEDIA::isShoutCastUrl(_files.first())) ) {

                //! remote playlist --> download, make it local
                qDebug() << "PlaylistPopulator startDownload" << _files.first();
                QUrl url = QUrl( _files.takeFirst() );
                QByteArray bytes = GLOBAL::http()->syncGetBytes(url);
                
                if(bytes.isEmpty()) {
		  qDebug() << "PlaylistPopulator received empty bytes";  
		  //emit emptyRemotePlaylist(url.toString());
		}
		else {
		  QFileInfo fileInfo(url.path());
		  QString _tempFileName = fileInfo.fileName();
		  if (QFile::exists(UTIL::CONFIGDIR + "/download/" +  _tempFileName)) {
		    QFile::remove(UTIL::CONFIGDIR + "/download/" + _tempFileName);
		  }
		  QFile *_tempFile =  new QFile(UTIL::CONFIGDIR + "/download/" + _tempFileName);
		  if (_tempFile->open(QIODevice::WriteOnly)) {
		    QDataStream stream( _tempFile ); 
		    stream.writeRawData(bytes.constData(), bytes.size());
		    _tempFile->close();
		    _files.prepend(UTIL::CONFIGDIR + "/download/" + _tempFileName);
		  }
		}
        }
      }
      
      //! ---- cas des playlists locale ---- 
      if (_files.size()>0) {
      //! local playlist
        if( MEDIA::isUrlLocal(_files.first()) &&
            MEDIA::isPlaylistFile(_files.first())) {
              //! local playlist
              qDebug() << "PlaylistPopulator readPlaylistFile local " << _files.first();
              QList<MediaItem*> list = MEDIA::readPlaylistFile(_files.takeFirst());
              foreach (const MediaItem *mi, list) {
		if(mi->type == TYPE_TRACK) {
                    MediaItem *searchmi = MEDIA::FromDataBase("PlaylistPopulatorConnection", mi->data[TRACK::url].toString());
                    if(!searchmi) searchmi = MEDIA::FromLocalFile(mi->data[TRACK::url].toString());
                    
		    searchmi->data[TRACK::isPlayed]  = false;		    
		    _playlistModel->addMediaItem(searchmi);
		}
                else {
                  MediaItem* newMi = MEDIA::miCopy(mi);
                  newMi->data[STREAM::isPlayed]  = false;
                  _playlistModel->addMediaItem(newMi);
                }
             } // foreach MediaItem
          }
      }

      //! ---- cas des fichiers et Url ---- 
      if (_files.size() > 0) {
          if(  MEDIA::isUrlLocal(_files.first()) && 
               MEDIA::isAudioFile(_files.first()) && 
              !MEDIA::isPlaylistFile(_files.first())) 
          {
              //QString fileName = QUrl(_files.takeFirst()).toLocalFile();
              QString fileName = QFileInfo(_files.takeFirst()).canonicalFilePath();
	      
              MediaItem *mi = MEDIA::FromDataBase("PlaylistPopulatorConnection", fileName);
              if(!mi) mi = MEDIA::FromLocalFile(fileName);
		
              _playlistModel->addMediaItem(mi);
          }
          else if(!MEDIA::isUrlLocal(_files.first())) {
              //! remote file
              qDebug() << "remote uRL" << _files.first();
              QString url = _files.takeFirst();
              MediaItem* mi = new MediaItem();
              mi->type                    = TYPE_STREAM;
              mi->data[STREAM::id]        = -1;
              mi->data[STREAM::url]       = url;
              mi->data[STREAM::name]      = url;
              mi->data[STREAM::title]     = QString();
              mi->data[STREAM::artist]    = QString();
              mi->data[STREAM::album]     = QString();
              mi->data[STREAM::categorie] = QString();
              mi->data[STREAM::isFavorite]= false;
              mi->data[STREAM::isPlaying] = false;
              mi->data[STREAM::isBroken]  = false;
              mi->data[STREAM::isPlayed]  = false;
             
             _playlistModel->addMediaItem(mi);
          }
          else {
              //!WARNING on doit sortir les éléments non traité (si boucle while)
              qDebug() << "PlaylistPopulator --> unsupported media !" << _files.takeFirst();
          }
      }
  } //! END !_files.isEmpty()
  
  _isRunning = false;
  emit playlistPopulated();
}

   

//! -------- User methode ------------------------------------------------------
/* 
  addFile  : add one file to proceed
  addFiles : add files
  addUrls  : add Urls
*/
//! -------- PlaylistPopulator::addFile ----------------------------------------
void PlaylistPopulator::addFile(const QString &file)
{
    //qDebug() << "PlaylistPopulator append file :" << file;	
    _files.append(file);
}
//! -------- PlaylistPopulator::addFiles ---------------------------------------
void PlaylistPopulator::addFiles(const QStringList &files)
{
    //qDebug() << "PlaylistPopulator append files :" << files;	    
    _files.append(files);
}
//! -------- PlaylistPopulator::addUrls ----------------------------------------
void PlaylistPopulator::addUrls(QList<QUrl> listUrl)
{
    //qDebug() << "PlaylistPopulator append url :" << listUrl;	    
    foreach (const QUrl &url, listUrl) {
      if(!url.toString().contains(QRegExp("^http://")))
        _files.append(url.toLocalFile());
      else
        _files.append(url.toString());
    }
}
