/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "Playlist/playlistwidget.h" 
#include "Widget/statusbar.h" 
#include "utilities.h"

#include <QSettings>


namespace GLOBAL {
    QMap<QString, QAction*>* globalActions();
}


PlaylistWidget::PlaylistWidget(QWidget *parent,PlaylistView *playlistview)
{
    Q_UNUSED(parent)
  
    _playlistdelegate = playlistview->getPlaylistDelegate();

    //! Create Gui
    QVBoxLayout *vl = new QVBoxLayout(this);
    vl->setSpacing(0);
    vl->setContentsMargins(0, 0, 0, 0);
    vl->addWidget(playlistview);
  
    //! Playlist Mode
    QMap<QString, QAction*> *actions = GLOBAL::globalActions();

    connect(actions->value("playlistModeTitle"), SIGNAL(triggered()),this,  SLOT(modeTitleTriggered()));
    connect(actions->value("playlistModeAlbum"), SIGNAL(triggered()),this,  SLOT(modeAlbumTriggered()));
    connect(actions->value("playlistModeArtist"), SIGNAL(triggered()),this, SLOT(modeArtistTriggered()));
  
    //! Restore previous state
    restoreLastPlaylistMode();
}

    
void PlaylistWidget::setPlaylistModel(PlaylistModel *model) 
{
    _playlistmodel = model;
    connect(_playlistmodel, SIGNAL(layoutChanged()), this, SLOT(updatePlaylistSummary()));
    
    updatePlaylistSummary();
}


//! ---- Save & REstore Settings -----------------------------------------------
void PlaylistWidget::saveLastPlaylistMode()
{
    QSettings *_settings = new QSettings(UTIL::CONFIGFILE,QSettings::IniFormat,this);
    _settings->setValue("Playlist/mode",_playlistMode );
}

void PlaylistWidget::restoreLastPlaylistMode()
{
  QSettings *_settings = new QSettings(UTIL::CONFIGFILE,QSettings::IniFormat,this);
  
  if(_settings->contains("Playlist/mode"))
    _playlistMode = PLAYQUEUE::Q_MODE(_settings->value("Playlist/mode").toInt());
  else 
    _playlistMode = PLAYQUEUE::MODE_TITLE;
  
  UpdateViewMode(_playlistMode);
}


//! ---- Mode Switch -----------------------------------------------------------
void PlaylistWidget::UpdateViewMode(PLAYQUEUE::Q_MODE mode)  
{
   QMap<QString, QAction*> *actions = GLOBAL::globalActions();

  _playlistMode = mode;
  
  ( actions->value("playlistModeTitle"))->setChecked(_playlistMode == PLAYQUEUE::MODE_TITLE);
  ( actions->value("playlistModeAlbum"))->setChecked(_playlistMode == PLAYQUEUE::MODE_ALBUM);
  ( actions->value("playlistModeArtist"))->setChecked(_playlistMode == PLAYQUEUE::MODE_ARTIST);
  
  _playlistdelegate->_playlistMode = _playlistMode;   
}


void PlaylistWidget::modeTitleTriggered() 
{
      UpdateViewMode(PLAYQUEUE::MODE_TITLE);
}

void PlaylistWidget::modeAlbumTriggered()
{
      UpdateViewMode(PLAYQUEUE::MODE_ALBUM);
}

void PlaylistWidget::modeArtistTriggered()
{
      UpdateViewMode(PLAYQUEUE::MODE_ARTIST);
}

//! ---- List Summary ----------------------------------------------------------
void PlaylistWidget::updatePlaylistSummary()
{
    const int rowCount = _playlistmodel->rowCount(QModelIndex());
    QString duree = _playlistmodel->getListDuration(); 

    QString playlistInfo;
  
    if (rowCount < 1)
      playlistInfo = tr("Empty Playlist");
    else 
      playlistInfo = QString(tr("Playlist : <b>%1</b> tracks - %2")).arg(QString::number(rowCount), duree);

    StatusBar::instance()->updatePlaylistInfo(playlistInfo);
}
