/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <QtGui>
#include <QtNetwork>

#include "RadioModel/shoutcastmanager.h"
#include "Web/networkaccess.h"
#include "utilities.h"

namespace GLOBAL {
    NetworkAccess* http();
}

static const QStringList CONST_SHOUTCAST_GENRE = QStringList()
<< "Alternative"
<< "Blues"
<< "Classical"
<< "Country"
<< "Easy Listening"
<< "Electronic"
<< "Folk"
<< "Themes"
<< "Rap"
<< "Inspirational"
<< "International"
<< "Jazz"
<< "Latin"
<< "Metal"
<< "New Age"
<< "Decades"
<< "Pop"
<< "R&B/Urban"
<< "Reggae"
<< "Rock"
<< "Seasonal/Holiday"
<< "Soundtracks"
<< "Talk"
<< "Misc";
 
//! ------------------ class IceCastManager ------------------------------------  
ShoutCastManager::ShoutCastManager() 
{
    qDebug() << " ---- ShoutCastManager --> creation";
    _isUpdating  = false;
    _isError     = false;
    _downloadTry = 0;
}


QList<MediaItem*> ShoutCastManager::getShoutCastStream() const
{
    return readFile();
}


bool ShoutCastManager::isReady()
{
 //! pour savoir si il faut tlcharger ou non
 QFile file(UTIL::CONFIGDIR + "/radio/" + "shoutcast.xml");
 return (file.exists() && !_isUpdating);
}

bool ShoutCastManager::isError()
{
 return _isError;
}



//! ------------------ startDownload -------------------------------------------  
void ShoutCastManager::startDownload() 
{
    qDebug() << " ---- ShoutCastManager --> startDownload";
    if(_isUpdating) return;
    
    if(_downloadTry++ > 2) {
        _isError = true;
        return;
    }
    _isUpdating = true;
    sendGenreRequest();
}
   
//! ------------------  genre request  -----------------------------------------  
void ShoutCastManager::sendGenreRequest() 
{
  qDebug() << " ---- ShoutCastManager -> sendGenreRequest";
  
  //! Genre request to shoutcast directory
  QUrl url("http://api.shoutcast.com/legacy/genrelist?k=fa1669MuiRPorUBw");
  //QUrl url("http://yp.shoutcast.com/sbin/newxml.phtml");

  QObject *reply = GLOBAL::http()->get(url);
  connect(reply, SIGNAL(data(QByteArray)), this, SLOT(parseGenreResponse(QByteArray)));
  connect(reply, SIGNAL(error(QNetworkReply*)), this, SLOT(shoutCastStopError()));
}

void ShoutCastManager::parseGenreResponse(QByteArray bytes) 
{
    qDebug() << " ---- ShoutCastManager -> parseGenreResponse ";

    QXmlStreamReader xml(bytes);
    _shoutGenreList.clear();

    while(!xml.atEnd() && !xml.hasError()) { 
      xml.readNext();
      if (xml.isStartElement() && xml.name() == "genre") {
        QString genre = xml.attributes().value("name").toString();
        //_shoutGenreList.append(QString(xml.readElementText()));
        _shoutGenreList.append(genre); 
        qDebug() << " ---- ShoutCastManager -> read genre " << genre;
      }
    }
    
    //! clear de la liste des stations
    _stationList.clear();
    if(_shoutGenreList.isEmpty()) {
       qDebug() << " ---- ShoutCastManager -> take predefined genre ";
      _shoutGenreList = CONST_SHOUTCAST_GENRE;
    }
    foreach (const QString &genre, _shoutGenreList) {
      sendStationRequest(genre);
    }
    writeFile(_stationList);  
    shoutCastStop();
}
//! ------------------  station request ----------------------------------------  
void ShoutCastManager::sendStationRequest(QString genre) 
{
  qDebug() << " ---- ShoutCastManager -> sendStationRequest for genre" << genre;

  //QString request = QString("http://yp.shoutcast.com/sbin/newxml.phtml?genre=%1&limit=50").arg(genre);
  
  QString request = QString("http://api.shoutcast.com/legacy/genresearch?k=fa1669MuiRPorUBw&genre=%1&limit=50").arg(genre);
  
  QUrl url(request);
  
  QByteArray bytes = GLOBAL::http()->syncGetBytes(url);
  parseStationResponse(bytes, genre);
}

void ShoutCastManager::parseStationResponse(QByteArray bytes, QString &genre) 
{
    QXmlStreamReader xml(bytes);

    while(!xml.atEnd() && !xml.hasError()) { 
      xml.readNext();
      if (xml.isStartElement() && xml.name() == "station") {
        MediaItem * mi = new MediaItem();
        mi->type = TYPE_STREAM;
        
        QString name  = xml.attributes().value("name").toString();
        QString id    = xml.attributes().value("id").toString();
        QString url   = QString("http://yp.shoutcast.com/sbin/tunein-station.pls?id=%1").arg(id);

        mi->data[STREAM::name]      = name;
        mi->data[STREAM::url]       = url;
        mi->data[STREAM::categorie] = genre;
	    
        qDebug() << " ---- ShoutCastManager -> added stream" << url;
        _stationList.append(mi); 
      }
    }
}

//! ------------------  local shoutcast file read/write ------------------------  
void ShoutCastManager::writeFile(const QList<MediaItem*>& liste)  
{
  //! open file to write
  QFile file(UTIL::CONFIGDIR + "/radio/" + "shoutcast.xml");
  if(file.exists()) file.remove();
  if(!file.open(QIODevice::WriteOnly)) return;  
  
  QXmlStreamWriter xml(&file);
  xml.writeStartDocument();
  xml.writeStartElement("shoutcast");
  xml.writeStartElement("trackList");
  
  foreach (MediaItem* mi, liste) 
  {
    if (!mi) continue;
    if (mi->type != TYPE_STREAM) continue;
    
    xml.writeStartElement("track");
    xml.writeTextElement("location", mi->data[STREAM::url].toString());
    xml.writeTextElement("title", mi->data[STREAM::name].toString());
    xml.writeStartElement("extension");
    xml.writeAttribute("application", "yarock");
    xml.writeTextElement("category", mi->data[STREAM::categorie].toString());
    xml.writeEndElement(); //extension
    xml.writeEndElement(); //track
  }
  xml.writeEndElement(); //trackList
  xml.writeEndElement(); //shoutcast
  xml.writeEndDocument(); 
}

QList<MediaItem*> ShoutCastManager::readFile() const
{
    QList<MediaItem*> listStream;
      
    QFile file(UTIL::CONFIGDIR + "/radio/" + "shoutcast.xml");

    if(!file.exists()) return listStream;
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) return listStream;

    QXmlStreamReader xml(&file); 
    MediaItem *mi = 0;

    while(!xml.atEnd() && !xml.hasError()) { 
      xml.readNext();
      
      if (xml.isStartElement()) {
         if(xml.name() == "track") {
            //! new stream entry
            mi = new MediaItem();
            mi->type = TYPE_STREAM;
            
         } else if (xml.name() == "title" && mi!=0) {
           mi->data[STREAM::name] = QString(xml.readElementText());
         } else if (xml.name() == "location" && mi!=0) {
           mi->data[STREAM::url]  = QString(xml.readElementText());
         } else if (xml.name() == "category" && mi!=0) {
           mi->data[STREAM::categorie] = QString(xml.readElementText());              
         }
      }
      else if (xml.isEndElement() && xml.name() == "track") {
         //! if stream OK --> add it
         if (!mi->data[STREAM::url].toString().isEmpty() &&
             !mi->data[STREAM::name].toString().isEmpty() &&
             !MEDIA::isUrlLocal(mi->data[STREAM::url].toString())) 
          {
             listStream.append(mi);
          }
          mi = 0;
      }
   }
   
   if(!_showDuplicate) 
   {
      qSort(listStream.begin(), listStream.end(), MEDIA::compareStreamName);
      QString _current;
      foreach (MediaItem* mi, listStream) {
	QString _new = mi->data[STREAM::name].toString();
	if(_current == _new) {
	  listStream.removeOne ( mi );
	}
	else {
	  _current = _new;
	}
      }
    }
    
    qSort(listStream.begin(), listStream.end(), MEDIA::compareStreamCategorie);
    return listStream;
}


//! ------------------  shoutCastStop ------------------------------------------  
void ShoutCastManager::shoutCastStop()  
{
    qDebug() << " ---- ShoutCastManager -> shoutCastStop";
    _isUpdating = false;
    emit shoutCastUpdated();
}

void ShoutCastManager::shoutCastStopError()  
{
    qDebug() << " ---- ShoutCastManager -> shoutCastStopError";
    _isUpdating = false;
    emit shoutCastStoppedError();
}
