/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "RadioModel/streammodel.h"
#include "MediaItem/mediaitem.h"

StreamModel* StreamModel::INSTANCE = 0;
StreamModel* _StreamModel() {return StreamModel::instance();}
/*******************************************************************************
    StreamModel
*******************************************************************************/
StreamModel::StreamModel(QObject *parent) : QObject(parent) 
{
    qDebug() << " ---- StreamModel --> creation";
    INSTANCE          = this;
    modelUpdated      = false;
    _filterPattern    = "";
}

void StreamModel::clear()
{
  qDeleteAll(_listStream.constBegin(), _listStream.constEnd());
  _listStream.clear();
}

//! ------------------ StreamModel::updateListeStream --------------------------
void StreamModel::updateListeStream(const QList<MediaItem*> &liste)
{
    qDebug() << " ---- StreamModel --> updateListeStream";
    this->clear();
    if (!liste.empty()) {
      _listStream.append(liste);
    }
    modelUpdated = true;
}


//!------------StreamModel::getStreamItem ---------------------------------------
MediaItem* StreamModel::getStreamItem(int row) 
{
   if (row < 0 || row >= _listStream.size())
        return 0;

   return _listStream.at(row);
}

//! --------- playing item method ----------------------------------------------
void StreamModel::updatePlayingItem(const QString& urlName)
{
  qDebug() << " ---- StreamModel --> updatePlayingItem" << urlName;

  for ( int i = 0; i < _listStream.size(); i++ )
  {
     MediaItem* mi = _listStream.at(i);
     if (urlName ==  mi->data[STREAM::url].toString()) 
     {
       mi->data[STREAM::isPlaying]         = true;
       emit signalPlayingStatusChanged();
       break;
     }
  }
}

void StreamModel::stopPlayingItem()
{
  for ( int i = 0; i < _listStream.size(); i++ )
  {
     MediaItem* mi = _listStream.at(i);
     if (mi->data[STREAM::isPlaying].toBool()) {
       mi->data[STREAM::isPlaying] = false;
       emit signalPlayingStatusChanged();
       break;
    }
  }
}  
  
//! --------- Filtering method -------------------------------------------------
bool StreamModel::isStreamFiltered(const int row)
{
  MediaItem*mi = getStreamItem(row);
  if (mi->data[STREAM::name].toString().contains(_filterPattern, Qt::CaseInsensitive) ||
      mi->data[STREAM::categorie].toString().contains(_filterPattern, Qt::CaseInsensitive))
      return true;
  
  return false;
}


//! --------- broken item method ----------------------------------------------
void StreamModel::updateBrokenItem(const QString& urlName)
{
  qDebug() << " ---- StreamModel --> updateBrokenItem" << urlName;

  for ( int i = 0; i < _listStream.size(); i++ )
  {
     MediaItem* mi = _listStream.at(i);
     if (urlName ==  mi->data[STREAM::url].toString()) 
     {
       mi->data[STREAM::isBroken]         = true;
       emit signalPlayingStatusChanged();
       break;
     }
  }
}
