/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <QtGui>
#include <QtNetwork>

#include "RadioModel/xspfstreamfile.h"
#include "utilities.h"

//! --------------------------------------------------------------------
QList<MediaItem*> STREAM::readXspfStreamFile(const QString& filename)
{
    QList<MediaItem*> _listStream;
      
    qDebug() << " ---- CustomStreamReader -> readXspfFile  filename" << filename;
    QFile file(UTIL::CONFIGDIR + "/radio/" + filename);

    if(!file.exists()) return _listStream;
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) return _listStream;

    QXmlStreamReader xml(&file); 
    MediaItem * mi = 0;

    while(!xml.atEnd() && !xml.hasError()) { 
      xml.readNext();
      
      if (xml.isStartElement()) {
        if(xml.name() == "track") {
            //! new stream entry
            mi = new MediaItem();
            mi->type = TYPE_STREAM;
           
        } else if (xml.name() == "title" && mi!=0) {
            mi->data[STREAM::name] = QString(xml.readElementText());
        } else if (xml.name() == "location" && mi!=0) {
            mi->data[STREAM::url]  = QString(xml.readElementText());
        } else if (xml.name() == "category" && mi!=0) {
            mi->data[STREAM::categorie] = QString(xml.readElementText());
        }
      }
      else if (xml.isEndElement() && xml.name() == "track") {
              //! if stream OK --> add it
              if (!mi->data[STREAM::url].toString().isEmpty() && 
                  !mi->data[STREAM::name].toString().isEmpty()) {
                  _listStream.append(mi);
              }
              mi = 0;
          }
      }
   
    qSort(_listStream.begin(), _listStream.end(), MEDIA::compareStreamCategorie);
    return _listStream;
}

//! --------------------------------------------------------------------
QList<MediaItem*> STREAM::readXspfStreamFavorite()
{
    QList<MediaItem*> _listStream;
      
    QFile file(UTIL::CONFIGDIR + "/favorite.xspf");

    if(!file.exists()) return _listStream;
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) return _listStream;

    QXmlStreamReader xml(&file); 
    MediaItem * mi = 0;

    while(!xml.atEnd() && !xml.hasError()) { 
      xml.readNext();
      
      if (xml.isStartElement()) {
        if(xml.name() == "track") {
            //! new stream entry
            mi = new MediaItem();
            mi->type = TYPE_STREAM;
	    
        } else if (xml.name() == "title" && mi!=0) {
            mi->data[STREAM::name] = QString(xml.readElementText());
        } else if (xml.name() == "location" && mi!=0) {
            mi->data[STREAM::url]  = QString(xml.readElementText());
        } else if (xml.name() == "category" && mi!=0) {
            mi->data[STREAM::categorie] = QString(xml.readElementText());
        }
      }
      else if (xml.isEndElement() && xml.name() == "track") {
              //! if stream OK --> add it
              if (!mi->data[STREAM::url].toString().isEmpty() &&
                  !mi->data[STREAM::name].toString().isEmpty() &&
                  !MEDIA::isUrlLocal(mi->data[STREAM::url].toString()) ) {
                  _listStream.append(mi);
              }
              mi = 0;
          }
      }
   
    //! sort stream by Categorie
    qSort(_listStream.begin(), _listStream.end(), MEDIA::compareStreamCategorie);
    return _listStream;
}

//! --------------------------------------------------------------------
void STREAM::saveXspfStreamFavorite(QList<MediaItem*> _listStreamItem) 
{
  //! open file to write
  QFile file(UTIL::CONFIGDIR + "/favorite.xspf");  
  if(file.exists()) file.remove();
  if(!file.open(QIODevice::WriteOnly)) return;  
  
  QXmlStreamWriter xml(&file);
  xml.writeStartDocument();
  xml.writeStartElement("playlist");
  xml.writeAttribute("version", "1");
  xml.writeDefaultNamespace("http://xspf.org/ns/0/");
  xml.writeStartElement("trackList");
  
  foreach (const MediaItem* mi, _listStreamItem) 
  {
    if (!mi) continue;
    if (mi->type != TYPE_STREAM) continue;
    
    xml.writeStartElement("track");
    xml.writeTextElement("location", mi->data[STREAM::url].toString());
    xml.writeTextElement("title", mi->data[STREAM::name].toString());
    xml.writeStartElement("extension");
    xml.writeAttribute("application", "yarock");
    xml.writeTextElement("category", mi->data[STREAM::categorie].toString());
    xml.writeEndElement(); //extension
    xml.writeEndElement(); //track
  }
  xml.writeEndElement(); //trackList
  xml.writeEndElement(); //playlist
  xml.writeEndDocument(); 
}
