/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "RadioView/radioscene.h"
#include "RadioModel/streammodel.h"
#include "RadioModel/xspfstreamfile.h"
#include "MediaItem/mediaitem.h"
#include "MediaItem/mediaplaylist.h"
#include "Web/networkaccess.h"
#include "utilities.h"

namespace GLOBAL {
    NetworkAccess* http();
}

namespace GLOBAL {
    QMap<QString, QAction*>* globalActions();
}

/*******************************************************************************
    RadioScene
*******************************************************************************/
RadioScene::RadioScene(QWidget *parent)
{
  setParent(parent);
  _parentWidget = parent;
  setItemIndexMethod(QGraphicsScene::NoIndex);
    
  _infoSize        = 0;
 
  //! set model  
  _streamModel = StreamModel::instance();
  connect(_streamModel, SIGNAL(signalPlayingStatusChanged()), this, SLOT(update()));

  //! set player  
  _player = Player::instance();
  //connect(_player, SIGNAL(trackChanged(QString)), this, SLOT(updateStreamPlaying(QString))); 
  
  //! ice cast
  _iceCastManager = new IceCastManager();
  connect(_iceCastManager, SIGNAL(iceCastUpdated()), this, SIGNAL(repopulateRadioScene())); 
  
  //! shout cast
  _shoutCastManager = new ShoutCastManager();
  connect(_shoutCastManager, SIGNAL(shoutCastUpdated()), this, SIGNAL(repopulateRadioScene())); 

  
  _radioMode             = MAIN::ViewIceCast;
  _currentPlaylist       = "";
  _mouseGrabbedItem      = 0;
  
  //! item double click 
  QMap<QString, QAction*> *actions = GLOBAL::globalActions();  
  //actions->insert("streamDoubleClick", new QAction(this)); 
  QObject::connect(actions->value("itemDoubleClick"), SIGNAL(triggered()), this, SLOT(streamDoubleClick()), Qt::DirectConnection);
  
  //! remove icecas duplicate
  removeDuplicateAct = new QAction(QIcon(),tr("Remove duplicate radio"),this);
  removeDuplicateAct->setCheckable(true);
  removeDuplicateAct->setChecked(false);
  QObject::connect(removeDuplicateAct, SIGNAL(triggered()), this, SLOT(removeDuplicateStream()), Qt::DirectConnection);
}   

RadioScene::~RadioScene()
{
  clear(); 
}   

void RadioScene::setVirtualPlaylist(VirtualPlaylist  *playlist)
{
  _virtualPlaylist = playlist;
  _virtualPlaylist->StopPlaying();
}

//! --------------------- playing status method --------------------- 
void RadioScene::updateStreamPlaying(const QString& urlName)
{  
  qDebug() << "## ---- RadioScene -> updateStreamPlaying" << urlName;
  qDebug() << "## ---- RadioScene -> updateStreamPlaying _currentPlaylist" << _currentPlaylist;
  if(!urlName.isEmpty()) 
  {
    _streamModel->stopPlayingItem();
    if  (!MEDIA::isUrlLocal(urlName)) {
        if(_currentPlaylist.isEmpty())
          _streamModel->updatePlayingItem(urlName);
        else
          _streamModel->updatePlayingItem(_currentPlaylist);
    }
  }
  else
  {  
     _currentPlaylist = "";
     _streamModel->stopPlayingItem();
     //_virtualPlaylist->StopPlaying();
  }
}
//! --------------------- broken status method --------------------- 
void RadioScene::updateStreamBroken(const QString& urlName)
{
    qDebug() << "## ---- RadioScene -> updateStreamBroken" << urlName;
    _streamModel->updateBrokenItem(urlName);    
}

//! --------------------- playing method ---------------------------
void RadioScene::playStream() 
{
  if(!_mouseGrabbedItem) return;
  StreamGraphicItem *item = static_cast<StreamGraphicItem*>(_mouseGrabbedItem);
  
  const QString streamUrl = item->media->data[STREAM::url].toString();

  qDebug() << " ---- RadioScene -> playStream  :" << QFileInfo(streamUrl).fileName() ;

  if( !MEDIA::isPlaylistFile(streamUrl) && !MEDIA::isShoutCastUrl(streamUrl)) {
    qDebug() << " ---- RadioScene -> playStream  we can play it's an url"<< streamUrl;

    _currentPlaylist = "";
    _virtualPlaylist->AppendFileInPlaylist(streamUrl);
  }
  else 
  {
    qDebug() << " ---- RadioScene -> playStream  it's a playlist -> download to do" ;
    _currentPlaylist = streamUrl;

    //! start a synchronous download
    QByteArray bytes = GLOBAL::http()->syncGetBytes(streamUrl);
    if(bytes.isEmpty()) {
      qDebug() << " ---- RadioScene -> playStream  received empty bytes";
      updateStreamBroken(_currentPlaylist);
      return;
    }
    

    QFileInfo fileInfo(QUrl(streamUrl).path());
    QString _tempFileName = fileInfo.fileName();

    if (QFile::exists(UTIL::CONFIGDIR + "/download/" +  _tempFileName)) {
      QFile::remove(UTIL::CONFIGDIR + "/download/" + _tempFileName);
    }
    QFile *_tempFile =  new QFile(UTIL::CONFIGDIR + "/download/" + _tempFileName);
    if (_tempFile->open(QIODevice::WriteOnly)) {
      QDataStream stream( _tempFile ); 
      stream.writeRawData(bytes.constData(), bytes.size());
      _tempFile->close();
    }
    
    QList<MediaItem*> listMediaItem = MEDIA::readPlaylistFile(UTIL::CONFIGDIR + "/download/" + _tempFileName);
    QStringList list;
    foreach(MediaItem *mi, listMediaItem) {
      list << mi->data[STREAM::url].toString();
    }
    
    //!QStringList list = MEDIA::readPlaylistFile(UTIL::CONFIGDIR + "/download/" + _tempFileName);
    
    if(list.isEmpty()) {
      qDebug() << " ---- RadioScene -> playStream empty playlist downloaded";
      updateStreamBroken(_currentPlaylist);
      return;      
    }
    
    //! nominal case
    _virtualPlaylist->AppendFileInPlaylist(list.first());
  }
}    

//! --------------------- RadioScene Populator ---------------------------
void RadioScene::populateScene() 
{
  qDebug() << " ---- RadioScene -> PopulateScene ----";
  
  //! clear the scene FIRST
  clear();
  
  switch(_radioMode) {
    case (MAIN::ViewIceCast) : 
      if(_iceCastManager->isReady()) {
        qDebug() << " ---- RadioScene -> icecastready ----";
        _streamModel->updateListeStream(_iceCastManager->getIceCastStream());
        populateStreamScene(); 
      }
      else {
        qDebug() << " ---- RadioScene -> icecast startdownload ----";
        _iceCastManager->startDownload();
        populateWaitingScene("Start downloading icecast stream....please wait...");
      }    
      break;
    
    case (MAIN::ViewShoutCast) : 
      if(_shoutCastManager->isReady()) {
        qDebug() << " ---- RadioScene -> shoutcastready ----";
        _streamModel->updateListeStream(_shoutCastManager->getShoutCastStream());
        populateStreamScene(); 
      }
      else {
        qDebug() << " ---- RadioScene -> shoutcast startdownload ----";
        _shoutCastManager->startDownload();
        populateWaitingScene("Start downloading shoutcast stream....please wait...");
      }    
      break;
      
    case (MAIN::ViewCustomRadio) :
        _streamModel->updateListeStream(STREAM::readXspfStreamFile(currentXspfFilename));
        populateStreamScene();
    break;
      
    case (MAIN::ViewFavoriteRadio) :
        _streamModel->updateListeStream(STREAM::readXspfStreamFavorite());
        populateFavoriteStreamScene();
    break;

    default: break;
  } 
  
  if(_player->playerState() == Phonon::PlayingState)
    updateStreamPlaying(_player->currentMediaSource());
    
  //! we need to ajust SceneRect 
  int diff = _parentWidget->height() - itemsBoundingRect().height() + 40;
  int ajust  = ( diff > 0) ? diff : 0; 
  //setSceneRect ( itemsBoundingRect().adjusted(-10, 0, +10, +10 + ajust) );
  setSceneRect ( itemsBoundingRect().adjusted(-10, 0, +10, +10) );

}



void RadioScene::populateStreamScene() 
{
  qDebug() << " ----RadioScene -> populateStreamScene ----";
  
  int streamRow      = 0;
  int categoriesRow  = 0;
  _infoSize          = 0;
  
  //! categories list (for up/down navigation)
  _listCategorie.clear();
  
  QString currentCategoryName;
  //! stream model loop
  for ( int i = 0; i < _streamModel->itemCount(); i++ ) 
  {
    if(!_streamModel->isStreamFiltered(i)) continue;
    
    if(currentCategoryName != _streamModel->getStreamItem(i)->data[STREAM::categorie].toString()) {
      //! new category
      currentCategoryName = _streamModel->getStreamItem(i)->data[STREAM::categorie].toString();
     
      // hack pour une présentation correcte on rajoute une fausse ligne de stream avant une nouvelle categorie
      streamRow++; 
      CategorieGraphicItem *_cat = new CategorieGraphicItem();
      _cat->_catName = currentCategoryName;
      _cat->setPos( 10 ,categoriesRow*40 + streamRow*20 );
      _cat->_width   = _parentWidget->width();

      addItem(_cat);
      categoriesRow++;
      _listCategorie.append(_cat);
    }

    StreamGraphicItem *_stream = new StreamGraphicItem();
    _stream->media = _streamModel->getStreamItem(i);
    _stream->media->data[STREAM::isFavorite] = false;
    _stream->setPos(0, categoriesRow*40 + streamRow*20);
    _stream->setToolTip(_stream->media->data[STREAM::url].toString());
    addItem(_stream); 
 
    streamRow++;
    _infoSize++;
  }  
}

void RadioScene::populateFavoriteStreamScene() 
{
  qDebug() << " ----RadioScene -> populateStreamScene ----";
  
  int streamRow      = 0;
  int categoriesRow  = 0;
  _infoSize          = 0;
  
  //! categories list (for up/down navigation)
  _listCategorie.clear();
  
  QString currentCategoryName;
  //! stream model loop
  for ( int i = 0; i < _streamModel->itemCount(); i++ ) 
  {
    if(!_streamModel->isStreamFiltered(i)) continue;
    
    if(currentCategoryName != _streamModel->getStreamItem(i)->data[STREAM::categorie].toString()) {
      //! new category
      currentCategoryName = _streamModel->getStreamItem(i)->data[STREAM::categorie].toString();

      // hack pour une présentation correcte on rajoute une fausse ligne de stream avant une nouvelle categorie
      streamRow++; 
      
      CategorieGraphicItem *_cat = new CategorieGraphicItem();
      _cat->_catName = currentCategoryName;
      _cat->setPos( 10 ,categoriesRow*40 + streamRow*20 );
      _cat->_width   = _parentWidget->width();

      addItem(_cat);
      categoriesRow++;
      _listCategorie.append(_cat);
    }

    StreamGraphicItem *_stream = new StreamGraphicItem();
    _stream->media = _streamModel->getStreamItem(i);
    _stream->media->data[STREAM::isFavorite] = true;
    _stream->setPos(0, categoriesRow*40 + streamRow*20);
    _stream->setToolTip(_stream->media->data[STREAM::url].toString());
    addItem(_stream); 
   
    streamRow++;
    _infoSize++;
  }  
}

void RadioScene::populateWaitingScene(QString text) 
{
  qDebug() << " ----RadioScene -> populateWaitingScene ----";
   _listCategorie.clear();
  
  InfoGraphicItem *item = new InfoGraphicItem();
  item->_text = text;
  item->_width   = _parentWidget->width();
  item->setPos( 0 , _parentWidget->height()/2);
  addItem(item);
}

//! --------------------- Favorite Methods --------------------- 
void RadioScene::addStreamToFavorite() 
{
  qDebug() << " ---- RadioScene -> addStreamToFavorite";
  if(!_mouseGrabbedItem) return;
  StreamGraphicItem *item = static_cast<StreamGraphicItem*>(_mouseGrabbedItem);
  
  bool exist = false;

  MediaItem *streamToAdd = item->media;

  //! get current favorite
  QList <MediaItem *> currentList = STREAM::readXspfStreamFavorite();
  
  //! add stream if does not exist 
  foreach (MediaItem* mi, currentList) {
    if(mi->data[STREAM::url].toString() == streamToAdd->data[STREAM::url].toString()) {
      exist = true;
      break;
    }
  }
  
  if(!exist) {
    currentList.append(streamToAdd);
    qSort(currentList.begin(), currentList.end(), MEDIA::compareStreamCategorie);

    //! save favorite
    STREAM::saveXspfStreamFavorite(currentList);
  }
}  


void RadioScene::removeStreamToFavorite() 
{
  qDebug() << " ---- RadioScene -> removeStreamToFavorite";
  if(!_mouseGrabbedItem) return;
  StreamGraphicItem *item = static_cast<StreamGraphicItem*>(_mouseGrabbedItem);

  bool removed = false;

  MediaItem *streamToRemove = item->media;

  //! get current favorite
  QList <MediaItem *> currentList = STREAM::readXspfStreamFavorite();
  
  //! remove stream if exist 
  foreach (MediaItem* mi, currentList) {
    if(mi->data[STREAM::url].toString() == streamToRemove->data[STREAM::url].toString()) {
      currentList.removeAt(currentList.indexOf(mi));
      removed = true;
      break;
    }
  }
  
  if(removed) {
    //! save favorite
    qSort(currentList.begin(), currentList.end(), MEDIA::compareStreamCategorie);
    STREAM::saveXspfStreamFavorite(currentList);
    populateScene();
  }
}  


//! --------------------- contextMenuEvent ------------------------------------- 
void RadioScene::contextMenuEvent(QGraphicsSceneContextMenuEvent *event) 
{
  _mouseGrabbedItem = this->mouseGrabberItem();
  
  if(!_mouseGrabbedItem) {
      QAction *reloadStreamListAct = new QAction(QIcon(":/images/rebuild.png"),tr("Reload stream list"),this);
      QObject::connect(reloadStreamListAct, SIGNAL(triggered()), this, SLOT(reloadList()), Qt::DirectConnection);

      QMenu *streamMenu      = new QMenu();
      streamMenu->addAction(reloadStreamListAct);
      streamMenu->addSeparator();
      if(_radioMode == MAIN::ViewIceCast || _radioMode == MAIN::ViewShoutCast)
        streamMenu->addAction(removeDuplicateAct);
      streamMenu->exec(event->screenPos());
  }
  else if (_mouseGrabbedItem->type()== GraphicsItem::StreamType) {
    
      StreamGraphicItem *item = static_cast<StreamGraphicItem*>(_mouseGrabbedItem);

      //! Actions
      QAction *streamPlayAct = new QAction(QIcon(":/images/media-play.png"), tr("Play Stream"), this);
      QObject::connect(streamPlayAct, SIGNAL(triggered()), this, SLOT(playStream()), Qt::DirectConnection);
      QAction *favoriteStreamAct = new QAction(QIcon(":/images/favorites-48x48.png"), tr("Add stream to favorites"), this);

      if(item->media->data[STREAM::isFavorite].toBool()) {    
        favoriteStreamAct->setText (tr("Remove Stream from favorites"));
        QObject::connect(favoriteStreamAct, SIGNAL(triggered()), this, SLOT(removeStreamToFavorite()), Qt::DirectConnection);
      }
      else {
        QObject::connect(favoriteStreamAct, SIGNAL(triggered()), this, SLOT(addStreamToFavorite()), Qt::DirectConnection);
      }
      
      QAction *reloadStreamListAct = new QAction(QIcon(":/images/rebuild.png"),tr("Reload stream list"),this);
      QObject::connect(reloadStreamListAct, SIGNAL(triggered()), this, SLOT(reloadList()), Qt::DirectConnection);
        
      QMenu *streamMenu      = new QMenu();
      streamMenu->addAction(streamPlayAct);
      streamMenu->addSeparator();
      streamMenu->addAction(favoriteStreamAct);
      streamMenu->addSeparator();
      streamMenu->addAction(reloadStreamListAct);
      streamMenu->addSeparator();
      if(_radioMode == MAIN::ViewIceCast || _radioMode == MAIN::ViewShoutCast)
        streamMenu->addAction(removeDuplicateAct);
      streamMenu->exec(event->screenPos());
  } 
}


//! --------------------- itemDoubleClick ------------------------------------- 
void RadioScene::streamDoubleClick()
{
   QList<QGraphicsView *> view = this->views();
   if(view.isEmpty()) {
     //qDebug() << " ---- RadioScene:: View Not used";    
     return;
   }
  
   //qDebug() << " ---- RadioScene::itemDoubleClick";
   QMap<QString, QAction*> *actions = GLOBAL::globalActions();
   
   QGraphicsItem *gItem = qvariant_cast<QGraphicsItem *>( (actions->value("itemDoubleClick"))->data() );
   _mouseGrabbedItem = gItem;
   if(!_mouseGrabbedItem) return;
   
   switch(gItem->type()) {
     case GraphicsItem::StreamType: playStream();break;
     default:break;    
   }
}

//! --------------------- reloadList ------------------------------------------
void RadioScene::reloadList()
{
    if(_radioMode == MAIN::ViewIceCast) {
       QFile file(UTIL::CONFIGDIR + "/radio/" + "icecast.xml");
       if(file.exists() && _iceCastManager->isReady()) file.remove();
    }
    else if(_radioMode == MAIN::ViewShoutCast) {
       QFile file(UTIL::CONFIGDIR + "/radio/" + "shoutcast.xml");
       if(file.exists() && _shoutCastManager->isReady()) file.remove();
    }

    populateScene();
}



//! --------------------- findBrokenStream ------------------------------------
void RadioScene::findBrokenStream()
{
  for ( int i = 0; i < _streamModel->itemCount(); i++ ) {
    MediaItem *mi = _streamModel->getStreamItem(i);
    QString url = mi->data[STREAM::url].toString();
    bool isUrlOK = GLOBAL::http()->syncTest( QUrl(url) );
    mi->data[STREAM::isBroken] = !isUrlOK;
  }
}

//! --------------------- removeDuplicateStream --------------------------------
void RadioScene::removeDuplicateStream()
{
    _iceCastManager->setDuplicate(!removeDuplicateAct->isChecked());
    _shoutCastManager->setDuplicate(!removeDuplicateAct->isChecked());
     
    populateScene();
}
