/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "miniwidget.h"
#include "Player/player.h"
#include "ScreenMode/nowplaying.h" 

#include "Widget/spacer.h" 



namespace GLOBAL {
    QMap<QString, QAction*>* globalActions();
}


MinimalWidget::MinimalWidget(QWidget *parent) : QDialog(parent)
{
    _player         = Player::instance();
    QMap<QString, QAction*> *actions  = GLOBAL::globalActions();
    
    //! create toolbar
    QToolBar *_toolbar = new QToolBar(this);
    _toolbar->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Minimum );
    _toolbar->setIconSize( QSize( 28, 28 ) );
    _toolbar->setFloatable(false);
    _toolbar->setMovable(false);
    _toolbar->setContentsMargins( 0, 0, 0, 0);

    QPalette palette;
    palette.setColor(QPalette::Background, Qt::white);
    _toolbar->setPalette(palette);
    _toolbar->setFocusPolicy( Qt::NoFocus );
    _toolbar->setAutoFillBackground(true);  
    
    QToolButton* _prevButton = new QToolButton(this);
    _prevButton->setDefaultAction(actions->value("prev"));
    _prevButton->setIconSize( QSize( 28, 28 ) ); 
      
    QToolButton* _nextButton = new QToolButton(this);
    _nextButton->setDefaultAction(actions->value("next"));  
    _nextButton->setIconSize( QSize( 28, 28 ) );
  
    QToolButton* _playButton = new QToolButton(this);
    _playButton->setDefaultAction(actions->value("play"));    
    _playButton->setIconSize( QSize( 28, 28 ) );
    
    QToolButton* _stopButton = new QToolButton(this);
    _stopButton->setDefaultAction(actions->value("stop"));  
    _stopButton->setIconSize( QSize( 28, 28 ) );
    
    QToolButton* _normalModeButton = new QToolButton(this);
    _normalModeButton->setDefaultAction(actions->value("screenNormal"));
    _normalModeButton->setToolButtonStyle(Qt::ToolButtonIconOnly);
    _normalModeButton->setAutoRaise(true);
    _normalModeButton->setIconSize( QSize( 18, 18 ) );
  
    QToolBar *tb = new QToolBar(this);
    tb->setIconSize( QSize( 18, 18 ) ); 
    tb->addWidget( _normalModeButton);
  
    _toolbar->addWidget( new FixedSpacer(this, QSize(15,0)) );
    _toolbar->addWidget( _prevButton);
    _toolbar->addWidget( _playButton);
    _toolbar->addWidget( _stopButton);
    _toolbar->addWidget( _nextButton);
    _toolbar->addWidget( new FixedSpacer(this, QSize(5,0)) );
    _toolbar->addWidget( tb );
        
    //! create Gui
    this->setObjectName(QString::fromUtf8("MinimalWidget"));
    this->setWindowTitle("YaRock");
    this->setContentsMargins(0, 0, 0, 0);

    this->setAttribute(Qt::WA_TranslucentBackground);
    this->setStyleSheet("background:transparent;");

    this->setFixedSize(QSize(200,200));
    this->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
    
    _nowPlaying = new NowPlaying(this);

    QVBoxLayout* vl0 = new QVBoxLayout(this);
    vl0->setSpacing(0);
    vl0->setContentsMargins(0, 0, 0, 0);
    vl0->addWidget(_nowPlaying);
    vl0->addWidget(_toolbar);
}


//! --------- paintEvent -------------------------------------------------------
void MinimalWidget::paintEvent(QPaintEvent *)
{
    //! draw background
    QPainter painter(this);
    QPen pen;
    QColor backColor = QColor(23, 23, 34);
    backColor.setAlpha(80); 
  
    QBrush backBrush(backColor);
    painter.setBrush(backBrush);

    painter.fillRect(rect(), backColor);
}
