/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "ScreenMode/nowplaying.h" 
#include "Player/inforesolver.h"
#include "MediaItem/mediaitem.h"

#include <QPainter>

//! -------- NowPlaying class --------------------------------------------------
NowPlaying::NowPlaying(QWidget *parent) : QFrame(parent)
{
    this->setObjectName(QString::fromUtf8("nowplaying"));
  
    QPalette palette;
    palette.setColor(QPalette::Background, Qt::white);
    this->setPalette(palette);
  
    this->setFrameShape(QFrame::NoFrame);
  
    this->setFixedHeight (180);
    this->setFocusPolicy( Qt::NoFocus );
    this->setAutoFillBackground(false);
    this->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);

    
    //! set Info resolver
    _infoResolver = InfoResolver::instance();
    connect(_infoResolver, SIGNAL(streamInfoChanged()), this, SLOT(update()));  
    connect(_infoResolver, SIGNAL(playingInfoChanged()), this, SLOT(update()));  

    //! first init
    update();
}

//! -------- NowPlaying::handleStateChanged ---------------------------------
void NowPlaying::handleStateChanged() 
{
    if(_infoResolver->state() == PLAYING) {
     update();       
    }
}

//! -------- NowPlaying::paintEvent --------------------------------------------
void NowPlaying::paintEvent(QPaintEvent *)
{
  QPainter painter(this);
  QRectF rect(10, 0, 180, 20);
  
  QPen pen;
  pen.setColor(QColor("white"));
  painter.setPen(pen);
  
  //! Get data 
  MediaItem* mi  =  _infoResolver->mediaItem();
  QPixmap    pix =  _infoResolver->mediaPix();
      
  //! -- Playing State
  if(_infoResolver->state() == PLAYING) {
    if(mi->type == TYPE_TRACK) {
        //! paint artist
        painter.setFont(QFont("Arial", 10, QFont::Bold));
        QFontMetrics fm( QFont("Arial", 10, QFont::Bold));   
        QString artistTruncated = fm.elidedText ( mi->data[TRACK::artist].toString(), Qt::ElideRight, 180 );
        painter.drawText(rect, Qt::AlignVCenter | Qt::AlignLeft,artistTruncated );

        //! paint album    
        painter.setFont(QFont("Arial", 10, QFont::Normal));
        QFontMetrics fm2( QFont("Arial", 10, QFont::Normal));   
        QString albumTruncated = fm2.elidedText ( mi->data[TRACK::album].toString(), Qt::ElideRight, 180 );
        painter.drawText(rect.adjusted(0,15,0,15), Qt::AlignVCenter | Qt::AlignLeft,albumTruncated );

        //! paint title    
        QString titleTruncated = fm2.elidedText ( mi->data[TRACK::title].toString(), Qt::ElideRight, 180 );
        painter.drawText(rect.adjusted(0,145,0,145), Qt::AlignVCenter | Qt::AlignLeft,titleTruncated );

        //! paint pixmap (coverart)
        painter.drawPixmap(45,35,110,110, pix);
    }
    else if(mi->type == TYPE_STREAM) {
        if(mi->data[STREAM::album].toString().isEmpty()) { //! paint short stream name
            painter.setFont(QFont("Arial", 10, QFont::Bold));
            QFontMetrics fm( QFont("Arial", 10, QFont::Bold));   
            QString streamTruncated = fm.elidedText ( mi->data[STREAM::name].toString(), Qt::ElideRight, 180 );
            painter.drawText(rect, Qt::AlignVCenter | Qt::AlignLeft,streamTruncated );	   
        }
        else { //! paint album
            painter.setFont(QFont("Arial", 10, QFont::Bold));
            QFontMetrics fm( QFont("Arial", 10, QFont::Bold));   
            QString albumTruncated = fm.elidedText ( mi->data[STREAM::album].toString(), Qt::ElideRight, 180 );
            painter.drawText(rect, Qt::AlignVCenter | Qt::AlignLeft,albumTruncated );
       }
	  
       //! paint artist - title
       painter.setFont(QFont("Arial", 10, QFont::Normal));
       QFontMetrics fm2( QFont("Arial", 10, QFont::Normal));   
       QString titleTruncated = fm2.elidedText ( mi->data[STREAM::title].toString(), Qt::ElideRight, 180 );
       painter.drawText(rect.adjusted(0,15,0,15), Qt::AlignVCenter | Qt::AlignLeft,titleTruncated );

       //! paint pixmap (coverart)
       painter.drawPixmap(45,40,110,110, pix);
    }
  }
  //! -- Stopped State
  else 
  {
    painter.setFont(QFont("Arial", 12, QFont::Bold));
    QFontMetrics fm( QFont("Arial", 12, QFont::Bold));   
    QString text = fm.elidedText ( "Yarock Player", Qt::ElideRight, 180);
    painter.drawText(rect.adjusted(0,30,0,50), Qt::AlignHCenter,text );
    
    painter.drawPixmap(65,50,64,64, QPixmap(":/icon/64x64/yarock.png"));
   }
   painter.end();
}
