/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


//! local
#include "Scrobbler/scrobblermanager.h"
#include "Player/inforesolver.h"
#include "MediaItem/mediaitem.h"
#include "utilities.h"  

//! Qt
#include <QSettings>
#include <QString>
#include <QMessageBox>
#include <QDebug>
#include <QAbstractButton>
#include <QAction>


#include <QDebug>


namespace GLOBAL {
    QMap<QString, QAction*>* globalActions();
}



ScrobblerManager::ScrobblerManager(QObject *parent) 
{
    qDebug() << "--- ScrobblerManager :creation";

    _scrobbler = 0;
    
    QSettings settings(UTIL::CONFIGFILE,QSettings::IniFormat,this);
    settings.beginGroup("Scrobbler");
    _enable                      = settings.value("uselastfm", false).toBool();
    settings.endGroup();

    if(_enable) {
      _scrobbler     = new LastFm::Scrobbler();
      _player        = Player::instance();
      _infoResolver  = InfoResolver::instance();
      
      connect(_scrobbler, SIGNAL(needRegister()),this, SLOT(registerDialog()));
      connect(_player, SIGNAL(stateChanged(Phonon::State,Phonon::State)), this, SLOT(playerStateChanged(Phonon::State,Phonon::State)));
      
      connect(_infoResolver, SIGNAL(signal_lastFmLove(QString,QString)), this, SLOT(loveTrackRequest(QString,QString)));    
      connect(_infoResolver, SIGNAL(playingInfoChanged()), this, SLOT(trackChanged())); 
    }

    GLOBAL::globalActions()->value("lastfmlove")->setEnabled(_enable);
    GLOBAL::globalActions()->value("lastfmloveNowPlaying")->setEnabled(_enable);    
}

ScrobblerManager::~ScrobblerManager()
{
    qDebug() << "--- ScrobblerManager :delete";
    if(_scrobbler)
      delete _scrobbler;
}

void ScrobblerManager::settingsChanged()
{
    QSettings settings(UTIL::CONFIGFILE,QSettings::IniFormat,this);
    settings.beginGroup("Scrobbler");
    _enable                      = settings.value("uselastfm", false).toBool();
    settings.endGroup();
    
    if(_enable && _scrobbler == 0 ) {
      qDebug() << "--- ScrobblerManager :start scrobbling";
      //! start scrobbling
      _scrobbler     = new LastFm::Scrobbler();
      _player        = Player::instance();
      _infoResolver  = InfoResolver::instance();
      
      connect(_scrobbler, SIGNAL(needRegister()),this, SLOT(registerDialog()));
      connect(_player, SIGNAL(stateChanged(Phonon::State,Phonon::State)), this, SLOT(playerStateChanged(Phonon::State,Phonon::State)));

      connect(_infoResolver, SIGNAL(signal_lastFmLove(QString,QString)), this, SLOT(loveTrackRequest(QString,QString)));    
      connect(_infoResolver, SIGNAL(playingInfoChanged()), this, SLOT(trackChanged())); 
    }
    else if(!_enable && _scrobbler != 0 ) {
      //! stop scrobbling
      qDebug() << "--- ScrobblerManager :stop scrobbling";
      disconnect(_scrobbler, 0,this, 0);
      disconnect(_player, 0,this, 0);
      disconnect(_infoResolver, 0,this, 0);
      delete _scrobbler;
      _scrobbler = 0;
    }
    
    //! change status for send love action
    GLOBAL::globalActions()->value("lastfmlove")->setEnabled(_enable);
    GLOBAL::globalActions()->value("lastfmloveNowPlaying")->setEnabled(_enable);    
}




void ScrobblerManager::registerDialog() 
{
    qDebug() << "--- ScrobblerManager :registerApplication";

    QString str = tr("<h2>Warning</h2>" 
        "<p>Before using LastFm services, you need to register Yarock with LasFm Web site !!</p>");
     
    QMessageBox LastFmDialog;
    LastFmDialog.setIcon(QMessageBox::Warning);
    LastFmDialog.setWindowTitle(tr("LastFm register"));
    LastFmDialog.setText(str.toAscii());
    LastFmDialog.setStandardButtons(QMessageBox::Ok | QMessageBox::Cancel);
    
    
    QAbstractButton * sb = LastFmDialog.button(QMessageBox::Ok);
    connect(sb,SIGNAL(clicked()),_scrobbler,SLOT(getToken()));
    
    LastFmDialog.resize(445, 120);
    LastFmDialog.exec();
}



void ScrobblerManager::trackChanged()
{
    MediaItem *mi = _infoResolver->mediaItem();

    //! Submit the old track
    if (_scrobbler->currentTrack())
        _scrobbler->currentTrack()->scrobble();

    if(mi->type != TYPE_TRACK) return;

    uint stamp = QDateTime::currentDateTime().toTime_t();

    LastFm::Track *track = new LastFm::Track(_scrobbler);
    
    track->setArtist( mi->data[TRACK::artist].toString() );
    track->setTrackTitle( mi->data[TRACK::title].toString() );
    track->setAlbum( mi->data[TRACK::album].toString() );
    track->setTrackLength(mi->data[TRACK::duration].toInt());
    track->setGenre( mi->data[TRACK::genre].toString() );
    track->setTrackNumber( int( mi->data[TRACK::num].toInt() ) );
    track->setPlaybackStart(stamp);
    
    //! set current track
    _scrobbler->setCurrentTrack(track);

    //! Set "Now playing"
    track->nowPlaying();
}

void ScrobblerManager::playerStateChanged(Phonon::State newState, Phonon::State oldState)
{
    if ((oldState == Phonon::PausedState) && (newState == Phonon::PlayingState))
        if (_scrobbler->currentTrack())
            _scrobbler->currentTrack()->pause(false);

    if ((oldState == Phonon::PlayingState) && (newState == Phonon::PausedState))
        if (_scrobbler->currentTrack())
            _scrobbler->currentTrack()->pause(true);

}



void ScrobblerManager::loveTrackRequest(QString artist, QString title)
{
    qDebug() << "--- ScrobblerManager :loveTrackRequest track :" << title;
    LastFm::Track *track = new LastFm::Track(_scrobbler);
    track->setTrackTitle(title);
    track->setArtist(artist);
    
    track->love();
    connect(track, SIGNAL(loved()),track, SLOT(deleteLater()));
}