/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "SmartPlaylist/smartplaylist.h"
#include "MediaItem/mediaitem.h"
#include "CollectionModel/collectionmodel.h"
#include "CollectionModel/histomodel.h"
#include "utilities.h"
#include "Database/database.h"


#include <QtSql/QSqlDatabase>
#include <QtSql/QSqlQuery>
#include <QtSql/QSqlDriver>
#include <QtSql/QSqlField>
#include <QtSql/QSqlError>
#include <QtSql/QSqlResult>

#include <QDebug>
#include <QString>


namespace SMART_PLAYLIST
{
  QStringList mostPlayedUrls();
  QList<MediaItem*> mostPlayedMediaItem();
}


//! private SMART_PLAYLIST STUFF
const QStringList _playlist = QStringList() 
            << "50 Random tracks" 
            << "10 Random albums"
            << "50 last played" 
            << "50 most played"; 
    
const QStringList _icon = QStringList() 
            << ":/images/media-smartplaylist1-110x110.png" 
            << ":/images/media-smartplaylist1-110x110.png"
            << ":/images/media-smartplaylist2-110x110.png"
            << ":/images/media-smartplaylist3-110x110.png";
    

void SMART_PLAYLIST::initDb(QSqlDatabase *sqlDB) 
{
    QSqlField data("col",QVariant::String);

    int i = 0;
    foreach (const QString& name, _playlist) {
      qDebug() << "--- SMART_PLAYLIST::initDb -> insert smart playlist :" << name;
      
      QSqlQuery playlistsQuery("SELECT id,filename,name,icon,rules,type,mtime,favorite FROM playlists WHERE name="+name+"",*sqlDB);
      
      qDebug() << playlistsQuery.next();
      
      if(!playlistsQuery.isValid()) {
        QSqlQuery query(*sqlDB);
        query.prepare("INSERT INTO `playlists`(`filename`,`name`, `icon`, `rules`, `type`,`favorite`, `mtime`)" \
                      "VALUES(?," \
                      "       ?," \
                      "       ?," \
                      "       ?," \
                      "       ?," \
                      "       ?," \
                      "       ?);");
        query.addBindValue("");
        query.addBindValue(name);
        query.addBindValue(_icon.at(i++));
        query.addBindValue("");
        query.addBindValue((int) T_SMART_AUTO);
        query.addBindValue(0);
        query.addBindValue("");
        qDebug() << "query exec " << query.exec();
      }
    }
}



QStringList SMART_PLAYLIST::tracks(T_PLAYLIST type, QString name, QString rules)
{
    QStringList list;
    qDebug() << "--- SMART_PLAYLIST::tracks -> requested playlist name :" << name;
  
    //! user smart playlist
    if(type == T_SMART_USER) {
    }
    //! auto smart playlist 
    else 
    {
       if(name == "50 Random tracks") {
         int max = CollectionModel::instance()->trackItemList.size();
         int limit = max < 50 ? max : 50;
      
         for(int i = 0; i< limit;i++) {
            int random = UTIL::randomInt(0, max -1);
            list << CollectionModel::instance()->trackItemList.at(random)->data[TRACK::url].toString();
         }
       }
       else if (name == "10 Random albums") {
          int max = CollectionModel::instance()->albumItemList.size();
          int limit = max < 10 ? max : 10;    
  
          for(int i = 0; i< limit;i++) {
            int random = UTIL::randomInt(0, max -1);
    
            MediaItem *album = CollectionModel::instance()->albumItemList.at(random);
            for(int j = 0; j < album->childCount();j++) {
              list << album->child(j)->data[TRACK::url].toString();
            }
          }
       }
       else if (name == "50 last played") {         
         if(!HistoModel::instance()->modelUpdated)
           HistoModel::instance()->updateModel();

          int max = HistoModel::instance()->itemCount();
          int limit = max < 10 ? max : 10;    

         for(int i = 0; i< limit;i++) {
            list << HistoModel::instance()->getItem(i)->data[TRACK::url].toString();
         }
       }
       else if (name == "50 most played") {
         list << SMART_PLAYLIST::mostPlayedUrls();
       }
    }
  
  return list;
}




QList<MediaItem*> SMART_PLAYLIST::mediaItem(T_PLAYLIST type, QString name, QString rules)
{
    QList<MediaItem*> list;
  
    //! user smart playlist
    if(type == T_SMART_USER) {
    }
    //! auto smart playlist 
    else 
    {
       if(name == "50 Random tracks") {
          int max = CollectionModel::instance()->trackItemList.size();
          int limit = max < 50 ? max : 50;
      
          for(int i = 0; i< limit;i++) {
            int random = UTIL::randomInt(0, max -1);
            list << CollectionModel::instance()->trackItemList.at(random);
          }
       }
       else if (name == "10 Random albums") {
          int max = CollectionModel::instance()->albumItemList.size();
          int limit = max < 10 ? max : 10;    
  
          for(int i = 0; i< limit;i++) {
            int random = UTIL::randomInt(0, max -1);
    
            MediaItem *album = CollectionModel::instance()->albumItemList.at(random);
            for(int j = 0; j < album->childCount();j++) {
              list << album->child(j);
            }
          }
       }
       else if (name == "50 last played") {
         if(!HistoModel::instance()->modelUpdated)
           HistoModel::instance()->updateModel();
 
          int max = HistoModel::instance()->itemCount();
          int limit = max < 10 ? max : 10;    

         for(int i = 0; i< limit;i++) {
            list << HistoModel::instance()->getItem(i);
         }
       }
       else if (name == "50 most played") {
         list << SMART_PLAYLIST::mostPlayedMediaItem();
       }       
    }
  
  return list;
}


QStringList SMART_PLAYLIST::mostPlayedUrls()
{
    QStringList list;

    Database db;
    if (!db.connect("SmartPlaylist")) {
        qDebug() << "--- SMART_PLAYLIST connect to database failed";
        return list;
    }

    QSqlQuery tracksQuery("SELECT `filename`,`playcount` FROM `tracks` ORDER BY `playcount` DESC;",
                          *db.sqlDb());
    
    while (tracksQuery.next() && list.size() < 50) 
    {
      int playcount = tracksQuery.value(1).toInt();
      if(playcount == 0) break;
      
      list << tracksQuery.value(0).toString();
    } 
    
    return list;
}



QList<MediaItem*> SMART_PLAYLIST::mostPlayedMediaItem()
{
    QList<MediaItem*> list;
 
    QStringList urlList = SMART_PLAYLIST::mostPlayedUrls();
    foreach (const QString & url, urlList)
      list << MEDIA::FromDataBase("SmartPlaylist",url);

    return list;
}


