/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

//! local
#include "settingsdialog.h"

//! Qt
#include <QDebug>
#include <QListWidget>
#include <QStringList>

/*******************************************************************************
    SettingsDialog
*******************************************************************************/
SettingsDialog::SettingsDialog(QWidget *parent): QDialog(parent)
{
    //! create GUI
    createGui();

    //! restore all settings 
    _settingGlobalPage->restoreSettings();
    _settingCollectionPage->restoreSettings();
    _settingRadioPage->restoreSettings();
    _settingShortcutPage->restoreSettings();
    _settingScrobblerPage->restoreSettings();
  
    //! Connection 
    QObject::connect(pageSelector, SIGNAL(itemClicked ( QListWidgetItem *)),this, SLOT(changePage(QListWidgetItem*)));
    QObject::connect(buttonBox, SIGNAL(accepted()), this, SLOT(on_buttonBox_accepted()));
    QObject::connect(buttonBox, SIGNAL(rejected()), this, SLOT(on_buttonBox_rejected()));
}


void SettingsDialog::createGui() 
{
    //! QDialog
    this->resize(550, 420);
    this->setModal(false); 
    this->setWindowTitle(tr("YaRock settings"));

    //! ListWidget 
    pageSelector = new QListWidget(); 
    pageSelector->setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Expanding);
    pageSelector->setMinimumWidth(112);
    pageSelector->setMaximumWidth(112);
    pageSelector->setDropIndicatorShown(false);
    pageSelector->setDragDropMode(QAbstractItemView::NoDragDrop);
    pageSelector->setIconSize(QSize(26, 26));
    pageSelector->setMovement(QListView::Static);
    pageSelector->setViewMode(QListView::IconMode);
    pageSelector->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    
    //! ListWidget Item
    QListWidgetItem *globalMenuItem     = new QListWidgetItem("Global setting");
    QListWidgetItem *collectionMenuItem = new QListWidgetItem("Collection setting");
    QListWidgetItem *radioMenuItem      = new QListWidgetItem("Radio setting");
    QListWidgetItem *shortcutMenuItem   = new QListWidgetItem("Shurtcut setting");
    QListWidgetItem *scrobblerMenuItem  = new QListWidgetItem("Scrobbler settings");

    globalMenuItem->setIcon(QIcon(":/icon/64x64/yarock.png"));
    globalMenuItem->setText(tr("Yarock"));
    globalMenuItem->setTextAlignment(Qt::AlignHCenter);
    globalMenuItem->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
    globalMenuItem->setSizeHint(QSize(105,50));  
  
    collectionMenuItem->setIcon(QIcon(":/images/folder-music-48x48.png"));
    collectionMenuItem->setText(tr("Collection"));
    collectionMenuItem->setTextAlignment(Qt::AlignHCenter);
    collectionMenuItem->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
    collectionMenuItem->setSizeHint(QSize(105,50));

    radioMenuItem->setIcon(QIcon(":/images/media-url-110x110.png"));
    radioMenuItem->setText(tr("Radio"));
    radioMenuItem->setTextAlignment(Qt::AlignHCenter);
    radioMenuItem->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
    radioMenuItem->setSizeHint(QSize(105,50));
  
    shortcutMenuItem->setIcon(QIcon(":/images/input-keyboard-48x48.png"));
    shortcutMenuItem->setText(tr("Shortcut"));
    shortcutMenuItem->setTextAlignment(Qt::AlignHCenter);
    shortcutMenuItem->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
    shortcutMenuItem->setSizeHint(QSize(105,50));  
    
    scrobblerMenuItem->setIcon(QIcon(":/images/lastfm.png"));
    scrobblerMenuItem->setText(tr("Scrobbler"));
    scrobblerMenuItem->setTextAlignment(Qt::AlignHCenter);
    scrobblerMenuItem->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
    scrobblerMenuItem->setSizeHint(QSize(105,50));  
    
    pageSelector->addItem(globalMenuItem);
    pageSelector->addItem(collectionMenuItem);
    pageSelector->addItem(radioMenuItem);
    pageSelector->addItem(shortcutMenuItem);
    pageSelector->addItem(scrobblerMenuItem);

    //! StackedWidget
    _settingGlobalPage      = new SettingGlobalPage;
    _settingCollectionPage  = new SettingCollectionPage;
    _settingRadioPage       = new SettingRadioPage;
    _settingShortcutPage    = new SettingShortcutPage;
    _settingScrobblerPage   = new SettingScrobblerPage;
    
    _stackedPages = new QStackedWidget();
    _stackedPages->addWidget(_settingGlobalPage);
    _stackedPages->addWidget(_settingCollectionPage);
    _stackedPages->addWidget(_settingRadioPage);
    _stackedPages->addWidget(_settingShortcutPage);
    _stackedPages->addWidget(_settingScrobblerPage);
     
    //! QDialogButtonBox
    buttonBox = new QDialogButtonBox(this);
    buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
    buttonBox->setOrientation(Qt::Horizontal);
    buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);
        
    //! Layout
    QVBoxLayout* vl0 = new QVBoxLayout(this);
     
    QHBoxLayout* hl0 = new QHBoxLayout();
    hl0->addWidget(pageSelector);
    hl0->addWidget(_stackedPages);    
  
    vl0->addLayout(hl0);
    vl0->addWidget(buttonBox);
}

//! --------- setStartPage -----------------------------------------------------
void SettingsDialog::setStartPage(SettingsPage::Page page)
{
    pageSelector->setCurrentRow(int(page));
    _stackedPages->setCurrentIndex(pageSelector->currentRow());
}


//! --------- changePage -------------------------------------------------------
void SettingsDialog::changePage(QListWidgetItem *itemClicked)
{
    _stackedPages->setCurrentIndex(pageSelector->row(itemClicked));
}

//! --------- on_buttonBox_accepted --------------------------------------------
void SettingsDialog::on_buttonBox_accepted()
{
    //! save setting
    _settingGlobalPage->saveSettings();
    _settingCollectionPage->saveSettings();
    _settingRadioPage->saveSettings();  
    _settingShortcutPage->saveSettings();  
    _settingScrobblerPage->saveSettings();  
  
    //! emit signal for change update  
    if(_settingCollectionPage->isChanged())
      emit signal_rebuildDatabase();

    if(_settingCollectionPage->isAlbumViewChanged())
      emit signal_albumViewChanged();

    if(_settingRadioPage->isChanged())
      emit signal_rebuildRadio();  

    if(_settingScrobblerPage->isChanged())
      emit signal_scrobblerChanged();
    
    this->close();
}

//! --------- on_buttonBox_rejected --------------------------------------------
void SettingsDialog::on_buttonBox_rejected()
{
    this->close();
}
