/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _SETTINGS_PAGES_H_
#define _SETTINGS_PAGES_H_

#include "databasemanager.h"
#include "shortcutdialog.h"

#include <QtGui>


//! ----------- Global Settings Page  ------------------------------------------  
class SettingGlobalPage : public QWidget
{
  public:
    SettingGlobalPage(QWidget *parent = 0);
    void restoreSettings();
    void saveSettings();
    
  private:
    QCheckBox    *minimazeToSystray;
    QCheckBox    *enableDbusNotification;
    void createGui();
};

//! ----------- Collection Settings Page  --------------------------------------  
class SettingCollectionPage : public QWidget
{
Q_OBJECT
  public:
    SettingCollectionPage(QWidget *parent = 0);
    void restoreSettings();
    void saveSettings();
    bool isChanged() {return _isChanged;}
    bool isAlbumViewChanged() {return _isAlbumViewChange;}
  
  private:
    bool         _isChanged; //! we need to rebuild database
    bool         _isAlbumViewChange; //! only repopulate the scene
    
    QListWidget  *collectionsPathsList;
    QCheckBox    *enableMultiDb;
    QCheckBox    *autoupdateCollectionsCheckbox;
    QCheckBox    *searchCoverFromDir;
    QCheckBox    *useGridView;
    QComboBox    *_comboBoxDb;

    QPushButton  *addPathButton;
    QPushButton  *removePathButton;
    QPushButton  *removeAllPathsButton;

    QToolButton  *_dbNewButton;
    QToolButton  *_dbDelButton;
    QToolButton  *_dbRenameButton;

    QMap<QString, DB::S_dbParam> _dbParamList;

  private: 
    void createGui();
    void addDatabaseParam(const QString& name, const DB::S_dbParam& dbParam); 
  
  private slots:
    void loadDatabaseParam(QString);
    void newDatabaseParam();
    void delDatabaseParam();
    void updateDatabaseParam1();
    void updateDatabaseParam2();
    void updateDatabaseParam3();
    void renameDatabaseParam();    
    void enableMultiDatabase();
    void on_addPathButton_clicked();
    void on_removePathButton_clicked();
    void on_removeAllPathsButton_clicked();
};

//! ----------- Radio Settings Page  -------------------------------------------  
class SettingRadioPage : public QWidget
{
  Q_OBJECT
  public:
    SettingRadioPage(QWidget *parent = 0);
    void restoreSettings();
    void saveSettings();
    bool isChanged() {return _isChanged;}

  private:
    void createGui();
    
  private:
    bool         _isChanged;  
    QListWidget  *RadioFileList;
    QPushButton  *addFileButton;
    QPushButton  *removeFileButton;
    QPushButton  *removeAllFileButton;
    
  private slots:
    void on_addFileButton_clicked();
    void on_removeFileButton_clicked();
    void on_removeAllFilesButton_clicked();
};



//! ----------- Shurtcut Settings Page  --------------------------------------  
class SettingShortcutPage: public QWidget
{
  Q_OBJECT
  public:
    SettingShortcutPage(QWidget *parent = 0);
    void restoreSettings();
    void saveSettings();
    
  private slots:
    void changeShortcut(QModelIndex index);
    void shortcutChanged(QModelIndex index, QKeySequence shortcut);

  private:
    ShortcutDialog      *_shortcutDialog;
    QTreeWidget         *shortcutsList;
 };
 
 //! ----------- Scrobbler Settings Page  --------------------------------------  
class SettingScrobblerPage: public QWidget
{
  Q_OBJECT
  public:
    SettingScrobblerPage(QWidget *parent = 0);
    void restoreSettings();
    void saveSettings();
    bool isChanged() {return _isChanged;}

  private:
    QCheckBox    *useLastFmScrobbler;
    bool         _isChanged;
    bool         _isEnableOld;
    
private slots:
    void enableChange();
 };
    
#endif // _SETTINGS_PAGES_H_
