/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "shortcutdialog.h"

#include <QtGui>


ShortcutDialog::ShortcutDialog(QModelIndex index, QWidget *parent) : QDialog(parent)
{
    _index = index;

    //! Gui setup
    this->setWindowTitle(tr("Change shortcut dialog"));
    this->resize(416, 199);
    QVBoxLayout * verticalLayout = new QVBoxLayout(this);
    
    QLabel * label= new QLabel(this);
    label->setAlignment(Qt::AlignCenter);
    label->setWordWrap(false);
    label->setText(tr("Press a keys that you want to set as a shortcut"));

    currentShortcut = new QLabel(this);
    currentShortcut->setAlignment(Qt::AlignCenter);
    currentShortcut->setText(index.sibling(index.row(), 1).data().toString());
    
    QDialogButtonBox *buttonBox = new QDialogButtonBox(this);
    buttonBox->setFocusPolicy(Qt::NoFocus);
    buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

    verticalLayout->addWidget(label);
    verticalLayout->addWidget(currentShortcut);
    verticalLayout->addWidget(buttonBox);
    
    //! connection
    connect(buttonBox, SIGNAL(rejected()),this, SLOT(close()));
    connect(buttonBox, SIGNAL(accepted()),this, SLOT(accept()));

    //! List legal modifiers
    _modifiers << Qt::ControlModifier << Qt::AltModifier << Qt::MetaModifier << Qt::ShiftModifier;
}




void ShortcutDialog::keyReleaseEvent(QKeyEvent *e)
{
    if (_modifiers.contains(e->modifiers())) {
        _shortcut = QKeySequence(e->modifiers() + e->key());
        currentShortcut->setText(_shortcut.toString(QKeySequence::NativeText));
    }
}

void ShortcutDialog::accept()
{
    emit shortcutChanged(_index, _shortcut);
    close();
}

