/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "audiocontrols.h"

#include <QSizePolicy>
#include <QDebug>



namespace GLOBAL {
    QMap<QString, QAction*>* globalActions();
}


//! ------- class RepeatControl ------------------------------------------------
RepeatControl::RepeatControl(QWidget *parent)  : QToolButton( parent )
{
    this->setSizePolicy(QSizePolicy::Minimum,QSizePolicy::Minimum);
    this->setIconSize( QSize( 20, 20 ) );
    this->setIcon(QIcon(":/images/media-repeat-1-64x64.png")); 
    this->setToolTip(tr("Repeat mode is off")); 
    
    _state = RepeatOff;
   
    connect(this, SIGNAL(clicked()), this, SLOT(onButtonClicked()));
}

void RepeatControl::onButtonClicked()
{
    //qDebug() << "- RepeatControl -> onButtonClicked"; 
    switch(_state) {
      case RepeatOff : setState( (int) RepeatTrack ); break;
      case RepeatTrack : setState( (int) RepeatAll ); break;
      case RepeatAll : setState( (int) RepeatOff ); break;
      default:break;
    }

   emit repeatStateChange((int) _state);
}

void RepeatControl::setState(int intState)
{
    _state = (RepeatMode) intState;
    
    switch(_state) {
      case RepeatOff :
        this->setIcon(QIcon(":/images/media-repeat-1-64x64.png")); 
        this->setToolTip(tr("Repeat mode is off")); 
      break;
      case RepeatTrack :
         this->setIcon(QIcon(":/images/media-repeat-2-64x64.png")); 
         this->setToolTip(tr("Repeat track")); 
      break;
      case RepeatAll :
         this->setIcon(QIcon(":/images/media-repeat-3-64x64.png")); 
         this->setToolTip(tr("Repeat all"));
      break;
      default:break;
    }
    
}



//! ------- class ShuffleControl -----------------------------------------------
ShuffleControl::ShuffleControl(QWidget *parent)  : QToolButton( parent )
{
    this->setIconSize( QSize( 20, 20 ) );
    this->setIcon(QIcon(":/images/media-shuffle-1-64x64.png")); 
    this->setToolTip(tr("Shuffle mode is off"));
    
    _state = ShuffleOff;
   
    connect(this, SIGNAL(clicked()), this, SLOT(onButtonClicked()));    
}


void ShuffleControl::onButtonClicked()
{
    //qDebug() << "- RepeatControl -> onButtonClicked"; 
    switch(_state) {
      case ShuffleOff : setState( (int) ShuffleOn ); break;
      case ShuffleOn  : setState( (int) ShuffleOff ); break;
      default:break;
    }
   emit shuffleStateChange((int) _state);
}

void ShuffleControl::setState(int intState)
{
    _state = (ShuffleMode) intState;
    
    switch(_state) {
      case ShuffleOff :
         this->setIcon(QIcon(":/images/media-shuffle-1-64x64.png")); 
         this->setToolTip(tr("Shuffle mode is off"));
      break;
      case ShuffleOn :
         this->setIcon(QIcon(":/images/media-shuffle-2-64x64.png")); 
         this->setToolTip(tr("Shuffle mode is on")); 
      break;
      default:break;
    }    
    
}