/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// thanks for clementine equalizer for inspiration

#ifndef _EQUALIZER_H_
#define _EQUALIZER_H_


#include <QtGui>
#include <QDialog>
#include <QMetaType>
#include <QMap>
#include <QDataStream>

#include "equalizerslider.h"


//! ----------------- Equalizer -------------------------------------------
class Equalizer : public QDialog 
{
Q_OBJECT

 public:
    Equalizer(QWidget *parent = 0);
    ~Equalizer();

    
    static const int kBands = 10;
    static const char* kGainText[kBands];

    struct EqPreset {
      EqPreset() {
	preamp = 0;
	for (int i=0 ; i<kBands ; ++i)
	  gain[i] = 0;
      }

      EqPreset(int g0, int g1, int g2, int g3, int g4, int g5,int g6, int g7, int g8, int g9, int p) {
        preamp = p;
	gain[0] = g0;  gain[1] = g1; gain[2] = g2;  gain[3] = g3; gain[4] = g4;
        gain[5] = g5;  gain[6] = g6; gain[7] = g7;  gain[8] = g8; gain[9] = g9;
      }

      bool operator ==(const EqPreset& other) const;
      bool operator !=(const EqPreset& other) const;
    
      int        preamp;
      int        gain[kBands];
    };

    bool isEnabled() const;
  
  protected:
   void closeEvent(QCloseEvent *);
  
  private: //! Gui stuff
    QComboBox *_comboBoxPreset;
    QCheckBox *_enableCheckBox;
    QWidget   *_slider_container;
  
  private:
    EqualizerSlider* eqSliderList[kBands];
    EqualizerSlider* eqSliderPreamp;

    QMap<QString, EqPreset> _presetList;
    
    bool _settingChanged;
    
    void addDefaultPreset();
    void addPreset(const QString& name, const EqPreset& eqPreset);
    void saveSettings();
    void restoreSettings();
    
  private slots:
    void applyPreset(const QString& name);
    void equalizerChanged();
    void enableChanged(bool enable);
    void savePreset();
    void delPreset();

 signals:
    void enabledChanged(bool enabled);    
    void newEqualizerValue(const int preamp, const QList<int>& gainsList);
};

Q_DECLARE_METATYPE(Equalizer::EqPreset);


QDataStream &operator<<(QDataStream& s, const Equalizer::EqPreset& p);
QDataStream &operator>>(QDataStream& s, Equalizer::EqPreset& p);

#endif // _EQUALIZER_H_
