/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef _STATUSBAR_H_
#define _STATUSBAR_H_


#include <QStatusBar>
#include <QString>
#include <QStringList>
#include <QMap>
#include <QWidget>
#include <QLabel>


class InfoResolver;
class StatusMessage;



//! Message Type to display
typedef enum {TYPE_INFO = 0, TYPE_ERROR, TYPE_WARNING, TYPE_COLLECTION} T_MESSAGE;


/*******************************************************************************
    StatusBar
*******************************************************************************/
//! StatusBar is displays various informations about state of the application
//! - Collection info
//! - Playlist info
//! - Now playing inf 
//! - Special Appli Process and status

class StatusBar : public QStatusBar
{
Q_OBJECT
    static StatusBar         *INSTANCE;

  public:
    //! Constructor
    StatusBar(QWidget *parent = 0);
    
    //! return global instance
    static StatusBar* instance() { return INSTANCE; }
    
  public slots:
    //! Label
    void updateCollectionInfo(const QString& info);
    void updatePlaylistInfo(const QString& info);

    //! Message methode
    void startShortMessage(const QString& message, T_MESSAGE type, int ms );
    uint startProgressMessage(const QString& action);
    void stopProgressMessage(uint id);
    void updateProgressMessage(uint id, const QString & message);
      
  private slots:
    void stopShortMessage();
    void updateMessage();
    void updateNowPlayingInfo();

  private: //! GUI STUFF
    StatusMessage            *_currentStatusMessage;
    QLabel                   *_playlistLabel;

  private:
    InfoResolver             *_infoResolver;
    QMap<uint,QString>        _longMessage;
    uint                      messageId;
    QString                   _playingInfo;
    bool                      isShortMessage;
};


#endif // _STATUSBAR_H_
