/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

//! Qt 
#include <QtGui/QApplication>
#include <QTextCodec>
#include <QTranslator>
#include <QNetworkReply>
#include <QDir>
#include <QString>
#include <QTime>
#include <QNetworkAccessManager>

#include <QDBusMetaType>
#include <QDBusConnection>  //! 0.0.50
#include <QDBusArgument>    //! register QImage as qbusmetatype
#include <QImage>           //! register QImage as qbusmetatype


//! local
#include "mainwindow.h"
#include "Widget/equalizer.h" //type EqPreset
#include "constants.h"
#include "utilities.h"       // CONFIGDIR

//! third
#include <qtsingleapplication.h>
#include <src3party/libechonest/Config.h>

   
//! ----------------------------------------------------------------------------
QDBusArgument& operator<< (QDBusArgument& arg, const QImage& image);
const QDBusArgument& operator>> (const QDBusArgument& arg, QImage& image);
  
  
/*******************************************************************************
    main
*******************************************************************************/
int main(int argc, char *argv[])
{
    //! --- Application settings -----------------------------------------------
    QApplication::setGraphicsSystem("raster");
    
    //! --- Single application process -----------------------------------------
    // QtSingleApplication replace QApplication for one instance
    QtSingleApplication yarock(argc, argv);
    yarock.setApplicationName(APP_NAME);
    yarock.setOrganizationName(ORG_NAME);
    yarock.setApplicationVersion(VERSION);

    /*if (yarock.isRunning()) {
      if (yarock.sendMessage("Wake up")) 
           return 0;
      // Otherwise, no message sent, start anyway
    }*/
    
    //! --- Locale + Translations ----------------------------------------------
    QString locale = QLocale::system().name().left(2);
    qDebug() << "== main == locale : " << locale;
      
    QTranslator qtTranslator;
    qtTranslator.load("qt_" + locale,QLibraryInfo::location(QLibraryInfo::TranslationsPath));
    yarock.installTranslator(&qtTranslator);

    QTranslator YaRockTranslator;
    //! avec localDir = /usr/share/locale/YaRock/
    QString localeDir = QLatin1String(LOCALEDIR);
    //qDebug() << "main -> localeDir : " << localeDir;

    // If app was not "installed" use the app directory
    if (!QFile::exists(localeDir)) {
        localeDir = qApp->applicationDirPath() + "/../../translation";
    }

    YaRockTranslator.load(QString("yarock_") + locale,localeDir);
    yarock.installTranslator(&YaRockTranslator);
    
    QTextCodec::setCodecForTr(QTextCodec::codecForName("utf8"));
    QTextCodec::setCodecForCStrings(QTextCodec::codecForName("UTF-8"));
    
    //! --- INIT QRAND-----------------------------------------------------------
    QTime time = QTime::currentTime();
    qsrand((uint)time.msec());

    //! --- DATABASE -----------------------------------------------------------

    //! --- FONT ---------------------------------------------------------------
    QFontDatabase::addApplicationFont(":/font/segoeuil.ttf");
    QFontDatabase::addApplicationFont(":/font/seguibk.ttf");
    QFontDatabase::addApplicationFont(":/font/seguibd.ttf");
    //QFontDatabase::addApplicationFont(":/font/seguisb.ttf");
    
    //!TEST verif des font
    /*QFontDatabase *fontDb = new QFontDatabase();
    QStringList list = fontDb->styles( "Segoe UI");
    foreach (const QString &s, list ) {
      qDebug() << "main -> font style : " << s;
    }  */
    
    //! --- ECHONEST -----------------------------------------------------------
    Echonest::Config::instance()->setAPIKey("DFLFLJBUF4EGTXHIG");
    Echonest::Config::instance()->setNetworkAccessManager(new QNetworkAccessManager() );
     
    //! --- CONFIG setup -------------------------------------------------------
    QDir().mkpath(UTIL::CONFIGDIR);
    QDir().mkpath(UTIL::CONFIGDIR + "/radio");
    QDir().mkpath(UTIL::CONFIGDIR + "/download");
    QDir().mkpath(UTIL::CONFIGDIR + "/albums");
    QDir().mkpath(UTIL::CONFIGDIR + "/lyrics");

    //! --- Qt Type registration  ----------------------------------------------
    qRegisterMetaType<QNetworkReply::NetworkError>("QNetworkReply::NetworkError");
    qRegisterMetaTypeStreamOperators<Equalizer::EqPreset>("Equalizer::Params");

    //! --- DBUS ---------------------------------------------------------------
    QDBusConnection::sessionBus().registerService("com.sebastien.YaRock");
    qDBusRegisterMetaType<QImage>();

    //! --- MainWindow start ---------------------------------------------------
    MainWindow         mainwindow;
    mainwindow.show();

    return yarock.exec();
}
