/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _MAINWINDOW_H
#define _MAINWINDOW_H

#include "Player/player.h"
#include "Playlist/playlistview.h"
#include "Playlist/playlistmodel.h"
#include "Playlist/playlistwidget.h"
#include "NowPlaying/nowplayingview.h"

#include "GraphicViewWidget/graphicviewwidget.h"
#include "CollectionModel/collectionpopulator.h"
#include "CollectionModel/plsfilepopulator.h"
#include "CollectionView/collectionscene.h"
#include "CollectionView/virtualplaylist.h"

#include "CentralWidget/centraltoolbar.h"
#include "Widget/playertoolbar.h"
#include "Widget/statusbar.h"

#include "CentralWidget/centralwidget.h"
#include "CentralWidget/menuview.h"
#include "Web/coverthread.h"

#include "threadpool.h"

#include "RadioModel/streammodel.h"
#include "RadioView/radioscene.h"

#include "ContextView/contextscene.h"
#include "ScreenMode/miniwidget.h"    //! 0.0.43
#include "Database/databasemanager.h" //! 0.0.44
#include "Player/inforesolver.h"      //! 0.0.44

#include "Histo/histomanager.h"         //! 0.0.45
#include "CollectionModel/histomodel.h" //! 0.0.45

#include "scrobblermanager.h" //! 0.0.48

#include <QMainWindow>
#include <QtGui>
#include <QtCore>
   
   
class MainWindow : public QMainWindow 
{
    Q_OBJECT
public:
    MainWindow(QWidget *parent = 0);
    ~MainWindow();
    
private slots:
    //! ------------- MainWindows  ---------------------------------------------
    void createGui();
    void createActions();
    void createMenuBar();
    void createShortcuts();
    void connectSlots();
    void createSystray();
    //! ------------- File management ------------------------------------------
    void addFileToPlaylist();
    void addDirToPlaylist();
    void addUrlToPlaylist();
    //! ------------- Playlist management --------------------------------------
    void clearPlaylist();
    void savePlaylist();
    void autoSavePlaylist();
    void updatePlaylist(QList<QUrl> listUrl);
    //! ------------- Player management ----------------------------------------
    void playOrPause();
    void stopPlayer();
    void enqueueNextTrack();
    void playNext();
    void playPrev();
    void playerStateChanged(Phonon::State newState, Phonon::State oldState);
    void playTrackFromPlaylist();
    void playTrackFromCollection();  
    void setPlayerTrack(QString & filename); //! 0.0.50f TO TEST
    void updateNowPlaying ();                //! 0.0.50f TO TEST
    void phononLoadingError();              //! 0.0.50f TO TEST   
    
    //! ------------- Settings Method ------------------------------------------
    void openCollectionSettingsDlg();
    void openAllSettingsDlg();
    void openRadioSettingsDlg();
    //! ------------- Database Method ------------------------------------------
    void firstStartDialog();
    void rebuildDatabase();
    void forceRebuildDatabaseDlg();
    void forceRebuildDatabase();
    void repopulateAll();
    void populateCollectionModel();
    void populatePlsFileModel();
    void updateCollectionScene();
    void updatePlsFileScene();
    void rebuildDatabaseProgress(int progress);
    //! ------------- Cover Search Thread --------------------------------------
    void startCoverSearchThread();
    void stopCoverSearchThread();
    //! ------------- Others Actions -------------------------------------------
    void on_actionAbout_YaRock_triggered();
    void on_actionAbout_Qt_triggered();
    void on_actionQuit_Yarock_triggered();
    //! ------------- Systray --------------------------------------------------
    void trayClicked(QSystemTrayIcon::ActivationReason);
    //! ------------- Screen Mode ----------------------------------------------
    void switchMinimalMode();
    void switchNormalMode();
    //! ------------- Equalizer ------------------------------------------------
    void openAudioEqualizer();
    void applyEqualizerEnableChange(bool eqActivated);
    void applyEqualizerParam(int, QList<int>);
    //! ------------- MenuBar --------------------------------------------------
    void showMenuBar();
    //! ------------- Repeat & Shuffle -----------------------------------------
    void handleRepeatMode(int mode);
    void handleShuffleMode(int mode);
    
protected:
    void closeEvent(QCloseEvent *event);
      	
private:
    enum E_AppState {AppClosing, AppRunning};
    enum E_PlayingFrom {FromCollection, FromPlayQueue};
    
    E_AppState            _applicationState;
    E_PlayingFrom         _playRequestFrom;
        
    Player                *_player;
    InfoResolver          *_infoResolver;
    PlayerToolBar         *_playerToolBar;
    StatusBar             *_statusBar;

    CentralToolBar        *_centralToolBar;
    CentralWidget         *_centralWidget;
    MenuView              *_menuView;
    MenuModel             *_menuModel;

    PlaylistView          *_playlistView;
    PlaylistModel         *_playlistModel;
    PlaylistWidget        *_playlistwidget;
    NowPlayingView        *_nowplayingview;
    FileViewer            *_fileViewerWidget;
  
    CollectionModel       *_collectionModel;
    PlsFileModel          *_plsFileModel;
    HistoModel            *_histoModel;
    CollectionScene       *_collectionScene;
    CollectionPopulator   *_collectionPopulator;
    PlsFilePopulator      *_plsFilePopulator;
    GraphicViewWidget     *_graphicViewWidget;

    VirtualPlaylist       *_virtualPlaylist;
      
    StreamModel           *_streamModel;
    RadioScene            *_radioScene;

    CoverThread           *_coverThread;
    ThreadPool            *_threadPool;
    
    QMap<QString, QAction*>  *actions;
    DatabaseManager          *_dbManager;
  
    QSettings             *_settings;
    QIcon                 *_appIcon;
    QSystemTrayIcon       *_trayIcon;
    bool                  _canClose;
    bool                  _systrayActivated;
    
    MinimalWidget         *_minimalwidget;    
    HistoManager          *_histoManager;
    
    ScrobblerManager      *_scrobblerManager;
    
    QMap<QString, uint>    messageIds;
    
    //! 0.0.50 try better management for broken stream
    QString                playingUrl;
    QString                loadingUrl;    
    QTimer                 *loadingTimeOut;
};

#endif // _MAINWINDOW_H
